-------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:36:02 05/20/2009 
-- Design Name: 
-- Module Name:    controller_phase2b - arch 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
--
-- 26th October 2010

-- USB_StatusRegister(7) changed from SA_EnableActuation to
-- USB_StatusRegister(7)				<= SA_ActuateEnabled; 
-- This now indicates the state of the system instead of reading the request

-- 26th October 2010
--
-- Work on the addition of the BPS signal has begun

-- 04th February 2011
--
-- Project Phase2
-- The project has been completely rebuilt from scratch in ISE for the phase 2 work
-- This will now be the baseline for phase 2

-- BPS Error Reset moved to bit 8 of usb_reset_register
-- Bit 7 is left out for future implementation of reset PC_Con_Error

-- 10th February 2010
-- Connected Additional USB BPS signals right through to the top module.
-- It should now be possible to connect to these registers using c++ module

-- 16th Feb 11
-- BPS code checked with software. We can read and write to the BPS errors ok
-- Added Ed's code to c++. Seems to generate BPS variables ok.

-- 17th Feb 11
-- Added PC_Counter checker. Generates PC error when times out

-- 22nd - 25th Feb 11
-- Added all the BPS registers to the Actuate Data.
-- Optical Amp USB registers added
-- Position Change signal added and tested (required for SPI DAC)
-- SPI DAC added
-- All other SPI interfaces added (DAC, ADC and EPOT)
-- All of this passes preliminary testing.

-- 28th Feb to 4th March 11
-- Added a get-out to the Capture state (1 min??)
-- Changed the actuation measurements to get a more accurate acceleration
-- Addde another input to the interlock reset so that the interlock error can only be reset 
-- if the interlocks are actually ok!
-- Remove Configuration Block

-- 28th April 2011
-- Matched IO to hardware, this meant adding/removing some exisiting IO

-- Early May
-- Added Engineering Functions

-- June 1st
-- Added a PWM_Boost function to the capture process.
-- This increases the PWM duty cycle by a set amount during the first part 
-- of the capture. Not sure whether to implement as a USB register or a constant
-- This is currently set as a constant.

-- 21st October 2011
-- C_I_interlock_3_24v is to become PPS interlock.
-- This should only throw an interlock error if the frame in NOT up and the interlock 
-- is not present.

-- Version 2p0 06/01/2012
-- Added the Quadrature ADC checks
-- If any quadrature ADC value falls below 50,000 then the controller throws an error

-- Version 2p1 06/01/2012
-- Added the Quadrature Continuity checks
-- If the quadrature signal does not change at least every 700us during actuation
-- then an error is thrown 

-- Version 2p2 12/01/2012
-- Added an Optics Mode so that the quadrature outputs can be seen on the 
-- Actuating/Actuating Trigger BNC's
-- CHA = Act2
-- CHB = Act3
-- INDX = ActTrig 2

-- March 2012
-- Phase 2b

-- Version 0p1/0p2
-- Removed superfluous firmware - All Engineering Tests
-- Plan is to add new Move to Hold States that utilise 3 IGBT's being on simulataneously
-- Add a BPS Enabled Flag

-- Quad Continuity Error
-- DISABLED THIS NEEDS RE-ENABLING!!!!!

-- 19/03/2012

-- Version 0p3
-- Both ROMS incorporated

-- 21/03/2012
-- Version 0p4
-- Engineering functionality restored
-- Added Velocity Function and New Actuation Data Register to hold value.

-- Version 0p5
-- 20/05/2012
-- Bug Fix - Ensure that no BPS error can be reset until the target frame has been raised. 

-- Version 0p6
-- 23/05/2012
-- Added new BPS state that drops a BPS error if frame state is incorrect
-- 1) Frame Not Raised when parked
-- 2) Frame Raised and Frame lowered togther (ill defined state)

-- Removed BPS error relay reset when frame raised (request of BPS)

----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;
use work.sercon_types.all;

entity controller_phase2b is
	
    Port (  
			int_sysclk 						: in  std_logic;
         int_logic_reset				: in  std_logic;
				
			C_I_ch_a							: in std_logic;
			C_I_ch_b							: in std_logic;
			C_I_index						: in std_logic;
			
			C_I_key_en						: in std_logic; -- func 1 
			C_I_isis_ms						: in std_logic; -- func 2
			C_I_mice_daq_ok				: in std_logic; -- func 3
			C_I_tgt_frame_raised			: in std_logic; -- C_I_interlock_1_24v
			C_I_tgt_frame_lowered 		: in std_logic; -- C_I_interlock_2_24v
			
			C_I_Function_1_5V				: in std_logic;	
			
			C_I_interlock_1_5v			: in std_logic;
			C_I_interlock_2_5v			: in std_logic; 
			C_I_interlock_3_5v			: in std_logic; 
			C_I_interlock_4_5v			: in std_logic;
			
			--C_I_interlock_1_24v			: in std_logic; 
			--C_I_interlock_2_24v			: in std_logic;
			
			C_I_interlock_3_24v			: in std_logic; -- PPS Interlock
			C_I_interlock_4_24v			: in std_logic;
			
			C_I_USB_nc						: in std_logic;
			
			-- Added 28/4/11
			C_O_ChA_LED						: out std_logic;
			C_O_ChB_LED						: out std_logic;
			C_O_Indx_LED					: out std_logic;
			-- Added 28/4/11
						
			C_O_mosfet						: out std_logic_vector(6 downto 1);
			C_O_mosfet_led					: out std_logic_vector(6 downto 1);
			C_O_target_pos					: out std_logic_vector(8 downto 0);			
						
			C_O_tgt_ready_50r				: out std_logic;
			
			C_O_parked_led					: out std_logic;
			C_O_hold_led					: out std_logic;
			C_O_sys_en_led					: out std_logic;
			C_O_actuate_en_led			: out std_logic;
			C_O_tgt_ready_led				: out std_logic;
			C_O_actuating_led				: out std_logic;
			C_O_frame_raised_led			: out std_logic;
			C_O_frame_lowered_led 		: out std_logic;		
			C_O_error_led					: out std_logic;
			
			C_O_actuating_50r_1			: out std_logic;
			C_O_actuating_50r_2			: out std_logic;
			C_O_actuating_50r_3			: out std_logic;
			
			C_O_actuating_trig_50r_1	: out std_logic;
			C_O_actuating_trig_50r_2	: out std_logic;
			
			C_O_trig_accept_50r			: out std_logic;
			
			C_O_Frame_Enable_50r			: out std_logic;
			C_O_Legal_Spill_50r			: out std_logic;
			--C_O_unused3						: out std_logic;
			
			C_O_DAQ_tx_dout				: out std_logic;
			C_O_DAQ_tx_sclk				: out std_logic;
			C_O_DAQ_tx_synch				: out std_logic;
			
			C_O_ChA_Epot_tx_dout			: out std_logic;
			C_O_ChA_Epot_tx_sclk			: out std_logic;
			C_O_ChA_Epot_tx_synch		: out std_logic;
			
			C_O_ChB_Epot_tx_dout			: out std_logic;		
			C_O_ChB_Epot_tx_sclk			: out std_logic;
			C_O_ChB_Epot_tx_synch		: out std_logic;
			
			C_O_Indx_Epot_tx_dout		: out std_logic;
			C_O_Indx_Epot_tx_sclk		: out std_logic;
			C_O_Indx_Epot_tx_synch		: out std_logic;
			
			C_I_ChA_ADC_rx_din			: in std_logic;
			C_O_ChA_ADC_rx_sclk			: out std_logic;
			C_O_ChA_ADC_rx_synch			: out std_logic;
			
			C_I_ChB_ADC_rx_din			: in std_logic;		
			C_O_ChB_ADC_rx_sclk			: out std_logic;
			C_O_ChB_ADC_rx_synch			: out std_logic;
			
			C_I_Indx_ADC_rx_din			: in std_logic;
			C_O_Indx_ADC_rx_sclk			: out std_logic;
			C_O_Indx_ADC_rx_synch		: out std_logic;
			
			-- Added 28/4/11
			C_O_Spare_2_50R				: out std_logic;
			C_O_Spare_3_50R				: out std_logic;
			
			C_O_BPS_50R						: out std_logic;
			C_O_BPS_RELAY_1				: out std_logic;
			C_O_BPS_RELAY_2				: out std_logic;
			C_O_BPS_RELAY_3				: out std_logic;
			
			C_O_LASER_RELAY_1				: out std_logic;
			C_O_LASER_RELAY_2				: out std_logic;
			C_O_LASER_RELAY_3				: out std_logic;
			
			C_O_FRAME_EN_RELAY			: out std_logic;
			C_O_SPARE_RELAY				: out std_logic;
			-- 
			
			--USB Register Signals
			USB_ControlReg					: in std_logic_vector(31 downto 0);
			USB_SP1Wide						: in std_logic_vector(31 downto 0);
			USB_MTH_Timer					: in std_logic_vector(31 downto 0);
			USB_HoldPosMax					: in std_logic_vector(31 downto 0);
			USB_PWMSetup					: in std_logic_vector(31 downto 0);
			USB_WDTValue					: in std_logic_vector(31 downto 0);
			USB_ISISDelay1					: in std_logic_vector(31 downto 0);
			USB_ISISDelay2					: in std_logic_vector(31 downto 0);
			USB_ErrorCorrection			: in std_logic_vector(31 downto 0); -- Includes ECCountMax ECThreshold and DefaultOffset
			USB_CapSettleTime				: in std_logic_vector(31 downto 0);
			USB_CapWaitTime				: in std_logic_vector(31 downto 0);
			USB_QCSetup						: in std_logic_vector(31 downto 0); -- Includes QC InitialPos and QCIndexOffset
			USB_DeadTime					: in std_logic_vector(31 downto 0);
			USB_KickBackTime				: in std_logic_vector(31 downto 0);
			USB_DropLimit					: in std_logic_vector(31 downto 0);
			USB_LegalSpillDelay			: in std_logic_vector(31 downto 0);
			USB_ResetRegister				: in std_logic_vector(31 downto 0);
			
			
			USB_StatusRegister			: out std_logic_vector(31 downto 0);
			USB_ActuationMinMax			: out std_logic_vector(31 downto 0);
			USB_ActuationCount			: out std_logic_vector(31 downto 0);
			USB_ActuationError			: out std_logic_vector(31 downto 0);
			USB_ActuationSP				: out std_logic_vector(31 downto 0);
			
			USB_ActuationSP1Time			: out std_logic_vector(31 downto 0);
			USB_ActuationMinTime			: out std_logic_vector(31 downto 0);
			USB_ActuationTime				: out std_logic_vector(31 downto 0);
			
			USB_NotifyRegister			: out std_logic_vector(31 downto 0);

			-- New Registers October 2010
			-- BPS
			USB_Sp1MaxTime					: in std_logic_vector(31 downto 0);
			USB_Sp1MinTime					: in std_logic_vector(31 downto 0);
			USB_MinPosMaxTime				: in std_logic_vector(31 downto 0);
			USB_MinPosMinTime				: in std_logic_vector(31 downto 0);
			USB_MinPosMaxValue			: in std_logic_vector(31 downto 0);
			USB_MinPosMinValue			: in std_logic_vector(31 downto 0);
			USB_ActMaxTime					: in std_logic_vector(31 downto 0);
			USB_ActMinTime					: in std_logic_vector(31 downto 0);
			
			-- General Register for Controller
			USB_BPSErrors					: out std_logic_vector(31 downto 0);
			
			-- Registers as part of actuation data
			USB_BPSErrorData				: out std_logic_vector(31 downto 0);
			
			-- New Register February 2011
			-- Counter Should be updated periodically to prevent target shutdown.
			USB_PCCounter					: in std_logic_vector(31 downto 0);
				
			-- BPS registers that are part of the actuation fifo
			-- That way the correct BPS values are aligned with the
			-- appropriate actuations
			USB_ActuationSp1MaxTime					: out std_logic_vector(31 downto 0);
			USB_ActuationSp1MinTime					: out std_logic_vector(31 downto 0);
			USB_ActuationMinPosMaxTime				: out std_logic_vector(31 downto 0);
			USB_ActuationMinPosMinTime				: out std_logic_vector(31 downto 0);
			USB_ActuationMinPosMaxValue			: out std_logic_vector(31 downto 0);
			USB_ActuationMinPosMinValue			: out std_logic_vector(31 downto 0);
			USB_ActuationActMaxTime					: out std_logic_vector(31 downto 0);
			USB_ActuationActMinTime					: out std_logic_vector(31 downto 0);
			
			-- Additional registers for setting gains via SPI
			USB_ChannelGains 		: in std_logic_vector(31 downto 0);
			--USB_ChBGain 			: in std_logic_vector(31 downto 0);
			--USB_IndxGain 			: in std_logic_vector(31 downto 0);
			
			-- Registers that report the value seen by the ADC
			USB_ActuationChAOutput 			: out std_logic_vector(31 downto 0);
			USB_ActuationChBOutput			: out std_logic_vector(31 downto 0);
			USB_ActuationIndxOutput			: out std_logic_vector(31 downto 0);

			-- Engineering Register
			USB_Engineering					: in std_logic_vector(31 downto 0);
			
			-- Capture Boost (increases PWM DC on capture)
			USB_CaptureBoost					: in std_logic_vector(31 downto 0);
			
			-- Output Velocity (Part of Actuation Data);
			USB_Velocity						: out std_logic_vector(31 downto 0)
			);
			
end controller_phase2b;

architecture arch of controller_phase2b is
	
	-- General Signals
	type T_state_type is 
	
						(
						State_Initialise,
						State_Initialise2,
						State_Parked,
						State_Moving_to_Hold_1,
						State_Moving_to_Hold_2,
						State_Moving_to_Hold_3,
						State_HoldMode,
						State_ActuateEnableMode,
						State_Moving_to_Park_1,
						State_Moving_to_Park_2,
						State_Actuate,
						State_Actuate2,
						State_Actuate3,
						State_Capture1,
						State_Capture2,
						State_CaptureAdjustment1,
						State_CaptureAdjustment2,
						State_MakeSafe1,
						State_MakeSafe2,
						State_MakeSafe3,
						State_Kill,
						State_Kill_2,
						State_Engineering,
						State_BPSError
						);
								
	signal T_state_now, T_state_next: T_state_type;
	
	
	type 	T_BPS_state_type is 
					(
						State_BPS_start,
						State_BPS1,
						State_BPS2,
						State_BPS3,
						State_BPS4,
						State_BPS5,
						State_BPS6,
						State_BPS7
					);
	signal T_BPS_state_now, T_BPS_state_next: T_BPS_state_type;
	
	type 	T_LASER_state_type is 
					(
						State_LASER_start,
						State_LASER1,
						State_LASER2,
						State_LASER3,
						State_LASER4,
						State_LASER5,
						State_LASER6,
						State_LASER7
					);
					
	signal T_LASER_state_now, T_LASER_state_next: T_LASER_state_type;
	
	type 	T_NUDGE_state_type is 
					(
						State_NUDGE_start,
						State_NUDGE1,
						State_NUDGE2,
						State_NUDGE3,
						State_NUDGE4,
						State_NUDGE5,
						State_NUDGE6,
						State_NUDGE7,
						State_NUDGE8,
						State_NUDGE9,
						State_NUDGE10,
						State_NUDGE11
						--State_NUDGE12,
						--State_NUDGE13,
						--State_NUDGE14,
						--State_NUDGE15,
						--State_NUDGE16,
						--State_NUDGE17,
						--State_NUDGE18
					);
					
	signal T_NUDGE_state_now, T_NUDGE_state_next: T_NUDGE_state_type;
				
	--State Initialise
	--General Signals
	signal SA_CurrentState				: std_logic_vector(3 downto 0);
	signal SA_move_to_hold_in 			: std_logic;
	signal SA_move_to_park_in 			: std_logic;
	signal SA_MosfetBin 					: std_logic_vector(6 downto 1);
	signal SA_MosfetLED					: std_logic_vector(6 downto 1);

	signal SA_hold_select				: std_logic;
	signal SA_actuate_select			: std_logic;
	signal SA_Hold							: std_logic;
	signal SA_Parked						: std_logic;
	signal SA_DropDetectEnable			: std_logic;
	signal SA_ActuateEnabled			: std_logic;
	signal SA_ActuateEnabledLED		: std_logic;
	signal SA_ActuateEnableError		: std_logic;
	signal SA_TargetReady				: std_logic;
	signal SA_TargetFrameEnable		: std_logic;
	signal SA_TargetFrameEnableMASTER : std_logic;
	signal SA_EnableActuation			: std_logic;
	signal SA_TargetFrameStateGood	: std_logic;
	
	signal SA_FatalError					: std_logic;
	signal SA_FatalErrorReset			: std_logic;
	
	signal sl_actuating_50r_2			: std_logic;
	signal sl_actuating_50r_3			: std_logic;
	signal sl_actuating_trig_50r_2	: std_logic;
	
	signal SA_SetIndexError				: std_logic;
	signal SA_ResetIndexError			: std_logic;
	signal SA_IndexErrorState			: std_logic;
	signal SA_NextIndexErrorState		: std_logic;
	
	signal SA_SetInitOKNotify			: std_logic;	
	signal SA_SetParkNotify				: std_logic;
	signal SA_SetFIFONotify				: std_logic;
	
	signal SA_SetWDTError				: std_logic;
	signal SA_ResetWDTError				: std_logic;
	signal SA_WDTErrorState				: std_logic;
	signal SA_NextWDTErrorState		: std_logic;
	
	signal SA_SetDDError					: std_logic;
	signal SA_ResetDDError				: std_logic;
	signal SA_DDErrorState				: std_logic;
	signal SA_NextDDErrorState			: std_logic;
	
	signal SA_SetUSBncError				: std_logic;
	signal SA_ResetUSBncError			: std_logic;
	signal SA_USBncErrorState			: std_logic;
	signal SA_NextUSBncErrorState		: std_logic;
	
	signal SA_SetKEError				: std_logic;
	signal SA_ResetKEError			: std_logic;
	signal SA_KEErrorState			: std_logic;
	signal SA_NextKEErrorState		: std_logic;

	--signal SA_SetQuadError			: std_logic;
	--signal SA_ResetQuadError		: std_logic;
	
	signal SA_Set5VIntError			: std_logic;
	signal SA_Reset5VIntError		: std_logic;
	signal SA_5VIntErrorState		: std_logic;
	signal SA_Next5VIntErrorState	: std_logic;
	
	signal SA_Set24VIntError		: std_logic;
	signal SA_Reset24VIntError		: std_logic;
	signal SA_24VIntErrorState		: std_logic;
	signal SA_Next24VIntErrorState: std_logic;
	
	signal SA_PPS_OK					: std_logic;
	
	signal SA_SetBPSError			: std_logic;
	signal SA_ResetBPSError			: std_logic;
	signal SA_BPSErrorState			: std_logic;
	signal SA_NextBPSErrorState	: std_logic;
	signal SA_BPSErrorStateEnable			: std_logic;
	signal SA_NextBPSErrorStateEnable	: std_logic;
	
	signal SA_SetPCError				: std_logic;
	signal SA_ResetPCError			: std_logic;
	signal SA_PCErrorState			: std_logic;
	signal SA_NextPCErrorState		: std_logic;
	
	signal SA_SetCapError			: std_logic;
	signal SA_ResetCapError			: std_logic;
	signal SA_CapErrorState			: std_logic;
	signal SA_NextCapErrorState	: std_logic;
	
	signal SA_SetQuadADCError				: std_logic;
	signal SA_ResetQuadADCError			: std_logic;
	signal SA_QuadADCErrorState			: std_logic;
	signal SA_NextQuadADCErrorState		: std_logic;
	
	signal SA_SetQuadADCErrorCHA			: std_logic;
	signal SA_SetQuadADCErrorCHB			: std_logic;
	signal SA_SetQuadADCErrorINDEX		: std_logic;
		
	--signal SA_QuadContinuityError					: std_logic;
	
	signal SA_SetQuadContinuityErrorA			: std_logic;
	signal SA_SetQuadContinuityErrorB			: std_logic;
	
	signal SA_SetQuadContinuityError				: std_logic;
	signal SA_ResetQuadContinuityError			: std_logic;
	signal SA_QuadContinuityErrorState			: std_logic;
	signal SA_NextQuadContinuityErrorState		: std_logic;
	
	signal SA_SetFrameNOTRaisedError1			: std_logic;
	signal SA_SetFrameNOTRaisedError2			: std_logic;
	signal SA_SetFrameNOTRaisedError				: std_logic;
	signal SA_ResetFrameNOTRaisedError			: std_logic;
	signal SA_FrameNOTRaisedErrorState			: std_logic;
	signal SA_NextFrameNOTRaisedErrorState		: std_logic;
			
	signal SA_SP1MINIMUM					: std_logic_vector(8 downto 0);
	
	signal SA_usb_nc_DG					: std_logic;
	signal SA_USBnc_Tick					: std_logic;
	signal SA_USBFatalResets			: std_logic_vector(12 downto 0);
	signal SA_ErrorCode					: std_logic_vector(3 downto 0);
	
	signal SA_USBAckActData				: std_logic;
	signal SA_ActDataAvailable			: std_logic;
	
	signal SA_ActuationMinMax			: std_logic_vector(31 downto 0);
	
	signal SA_USBAckNotify				: std_logic;
	signal SA_NotifyAvailable			: std_logic;
	
	-- Signals for connecting to Actuate/Hold_State_To_Mos_State
	signal SA_HoldFinalState			:std_logic_vector (3 downto 0);
	signal SA_Actuate_State				:std_logic_vector (3 downto 0);
	signal SA_Hold_bin					:std_logic_vector (6 downto 1);
	signal SA_Actuate_bin				:std_logic_vector (6 downto 1);
	
	-- Move to Hold
	signal SA_MTH_timer_value			: std_logic_vector(27 downto 0);
	signal SA_hold_position_max 		: std_logic_vector (5 downto 0);
	signal SA_move_to_hold_trigger 	: std_logic;
	signal SA_move_to_hold_complete 	: std_logic;
	signal SA_move_to_park_trigger 	: std_logic;
	signal SA_move_to_park_complete 	: std_logic;
	signal SA_HoldState1				   : std_logic_vector(5 downto 0);
	signal MOVE_TO_HOLD_NO_CONNECTION			: std_logic_vector(27 downto 0);
	
	-- Quadrature Interface
	signal SA_quad_direction			: std_logic;
	signal SA_quad_ErrorFlag			: std_logic;
	signal SA_quad_position				: std_logic_vector(8 downto 0);
	signal SA_quad_position_prev		: std_logic_vector(8 downto 0);
	signal SA_position_change			: std_logic;
	signal SA_quad_error					: std_logic_vector(2 downto 0);
	signal SA_QuadErrorValue			: std_logic_vector(8 downto 0);
	signal SA_QCPositionStart			: std_logic_vector(8 downto 0);
	signal SA_QCIndexOffset				: std_logic_vector(8 downto 0);
	signal SA_ResetQC						: std_logic;
	signal SA_IndexDirection			: std_logic;
	signal SA_IndexEnable				: std_logic;
	
	signal SA_DeGlitchedCHA				: std_logic;
	signal SA_DeGlitchedCHB				: std_logic;
	
	-- Actuate Module
	signal SA_Actuate_trigger			: std_logic;
	signal SA_Actuate_complete			: std_logic;
	signal SA_Actuating					: std_logic;
	signal SA_HoldStateQuad				: std_logic_vector(5 downto 0);
	signal SA_SetPoint1					: std_logic_vector(8 downto 0);
	signal SA_SP1LockedState			: std_logic_vector(8 downto 0);
	signal SA_NextSP1LockedState		: std_logic_vector(8 downto 0);
	signal SA_killActuation				: std_logic;
	
	signal SA_DeadTime					: std_logic_vector(11 downto 0);
	signal SA_KickBackTime				: std_logic_vector(17 downto 0);
	signal SA_ActMinPos					: std_logic_vector(8 downto 0);
	
	signal SA_resetActCount				: std_logic;
	signal SA_ActCount					: std_logic_vector(23 downto 0);
	signal SA_StartPos					: std_logic_vector(8 downto 0);
	
	signal SA_SP1Time						: std_logic_vector(31 downto 0);
	signal SA_MinTime						: std_logic_vector(31 downto 0);
	signal SA_ActTime						: std_logic_vector(31 downto 0);
	signal SA_Actuation_Timer			: std_logic_vector(31 downto 0);
	
	signal SA_QuadratureCheck			: std_logic;
	signal SA_SelectQCTime				: std_logic;
	signal SA_ActQuadrant				: std_logic_vector(2 downto 0);
	
	signal sl_ROMSelect					: std_logic;					
	
	-- Capture Module
	signal SA_HoldState2					: std_logic_vector(5 downto 0);
	signal SA_StartCapture				: std_logic;
	signal SA_CapComplete				: std_logic;
	signal SA_CapCount					: std_logic;
	signal SA_CapOverShoot				: std_logic;
	signal SA_CapUnderShoot				: std_logic;
	signal SA_CapActive					: std_logic;
	signal SA_CapSettleTime				: std_logic_vector(31 downto 0);
	signal SA_CapWaitTime				: std_logic_vector(31 downto 0);
	signal SA_KillCapture				: std_logic;
	signal SA_CapTrigger					: std_logic;
	signal SA_AquireActData				: std_logic;
	signal SA_AquireActData1			: std_logic;
	signal SA_AquireActData2			: std_logic;
	signal SA_UnderOverShootTrig		: std_logic;
	
	signal SA_CaptureStartProt			: std_logic;
	signal SA_CaptureStopProt			: std_logic;
	signal SA_CaptureOverflowProt		: std_logic;
	
	signal SA_Cap10sStart				: std_logic;	
	signal SA_Cap500usStart				: std_logic;
	
	signal SA_Cap10sComplete			: std_logic;
	signal SA_Cap500usComplete			: std_logic;

	
	-- Hold Switch
	signal SA_HoldSwitchOut				:std_logic_vector(5 downto 0);
	signal SA_HoldSwitch1				:std_logic;
	signal SA_HoldSwitch2				:std_logic;
	
	-- Mosfet Switch
	
	-- PWM module
	signal SA_PWMDutyCycle				: std_logic_vector(7 downto 0);
	signal SA_PWMTimer					: std_logic_vector(7 downto 0);
	signal SA_PWMCycleCountMax			: std_logic_vector(7 downto 0);
	signal SA_PWMSynchronise			: std_logic;
	signal SA_PWMEnable					: std_logic;
	signal SA_PWMFullOn					: std_logic;
	signal SA_PWMOut						: std_logic;
	
	-- PWM Boost Function
	signal SA_PWMBoostEnable			: std_logic;
	signal SA_boost_value				: unsigned(7 downto 0);
	--constant BOOST							: unsigned(7 downto 0) := to_unsigned(2,8);
	
	-- WDT module (This is badly named as it doesn't really perform this function)
	signal SA_WDT_MasterReset		: std_logic;
	signal SA_WDT_Reset				: std_logic;
	signal SA_WDT_Reset2				: std_logic;
	signal SA_WDT_Trigger			: std_logic;
	signal SA_WDT_Out					: std_logic;
	signal SA_WDT_TimerValue		: std_logic_vector(23 downto 0);
	
	-- Error Correction Module (Alters the SwitchPoint Depending upon Under of OverCaptures
	signal SA_SetPoint2				:std_logic_vector(8 downto 0);
	signal SA_ECCountMax				:std_logic_vector(7 downto 0);
	signal SA_ECThreshold			:std_logic_vector(7 downto 0);
	signal SA_ECDefaultOffset		:std_logic_vector(8 downto 0);
	
	-- IndexCheck Module
	-- This is to ensure that we see an index pulse before the first actuation
	signal SA_ResetIndexLatch		:std_logic;
	signal SA_DeGlitchedIndex		:std_logic;
	signal SA_IndexCheck				:std_logic;
	
	-- DropDetect Module
	signal 	SA_DropDetect 			: std_logic;
	signal 	SA_DropLimit			: std_logic_vector(8 downto 0);
	
	-- ISIS Delay Module
	signal 	SA_ISISDelay1 					: std_logic_vector(29 downto 0);
	signal 	SA_ISISDelay2 					: std_logic_vector(19 downto 0);
	signal   SA_LegalSpillDelay			: std_logic_vector(19 downto 0);
	--signal   SA_DAQTriggerDelay			: std_logic_vector(19 downto 0);
	signal 	SA_GMS							: std_logic;
	signal 	SA_DelayedGMS 					: std_logic;
	signal 	SA_TriggerEnable				: std_logic;
	signal 	SA_TriggerAccept50r			: std_logic;
	signal 	SA_triggerAccept50r_NoGate	: std_logic;
	signal 	SA_LegalSpill50r				: std_logic;
	
	-- Key Enable Module
	signal 	SA_KeyEnabled			: std_logic;
	
	-- FixedWidthPulse Module
	signal 	SA_ActuatingTrig		: std_logic;
	
	-- Interlock Module
	signal	SA_Interlocks5V_ok	: std_logic;
	signal 	SA_Interlocks24V_ok	: std_logic;
	
	--Notifications
	
	signal SA_NotificationCode		: std_logic_vector(7 downto 0); 
	
	-- Error Block
	
	--signal	EXT_ResetIndexError	: std_logic; -- Probably not needed?? 3/5/11
	signal 	SA_USBBPSResets		: std_logic_vector(7 downto 0);
	signal 	SA_BPSErrorTickNC		: std_logic;  -- Probably not needed?? 3/5/11
	
	-- BPS Signal
	signal 	SA_CheckLimits			: std_logic;
	signal 	SA_BPSErrors			: std_logic_vector(31 downto 0);
	signal 	sl_BPSDisabled			: std_logic;
	
	-- SPI DAC
	constant DAC_WIDTH			: integer := 24;
	
	signal 	DAC_FIFODataIn			: std_logic_vector(DAC_WIDTH-1 downto 0);
	signal 	DAC_fifofull			: std_logic;
	
	-- SPI EPOT
	constant EPOT_WIDTH		: integer := 8;
	
	signal 	ChA_EPOT_value_now			: std_logic_vector(EPOT_WIDTH - 1 downto 0);
	signal 	ChA_EPOT_value_prev			: std_logic_vector(EPOT_WIDTH - 1 downto 0);
	signal 	SA_EPOT_ChA_FifoWrite		: std_logic;
	
	signal 	ChB_EPOT_value_now			: std_logic_vector(EPOT_WIDTH - 1 downto 0);
	signal 	ChB_EPOT_value_prev			: std_logic_vector(EPOT_WIDTH - 1 downto 0);
	signal 	SA_EPOT_ChB_FifoWrite		: std_logic;
	
	signal 	Indx_EPOT_value_now			: std_logic_vector(EPOT_WIDTH - 1 downto 0);
	signal 	Indx_EPOT_value_prev			: std_logic_vector(EPOT_WIDTH - 1 downto 0);
	signal 	SA_EPOT_Indx_FifoWrite		: std_logic;
	
	signal 	ChA_EPOT_fifofull		: std_logic;
	signal 	ChB_EPOT_fifofull		: std_logic;
	signal 	Indx_EPOT_fifofull	: std_logic;
	
	-- SPI ADC
	constant ADC_WIDTH			: integer := 16;	
	
	signal 	SA_ChA_mon_complete		: std_logic;
	signal 	SA_ChB_mon_complete		: std_logic;
	signal 	SA_Indx_mon_complete		: std_logic;
	
	signal 	SA_ChA_mon_value			: std_logic_vector(ADC_WIDTH - 1 downto 0);
	signal	SA_ChB_mon_value			: std_logic_vector(ADC_WIDTH - 1 downto 0);
	signal	SA_Indx_mon_value			: std_logic_vector(ADC_WIDTH - 1 downto 0);
	
	signal 	SA_ChA_ADC_rx_sclk		: std_logic;
	signal 	SA_ChA_ADC_rx_synch		: std_logic;
	signal 	SA_ChA_ADC_rx_din			: std_logic;
	
	signal 	slv_CHAlive_value			: std_logic_vector(ADC_WIDTH - 1 downto 0);
	
	-- Misc
	signal	SA_SPI_Enable			: std_logic;
	
	signal 	SA_2s_tick				: std_logic;
	signal 	SA_1Hz_tick				: std_logic;
	signal 	SA_2Hz_tick				: std_logic;
	signal 	SA_50kHz_tick			: std_logic;
	signal 	SA_1MHz_tick			: std_logic;
	
	signal 	SA_2s_square			: std_logic;
	signal 	SA_1Hz_square			: std_logic;
	signal 	SA_2Hz_square			: std_logic;
	signal 	SA_50kHz_square		: std_logic;
	signal 	SA_1MHz_square			: std_logic;
	
	signal 	SA_reset_counters		: std_logic;
	signal 	SA_reset_counters1	: std_logic;
	signal	SA_reset_counters2	: std_logic;
	
	signal 	SA_Error_LED			: std_logic;
	
	-- Engineering
	signal	SA_EnableEngineering		: std_logic;
	signal 	SA_EngineerReg				: std_logic_vector(4 downto 0);
	
	signal 	sl_EngLampTestLED			: std_logic;
	signal 	slv_EngLampTestLED_now	: std_logic_vector(12 downto 0);
	signal 	slv_EngLampTestLED_next	: std_logic_vector(12 downto 0);
	signal 	sl_EngLampTestLEDReset		: std_logic;
	
	signal 	sl_EngLampTestIGBT 			: std_logic;
	signal 	slv_EngLampTestIGBT_now		: std_logic_vector(6 downto 0);
	signal 	slv_EngLampTestIGBT_next	: std_logic_vector(6 downto 0);
	signal 	sl_EngLampTestIGBTReset		: std_logic;
	
	signal 	sl_EngTrigAccept 				: std_logic;
	signal 	sl_EngTargetReady				: std_logic;
	signal 	sl_EngFrameEnable				: std_logic;
	signal 	sl_EngActTrig					: std_logic; 
	signal 	sl_EngBPS50R					: std_logic;
	signal 	sl_EngActuating				: std_logic;
	signal 	sl_EngSpillMarker				: std_logic;
	
	signal 	sl_EngBPSrelays				: std_logic;
	signal	sl_EngBPSControl				: std_logic_vector(2 downto 0);
	
	signal 	sl_EngLASERRelays				: std_logic;
	signal	sl_EngLASERControl			: std_logic_vector(2 downto 0);
	
	signal 	sl_EngFERelay					: std_logic;
	
	signal 	sl_EngPosTest					: std_logic;
	signal 	u_counter_next					: unsigned(DAC_WIDTH-9 downto 0);
	signal 	u_counter_now					: unsigned(DAC_WIDTH-9 downto 0);
	signal 	sl_TestCounterWrite			: std_logic;
	signal 	sl_TestCounterWrite2			: std_logic;
	signal 	sl_DAQ_FIFO_Write				: std_logic;
	
	signal 	sl_EngNudge						: std_logic;
	signal	slv_EngNUDGEControl			: std_logic_vector(6 downto 1);
	signal 	sl_Start_Nudge_Timer			: std_logic;
	signal 	sl_Nudge_Timer_Complete		: std_logic;
	constant NUDGE_TIME						: std_logic_vector(17 downto 0) := "001100001101010000"; --500 us
	-- 110000110101000000	2ms
	signal 	sl_NUDGE_TIME_NC				: std_logic_vector(17 downto 0);
	
	
	signal 	sl_Initialise_Timer_Complete		: std_logic;
	constant INITIALISE_TIME						: std_logic_vector(17 downto 0) := "001100001101010000"; --500 us
	-- 110000110101000000	2ms
	signal 	sl_INITIALISE_TIME_NC				: std_logic_vector(17 downto 0);
	signal 	sl_Start_Initialise_Timer			: std_logic;
	
	signal 	sl_StayAlive_ON_OFF			: std_logic;
	signal 	sl_FrameRaisedBPS_ON_OFF	: std_logic;
	signal 	sl_PC_Out						: std_logic;
	signal 	sl_OpticsMode					: std_logic; -- Not really used!
	signal 	sl_OpticsADCMode				: std_logic;
	
	-- End of Engineering
	
	signal 	slv_ChMonSelect				: std_logic_vector(1 downto 0);
	signal	sl_monitor_trigger			: std_logic;
	signal 	sl_parking						: std_logic;
	
	signal 	sl_DG_frame_raised			: std_logic;
	signal 	sl_DG_frame_lowered			: std_logic;
	
	-- Deglitched more signals 
	-- added by Ed :)
	signal 	sl_DG_More_frame_raised			: std_logic;
	signal 	sl_DG_More_frame_lowered			: std_logic;
	
	signal 	slv_Velocity					: std_logic_vector(31 downto 0);
	
	-- Will only allow errors to be reset if the target 
	-- is in its correct state.
	
	-- All Errors except BPS(i.e. Fatal Errors)
	-- Note AE is the exception.
	signal 	sl_Enable_Error_Reset_1		: std_logic;
	
	-- BPS Errors
	signal 	sl_Enable_Error_Reset_2		: std_logic;
	
	CONSTANT	MICE_DAQ_OK_VALUE				: std_logic := '0';
	-- 4.6ms Delay
	CONSTANT DAQ_TRIGGER_DELAY				: std_logic_vector(19 downto 0) := std_logic_vector(to_unsigned(184000,20));
	
	
begin
		-- *******************************************************
		-- *                                                     *
		-- *                                                     *
		-- *                                                     *
		-- *                                                     *
		-- *                                                     *
		-- *                                                     *
		-- *                    Instantiations							*
		-- *                                                     *
		-- *                                                     *
		-- *                                                     *
		-- *                                                     *
		-- *                                                     *
		-- *******************************************************
		
		-- Instantiate: Move_to_hold
		move_to_hold: entity work.move_to_hold_state_machine(arch)
		port map		
			(
			MTH_I_clk 									=> int_sysclk,
			MTH_I_reset 								=> int_logic_reset,
			MTH_I_hold_position_max 				=> SA_hold_position_max,
			MTH_I_stator_move_to_hold 				=> SA_move_to_hold_trigger,
			MTH_I_stator_move_to_park 				=> SA_move_to_park_trigger,
			MTH_O_stator_move_to_hold_complete 	=> SA_move_to_hold_complete,
			MTH_O_stator_move_to_park_complete 	=> SA_move_to_park_complete,
			MTH_timer_value							=>	SA_MTH_timer_value,
			MTH_O_coarse_pos 							=> SA_HoldState1,
			MTH_O_time_out 							=> MOVE_TO_HOLD_NO_CONNECTION
			);

		-- Instantiate: Hold_State_2_Mos_State
		hold_to_mos: entity work.hold_state_2_mos_state(arch)
		port map 	
			(
			HS2MS_I_Hold_State 	=> SA_HoldSwitchOut,
			HS2MS_O_Mos_State 	=> SA_HoldFinalState
			);
					
		-- Instantiate: Mos_State_2_Mos_Bin			
		Hold_mos_to_bin:	entity work.Mos_State_2_Bin(arch)
		port map 	
			(
			MS2B_I_Mos_state 	=>	SA_HoldFinalState,	
			MS2B_O_Mos_bin	 	=> SA_Hold_Bin
			);
					
		Actuate_mos_to_bin:	entity work.Mos_State_2_Bin(arch)
		port map 	
			(
			MS2B_I_Mos_state 	=>	SA_Actuate_State,	
			MS2B_O_Mos_bin	 	=> SA_Actuate_Bin
			);
					
		
		-- Instantiate: QC_Interface
		QC_Interface: entity work.QC_Interface(arch)
		port map 	
			(
			QCI_I_clk 			=> int_sysclk,
			QCI_I_reset 		=> int_logic_reset,
			QCI_I_RstToStart	=> SA_ResetQC,
	
			QCI_I_cha 			=> C_I_ch_a,
			QCI_I_chb 			=> C_I_ch_b,
			QCI_I_index 		=> C_I_index,
			QCI_I_IndexEnable	=> SA_IndexEnable,
			QCI_I_IndexDirection	=> SA_IndexDirection,
			
			QCI_ResetPosition	=> SA_QCPositionStart,	
			QCI_IndexOffset	=>	SA_QCIndexOffset,
			--QCI_ResetError		=> SA_ResetQuadError,
	
			QCI_O_direction 	=> SA_quad_direction,
			QCI_O_position	 	=> SA_quad_position,
		
			QCI_O_DeGlitchCHA		=> SA_DeGlitchedCHA,
			QCI_O_DeGlitchCHB		=> SA_DeGlitchedCHB,
	
			QCI_O_DeGlitchIndex 	=> SA_DeGlitchedIndex,
			
			QCI_O_ErrorFlag	=> SA_quad_ErrorFlag,
			QCI_O_error			=> SA_quad_error,
			QCI_O_error_value => SA_QuadErrorValue
			);
					
		-- Instantiate: Actuate
		Actuate: entity work.Actuate(arch)
		port map 	
			(
			ACT_I_clk 				=> int_sysclk,
			ACT_I_reset 			=> int_logic_reset,
			ACT_I_resetCount		=> SA_resetActCount,				-- Reset when moved to hold
			ACT_I_SetPoint1		=> SA_SP1LockedState,
			ACT_I_SP1MINIMUM		=> SA_SP1MINIMUM,
			ACT_I_SetPoint2		=> SA_SetPoint2,
			ACT_I_QuadPos			=> SA_quad_position,
			ACT_I_QuadDir			=> SA_quad_direction, 			-- : 0 = Down & 1 = Up
			ACT_I_StartAct			=> SA_actuate_trigger,
			ACT_I_KillActuation	=> SA_killActuation,
			
			ACT_I_DeadTime			=> SA_DeadTime,					-- : in std_logic_vector(11 downto 0);
			ACT_I_KickBackTime	=> SA_KickBackTime,				-- : in std_logic_vector(17 downto 0);

			ACT_I_ROMSelect		=> sl_ROMSelect,					-- 0 = OLD ROM, 1 = New ROM

			ACT_O_MosState			=> SA_actuate_state, 			-- : out std_logic_vector(3 downto 0);
			ACT_O_MinPos			=> SA_ActMinPos,
			ACT_O_ActComplete		=> SA_actuate_complete,			-- : out std_logic;
			ACT_O_HoldState		=> SA_HoldStateQuad, 			-- : out std_logic_vector(4 downto 0)
			ACT_O_ActCount			=> SA_ActCount,					-- : out std_logic_vector(23 downto 0);
			ACT_O_Actuating 		=> SA_Actuating,
			ACT_O_StartPos			=> SA_StartPos,
			
			ACT_O_SP1Time			=> SA_SP1Time,
			ACT_O_MinTime			=> SA_MinTime,
			ACT_O_ActTime			=> SA_ActTime,
			ACT_O_MasterTimer		=> SA_Actuation_Timer,
			ACT_O_Quad_Check		=> SA_QuadratureCheck,
			ACT_O_QCTime			=> SA_SelectQCTime,
			ACT_O_Quadrant_State	=> SA_ActQuadrant
			);
					
		-- Instantiate: Capture
		 Capture: entity work.Capture(arch)
		 Port map	
			( 
			CAP_I_clk 				=> int_sysclk, 				--	: in  STD_LOGIC;
			CAP_I_reset 			=> int_logic_reset, 			-- : in  STD_LOGIC;

			CAP_I_RequiredHold	=> SA_hold_position_max, 	-- : in std_logic_vector(5 downto 0);
			CAP_I_CurrentHold		=> SA_HoldStateQuad,			-- : in std_logic_vector(5 downto 0);
			CAP_I_StartCapture	=> SA_StartCapture,			-- : in std_logic;
			CAP_I_SettleTime		=> SA_CapSettleTime,			-- : in std_logic_vector(31 downto 0);
			CAP_I_KillCapture		=> SA_KillCapture,
			CAP_I_WaitTime			=> SA_CapWaitTime,			-- : in std_logic_vector(31 downto 0);

			CAP_O_HoldState		=> SA_HoldState2,				-- : out std_logic_vector(4 downto 0);
			CAP_O_CapComplete		=> SA_CapComplete,		   -- : out std_logic
			CAP_O_Trigger			=> SA_CapTrigger,
			CAP_O_Count				=> SA_CapCount,
			CAP_O_OverShoot		=> SA_CapOverShoot,
			CAP_O_UnderShoot		=> SA_CapUnderShoot,
			CAP_O_Active			=> SA_CapActive,				-- : out std_logic
			CAP_O_AquireActData	=> SA_AquireActData1,
			CAP_O_UnderOverShoot => SA_UnderOverShootTrig,
			CAP_O_BoostEnable		=> SA_PWMBoostEnable
			);
		
		-- 10s timer for capture
		
		capture_10s: entity work.Generic_Timer(arch)
			generic map (N=>32)
			port map   (
						clk 					=> int_sysclk,
						reset 				=> int_logic_reset,
						start 				=> SA_Cap10sStart,
						en 					=> '1',
						synch_clear 		=> '0',
						time_max_in 		=> "00010111110101111000010000000000", -- 10 seconds at 40Mhz - has to be binary as integer value not supported
						time_current_out  => open,
						tick 					=> SA_Cap10sComplete
					);
		
		-- 0.5ms timer for capture
		capture_500us: entity work.Generic_Timer(arch)
			generic map (N=>16)
			port map   (
						clk 					=> int_sysclk,
						reset 				=> int_logic_reset,
						start 				=> SA_Cap500usStart,
						en 					=> '1',
						synch_clear 		=> '0',
						time_max_in 		=> "0100111000100000", -- 500us
						time_current_out  => open,
						tick 					=> SA_Cap500usComplete
					);
		
		-- Instantiate: Mosfet_Swith
		MosfetSwitch: entity work.Mosfet_Switch(arch)
		port map		
			(
			MS_I_enable				=> SA_WDT_Out,
			MS_I_hold_select		=> SA_hold_select,		--	: in std_logic;
			MS_I_actuate_select	=> SA_actuate_select,	-- : in std_logic;

			MS_I_pwm_in				=> SA_PWMOut,

			MS_I_MOSFET_hold		=> SA_Hold_Bin, 			-- : in std_logic_vector(6 downto 1);
			MS_I_MOSFET_actuate	=> SA_Actuate_Bin, 		-- : in std_logic_vector(6 downto 1);

			MS_O_MOSFET				=> SA_MosfetBin,			-- : out std_logic_vector(6 downto 1)
			MS_O_MOSFETLED			=> SA_MosfetLed			-- : out std_logic_vector(6 downto 1)
			);
					
		-- Instantiate: Hold_Swith
		HoldSwitch: entity work.Hold_State_Switch(arch)
		port map		
			(		
			MS_I_enable				=> '1',						-- : in std_logic;
			MS_I_HoldSelect1		=> SA_HoldSwitch1,		-- : in std_logic;
			MS_I_HoldSelect2		=> SA_HoldSwitch2,		-- : in std_logic;

			MS_I_HOLD1				=> SA_HoldState1,			-- : in std_logic_vector(4 downto 0);
			MS_I_HOLD2				=> SA_HoldState2, 		-- : in std_logic_vector(4 downto 0);

			MS_O_HOLD				=> SA_HoldSwitchOut 		-- : out std_logic_vector(4 downto 0)
			);
						
		-- Instantiate: PWM Generator module
		PWM_module: entity work.PWM_generator(arch)
		generic map
			(
			PG_G_duty_cycle_bits => 8,
			PG_G_timer_bits => 8
			)
		port map		
			(
			PG_I_reset					=> int_logic_reset, 			-- : in	std_logic;
			PG_I_clk					 	=> int_sysclk,			 		-- : in	std_logic;
			PG_I_duty_cycle		 	=> SA_PWMDutyCycle, 			--	: in	std_logic_vector (PG_G_duty_cycle_bits-1 downto 0);
			PG_I_cycle_count_max 	=> SA_PWMCycleCountMax,		-- : in std_logic_vector (PG_G_duty_cycle_bits-1 downto 0);
			PG_I_timer				 	=> SA_PWMTimer,				--	: in std_logic_vector (PG_G_timer_bits-1 downto 0);
			PG_I_cycle_synch			=> SA_PWMSynchronise,		--	: in std_logic;
			PG_I_PWM_enable			=> SA_PWMEnable,				--	: in std_logic;
			PG_I_full_on				=> SA_PWMFullOn,				--	: in std_logic;
			
			--PG_I_Boost					=> BOOST,
			PG_I_Boost					=> SA_boost_value,
			PG_I_BoostEnable			=> SA_PWMBoostEnable,
			
			PG_O_pwm_out			 	=> SA_PWMOut					--	: out std_logic
			);
					
		-- Instantiate WDT

		-- Note SA_WDT_Out only goes low if the timer times out whilst
		-- trigger is held high
		-- WDT must recieve a reset if it latches low before operation can 
		-- resume
		
		WDT: entity work.WD_Timer(arch)
		generic map 
			(
			N => 24
			)
		port map		
			(
			WDT_I_clk 					=> int_sysclk,					-- : in  STD_LOGIC;
			WDT_I_reset 				=> int_logic_reset,			-- : in  STD_LOGIC;
			WDT_I_trigger 				=> SA_WDT_Trigger,			-- : in  std_logic;
			WDT_I_wdt_reset			=> SA_WDT_MasterReset,	--	: in  std_logic;
			WDT_I_timer_value			=> SA_WDT_TimerValue,
	
			WDT_O_out	 				=> SA_WDT_Out		--	: out std_logic
			);
		
		-- Instantiate Error Correction (SetPoint Adjust)
		EC: entity work.ErrorCorrection(arch)
		port map 	
			( 	
			EC_I_clk 				=> int_sysclk,			 	--: in std_logic;
			EC_I_reset 				=> int_logic_reset, 		--: in std_logic;
			EC_I_trigger			=> SA_CapTrigger,			--: in std_logic;
			EC_I_undershoot 		=> SA_CapUnderShoot,		--: in std_logic;
			EC_I_overshoot			=> SA_CapOverShoot,		--: in std_logic;
			EC_I_count				=> SA_CapCount,			--: in std_logic;
			EC_I_resetCount		=> '0',						--: in std_logic;
			EC_I_countMax			=> SA_ECCountMax,			--: in std_logic_vector(7 downto 0);
			EC_I_Threshold 		=> SA_ECThreshold,		--: in std_logic_vector(7 downto 0);
			EC_I_DefaultOffset	=> SA_ECDefaultOffset,
			EC_I_SP1					=> SA_SP1LockedState,	--: in std_logic_vector(8 downto 0);
			EC_I_SP1MINIMUM		=>	SA_SP1MINIMUM,	
			EC_O_SP2					=> SA_SetPoint2			--: out std_logic_vector(8 downto 0)
			);

		-- Original Settings
		--EC_I_countMax			=> SA_ECCountMax,			--: in std_logic_vector(7 downto 0);
		--EC_I_Threshold 			=> SA_ECThreshold,		--: in std_logic_vector(7 downto 0);
		--EC_I_DefaultOffset		=> SA_ECDefaultOffset,

	
		-- Instantiate Index Checker
		IndexChecker: entity work.IndexCheck(arch)
		Port map 	
			( 
			ERRCHK_I_clk 				=> int_sysclk,				--: in  STD_LOGIC;
			ERRCHK_I_reset 			=> int_logic_reset,		--: in  STD_LOGIC;
			ERRCHK_I_ResetLatch		=> SA_ResetIndexLatch,	--: in  std_Logic;
			ERRCHK_I_Index				=> SA_DeGlitchedIndex,	--: in  std_logic;
			ERRCHK_O_Latch				=> SA_IndexCheck			--: out std_logic
			);
			
		-- Instantiate Drop Checker
		DropCheck: entity work.DropDetect(arch)
		 Port map	
			(
			DD_DropLimit		=> SA_DropLimit,		 	-- Maybe this should be SP1 --: in std_logic_vector(8 downto 0);
			DD_Position			=> SA_quad_position,		--: in std_logic_vector(8 downto 0);
			DD_Enable			=> SA_DropDetectEnable,
			DD_DropDetect		=> SA_DropDetect			--: out std_logic
			);
					
					
		-- Instantiate ISIS Delay
		ISISDelay: entity work.ISIS_Timer_002(arch)
		Generic map
			(
			C_isis_t1_counter_width	=> 30,
			C_isis_t2_counter_width	=> 20
			--C_isis_mod_count_max	=> 64
			)
		port map		
			(
			I_rst						=> int_logic_reset,				-- : in	std_logic;
			I_40m_clk				=> int_sysclk,						-- : in	std_logic;

			I_delay_t1				=>	SA_ISISDelay1,					-- : in	std_logic_vector(C_isis_t1_counter_width - 1 downto 0);
			I_delay_t2				=>	SA_ISISDelay2,					-- : in	std_logic_vector(C_isis_t2_counter_width - 1 downto 0);
			I_LegalSpill			=> SA_LegalSpillDelay,
			I_DAQTrigger			=> DAQ_TRIGGER_DELAY,
			
			I_enableD2				=> SA_TriggerEnable,				-- : in  std_logic;
			I_isis_gms				=>	SA_GMS,							-- : in	std_logic;	
			O_actuate				=> SA_DelayedGMS,					-- : out	std_logic
			O_trigAccept 			=> SA_TriggerAccept50r,			-- : out std_logic
			O_trigAccept_NoGate	=> SA_triggerAccept50r_NoGate, 
			O_LegalSpill			=> SA_LegalSpill50r
			);
		
		-- Instantiate Key Enable
		KeyEnable: entity work.KeyEnable(arch)
		
		 Port map	
			( 
			clk 					=> int_sysclk,				--	: in  STD_LOGIC;
			reset 				=> int_logic_reset,		-- : in  std_logic;
	
			KE_I_KeyEnable 	=> C_I_key_en,				-- : in std_logic;
			KE_O_KeyEnable 	=> SA_KeyEnabled			-- : out std_logic
			);

		
		-- Instantiate Fixed Width Pulse Module
		FixedWidthPulseActuate: entity work.FixedWidthPulse(arch)
		Port map		
			( 
			clk 					=> int_sysclk,				-- : in  STD_LOGIC;
			reset			 		=> int_logic_reset,		-- : in  STD_LOGIC;
			FWP_I_SignalIn		=> SA_Actuating,			-- : in  std_logic;
			FWP_O_PulseOut		=>	SA_ActuatingTrig		-- : out std_logic
			);
						
		-- Instantiate Interlocks
		InterlocksOK: entity work.Interlocks(arch)	
		Port map			
			(
			clk						=> int_sysclk,
			reset 					=> int_logic_reset,
			
			INT_Frame_Raised		=> sl_DG_frame_raised,
			
			INT_I_Interlock1_5V	=> C_I_interlock_1_5v,	-- : in std_logic;
			INT_I_Interlock2_5V	=> C_I_interlock_2_5v,	-- : in std_logic;
			INT_I_Interlock3_5V	=> C_I_interlock_3_5v,	-- : in std_logic;
			INT_I_Interlock4_5V	=> C_I_interlock_4_5v,	-- : in std_logic;

			--INT_I_Interlock1_24V	=> C_I_interlock_1_24v,	-- : in std_logic;
			--INT_I_Interlock2_24V	=> C_I_interlock_2_24v,	-- : in std_logic;
			INT_I_Interlock3_24V	=> C_I_interlock_3_24v,	-- : in std_logic;
			INT_I_Interlock4_24V	=> C_I_interlock_4_24v,	-- : in std_logic;

			INT_O_Interlock_5V	=> SA_Interlocks5V_ok,	-- : out std_logic;
			INT_O_Interlock_24V	=> SA_Interlocks24V_ok,	-- : out std_logic;
			
			INT_O_5V_Tick			=> SA_Set5VIntError,	--: out std_logic;
			INT_O_24V_Tick			=> SA_Set24VIntError,	--: out std_logic
			INT_O_PPS_Interlock	=> SA_PPS_OK
			);
		
		--Output Block - Anything that goes to a hardware output goes through this block.
		OutputBlock: entity work.OutputBlock(arch)			
		Port map 	
			(
			clk 							=> int_sysclk,
			reset 						=> int_logic_reset,
			
			OB_I_Actuate				=> SA_Actuating,					-- : in 	std_logic;			 
			OB_O_Actuate1				=> C_O_actuating_50r_1,			-- : out std_logic;
			OB_O_Actuate2				=> sl_actuating_50r_2,			-- : out std_logic;
			OB_O_Actuate3				=> sl_actuating_50r_3,			-- : out std_logic;
			OB_O_ActuatingLED       => C_O_actuating_led,
			
			OB_I_ActuateTrig			=> SA_ActuatingTrig,				-- : out std_logic;
			OB_O_ActuateTrig1			=> C_O_actuating_trig_50r_1,	-- : out std_logic;
			OB_O_ActuateTrig2			=> sl_actuating_trig_50r_2,	-- : out std_logic;
			
			OB_I_MosfetLED				=> SA_MosfetLED,					-- : in std_logic_vector(6 downto 1);
			OB_O_MosfetLED				=>	C_O_Mosfet_LED,				-- : out std_logic_vector(6 downto 1);
 
			OB_I_Mosfet					=> SA_MosfetBin,					-- : in std_logic_vector(6 downto 1);
			OB_O_Mosfet					=>	C_O_Mosfet,						-- : out std_logic_vector(6 downto 1);
			
			OB_I_HoldLED				=> SA_Hold,							-- : in std_logic;
			OB_O_HoldLED				=> C_O_hold_led,					--	: out std_logic;

			OB_I_ParkLED				=> SA_Parked,						-- : in std_logic;
			OB_O_ParkLED				=> C_O_parked_led,				-- : out std_logic;
		
			OB_I_Position				=> SA_quad_position,				--	: in std_logic_vector(8 downto 0);
			OB_O_Position				=> C_O_target_pos,				-- : out std_logic_vector(8 downto 0);
			
			OB_I_KeyEnableDeGlitch	=> SA_KeyEnabled,
			OB_O_KeyEnabledDGLED		=> C_O_sys_en_led,				-- : out std_logic;
			
			OB_I_ActEnabledLED		=>	SA_ActuateEnabledLED,			-- : in std_logic;
			OB_O_ActEnabledLED		=>	C_O_actuate_en_led,			-- : out std_logic;
			
			OB_I_TargetReady			=> SA_TargetReady,
					
			OB_O_TargetReady50R		=> C_O_tgt_ready_50r,			-- : in std_logic;
			OB_O_TargetReadyLED		=> C_O_tgt_ready_led,			-- : out std_logic;
			
			OB_I_TargetFrameEnable	=> SA_TargetFrameEnableMASTER,		--	: in std_logic;
			OB_O_TargetFrameEnable	=> C_O_Frame_Enable_50r,		-- : out std_logic;
			OB_O_FrameEnableRelay	=> C_O_FRAME_EN_RELAY,
			
			-- Test Signals
			--OB_I_TriggerAccept		=> SA_ChA_mon_complete,
			--OB_I_TriggerAccept		=> SA_ChA_ADC_rx_din,
			--OB_I_TriggerAccept		=> sl_parking,
			-- Original Signals
			
			OB_I_TriggerAccept		=> SA_TriggerAccept50r,			--	: in std_logic;
			OB_O_TriggerAccept		=>	C_O_trig_accept_50r,			-- : out std_logic;
			
			
			OB_I_SpillMarker			=> SA_LegalSpill50r,
			OB_O_SpillMarker			=>	C_O_Legal_Spill_50r,
			
			OB_I_FrameRaised			=>	sl_DG_frame_raised,		-- : in std_logic;
			OB_O_FrameRaisedLED		=>	C_O_frame_raised_led,		-- : out std_logic;
			
			OB_I_FrameLowered			=>	sl_DG_frame_lowered,		-- : in std_logic;
			OB_O_FrameLoweredLED		=>	C_O_frame_lowered_led,		-- : out std_logic;
			
			OB_I_ErrorLED				=> SA_Error_LED,					--: in std_logic;
			OB_O_ErrorLED				=> C_O_Error_Led,					--: out std_logic;
			
			OB_I_BPS						=> (not SA_BPSErrorState),		-- Relays need to be active when this is low!
			OB_I_BPS_control			=> sl_EngBPSControl,
			OB_O_BPS_50R				=> C_O_BPS_50R,
			OB_O_BPS_Relay1			=>	C_O_BPS_RELAY_1,	
			OB_O_BPS_Relay2			=>	C_O_BPS_RELAY_2,	
			OB_O_BPS_Relay3			=>	C_O_BPS_RELAY_3,	
			
			OB_I_Engineering			=> SA_EnableEngineering,
			OB_I_1MHZ					=> SA_1MHz_tick,
			
			OB_I_EngLampTestLED		=> sl_EngLampTestLED,							--	: in std_logic;
			OB_I_EngLampTestLED_d	=> slv_EngLampTestLED_now(12 downto 1),	--: in std_logic_vector(8 downto 0);
			
			OB_I_EngLampTestIGBT		=> sl_EngLampTestIGBT,
			OB_I_EngLampTestIGBT_d	=> slv_EngLampTestIGBT_now(6 downto 1),
			
			OB_I_EngTrigAccept		=> sl_EngTrigAccept,
			OB_I_EngTargetReady		=> sl_EngTargetReady,
			OB_I_EngFrameEnable		=> sl_EngFrameEnable,
			OB_I_EngActTrig			=> sl_EngActTrig,
			OB_I_EngBPS50R				=> sl_EngBPS50R,
			OB_I_EngActuating			=> sl_EngActuating,		
			OB_I_EngSpillMarker		=> sl_EngSpillMarker,
			OB_I_EngBPSRelays			=> sl_EngBPSRelays,
			OB_I_EngFERelay 			=> sl_EngFERelay,
			
			OB_I_EngNudge				=> sl_EngNudge,
			OB_I_EngNudgeControl		=> slv_EngNUDGEControl,

			OB_I_FR_BPS_ON_OFF		=> sl_FrameRaisedBPS_ON_OFF,
			OB_I_BPS_LATCH_ENABLE	=> SA_BPSErrorStateEnable,

			OB_O_CHA_LED				=> C_O_ChA_LED,		
			OB_I_CHA_LED				=> C_I_ch_a,
			OB_O_CHB_LED				=> C_O_ChB_LED,		
			OB_I_CHB_LED				=> C_I_ch_b,
			OB_O_INDX_LED				=> C_O_Indx_LED,		
			OB_I_INDX_LED				=> SA_DeGlitchedIndex
			);
						
		USB_nc_filter:entity work.glitch_filter
		generic map	(	
						G_bit_width	=> 8
						)
		port map(
					I_rst			=> int_logic_reset,
					I_clk			=> int_sysclk,
					--
					-- Changed 30/06/2011 to turn USBnc OFf
					-- Changed 31/10/2011 to turn back on
					-- Unsure why it was turned off?
					--I_data_in	=> '1',
					I_data_in	=> C_I_USB_nc,
					O_data_out	=> SA_USB_nc_DG
					);
					
		USBnc_Tick: entity work.edge_detector(v0)
		port map(
			I_rst			=> int_logic_reset,
			I_clk			=> int_sysclk,
			--
			I_enable		=> '1',
			I_edge_type	=> '0',
			--
			I_data_in	=> SA_USB_nc_DG,
			O_edge_out	=> SA_USBnc_Tick
		);

		FatalError:entity work.Fatal_Error_Priority
		port map(
		
			clk 					=> int_sysclk,			--: in  STD_LOGIC;
			reset 				=>	int_logic_reset,	--: in  STD_LOGIC;
			
			FEP_I_WDT_Error	=>	SA_WDTErrorState,		--: in std_logic;
			FEP_I_DD_Error		=>	SA_DDErrorState,		--: in std_logic;
			FEP_I_Index_Error	=>	SA_IndexErrorState,	--: in std_logic;
			FEP_I_KE_Error		=>	SA_KEErrorState,		--: in std_logic;
			FEP_I_USBnc_Error	=>	SA_USBncErrorState,	--: in std_logic;
			FEP_I_IL5V_Error	=>	SA_5VIntErrorState,	--: in std_logic;
			FEP_I_IL24V_Error	=>	SA_24VIntErrorState,	--: in std_logic;
			FEP_I_ActEn_Error => SA_ActuateEnableError,
			FEP_I_PC_Error		=> SA_PCErrorState,
			FEP_I_Cap_Error	=> SA_CAPErrorState,
			FEP_I_BPS_Error 	=> SA_BPSErrorState,
			
			FEP_I_QuadADC_Error	=> SA_QuadADCErrorState,		
			FEP_I_QuadContinuity_Error	=>	SA_QuadContinuityErrorState,
			FEP_I_FrameNOTRaisedError => SA_FrameNOTRaisedErrorState,
					
			FEP_I_USBReset		=>	SA_USBFatalResets,	-- std_logic_vector(12 downto 0);
			FEP_I_MasterReset => SA_FatalErrorReset,
			
			FEP_O_ErrorCode	=>	SA_ErrorCode,	--: out std_logic_vector(3 downto 0);
			FEP_O_FatalError	=> SA_FatalError,
			
			FEP_O_WDTReset		=>	SA_ResetWDTError,		--: out std_logic;
			FEP_O_DDReset		=>	SA_ResetDDError,		--: out std_logic;
			FEP_O_IEReset		=>	SA_ResetIndexError,	--: out std_logic;
			FEP_O_KEReset		=>	SA_ResetKEError,		--: out std_logic;
			FEP_O_USBncReset	=>	SA_ResetUSBncError,	--: out std_logic;
			FEP_O_IL5VReset	=>	SA_Reset5VIntError,	--: out std_logic;
			FEP_O_IL24VReset	=>	SA_Reset24VIntError,	--: out std_logic
			FEP_O_PCReset		=> SA_ResetPCError,
			FEP_O_CEReset		=> SA_ResetCapError,
			FEP_O_BPSReset		=> SA_ResetBPSError,
			FEP_O_QuadADCReset			=> SA_ResetQuadADCError,
			FEP_O_QuadContinuityReset	=> SA_ResetQuadContinuityError,
			FEP_O_FrameNOTRaisedReset	=> SA_ResetFrameNOTRaisedError
			);
			
		ActuateData:entity work.Actuation_Data(arch)
		port map(
			AD_I_clk 				=> int_sysclk,					--	: in STD_LOGIC;
			AD_I_reset				=> int_logic_reset, 			--	: in STD_LOGIC;

			AD_I_CaptureData		=> SA_AquireActData,			-- : in std_logic;
			AD_I_ActuateTrig		=> SA_actuate_trigger,		-- : in std_logic;
			AD_I_USBAcknowledge	=> SA_USBAckActData,			-- : in std_logic;
			-- NOTE THAT SA_USBAckActData  <= USB_ResetRegister(30);
		
			AD_I_StartDepth		=> SA_StartPos,				-- : in std_logic_vector(8 downto 0);
			AD_I_MinDepth			=> SA_ActMinPos,				-- : in std_logic_vector(8 downto 0);
			AD_I_Count				=> SA_ActCount,				-- : in std_logic_vector(23 downto 0);
		
			AD_I_SP1Time			=> SA_SP1Time,					--: in std_logic_vector(31 downto 0);
			AD_I_MinTime			=> SA_MinTime,					--: in std_logic_vector(31 downto 0);
			AD_I_ActTime			=> SA_ActTime,					--: in std_logic_vector(31 downto 0);
		
			AD_I_QuadErrorValue	=> SA_QuadErrorValue,		-- : in std_logic_vector(8 downto 0);
			AD_I_QuadErrorCode	=> SA_quad_error,				-- : in std_logic_vector(2 downto 0);
			AD_I_QuadFlag			=> SA_quad_ErrorFlag,		-- : in std_logic;
		
			AD_I_UnderCapture		=> SA_CapUnderShoot,			-- : in std_logic;
			AD_I_OverCapture		=> SA_CapOverShoot,			-- : in std_logic;
			AD_I_UnderOverTrig	=> SA_UnderOverShootTrig,	-- : in std_logic;
		
			AD_I_SP1Locked			=> SA_SP1LockedState,		-- : in std_logic_vector(8 downto 0);
			AD_I_SP2					=> SA_SetPoint2,				-- : in std_logic_vector(8 downto 0);
		
			AD_I_BPSErrors			=>	SA_BPSErrors,
		
			AD_I_BPSSp1MaxTime			=> USB_Sp1MaxTime,
			AD_I_BPSSp1MinTime			=> USB_Sp1MinTime,
			AD_I_BPSMinPosMaxTime		=> USB_MinPosMaxTime,
			AD_I_BPSMinPosMinTime		=> USB_MinPosMinTime,
			AD_I_BPSMinPosMaxValue		=> USB_MinPosMaxValue,
			AD_I_BPSMinPosMinValue		=> USB_MinPosMinValue,
			AD_I_BPSActMaxTime			=> USB_ActMaxTime,
			AD_I_BPSActMinTime			=> USB_ActMinTime,
		
			AD_O_ActuationPos		=> SA_ActuationMinMax,		-- : out std_logic_vector(31 downto 0);
			AD_O_ActuationCount 	=> USB_ActuationCount,		-- : out std_logic_vector(31 downto 0);
			AD_O_ActuationError  => USB_ActuationError,		-- : out std_logic_vector(31 downto 0);
			AD_O_SetPoints			=> USB_ActuationSP,			-- : out std_logic_vector(31 downto 0);
		
			AD_O_SP1Time			=>	USB_ActuationSP1Time,		-- : out std_logic_vector(31 downto 0);
			AD_O_MinTime			=>	USB_ActuationMinTime,		-- : out std_logic_vector(31 downto 0);
			AD_O_ActTime			=>	USB_ActuationTime,			-- : out std_logic_vector(31 downto 0);
			
			AD_O_BPSErrors			=> USB_BPSErrorData,
			
			AD_O_BPSSp1MaxTime			=> USB_ActuationSp1MaxTime,
			AD_O_BPSSp1MinTime			=> USB_ActuationSp1MinTime,
			AD_O_BPSMinPosMaxTime		=> USB_ActuationMinPosMaxTime,		
			AD_O_BPSMinPosMinTime		=> USB_ActuationMinPosMinTime,		
			AD_O_BPSMinPosMaxValue		=> USB_ActuationMinPosMaxValue,	
			AD_O_BPSMinPosMinValue		=> USB_ActuationMinPosMinValue,	
			AD_O_BPSActMaxTime			=> USB_ActuationActMaxTime,		
			AD_O_BPSActMinTime			=> USB_ActuationActMinTime,		
			
			AD_I_CHA_Complete				=> SA_ChA_mon_complete,
			AD_I_CHB_Complete				=> SA_ChB_mon_complete,
			AD_I_INDX_Complete			=> SA_Indx_mon_complete,
		
			AD_I_CHA_Mon					=> SA_ChA_mon_value,
			AD_I_CHB_Mon					=> SA_ChB_mon_value,
			AD_I_INDX_Mon					=> SA_Indx_mon_value,
		
			AD_O_CHA_Mon					=> USB_ActuationChAOutput,
			AD_O_CHB_Mon					=> USB_ActuationChBOutput,
			AD_O_INDX_Mon					=> USB_ActuationIndxOutput,

			AD_I_Velocity					=> slv_Velocity,
			AD_O_Velocity					=> USB_Velocity,
			
			AD_O_DataGood			=> SA_ActDataAvailable,		-- : out std_logic
			AD_O_FIFOFull			=> SA_SetFIFONotify		
			);
			
		NotifyData:entity work.Notification_Handler(arch)
		port map(	
			NH_I_clk 					=> int_sysclk,					--: in  STD_LOGIC;
			NH_I_reset 					=> int_logic_reset,			--: in  STD_LOGIC;
			
			NH_I_Notify_001_set		=> SA_SetInitOKNotify,		--: in std_logic;
			NH_I_Notify_002_set		=> SA_SetParkNotify,			--: in std_logic;
			NH_I_Notify_003_set		=> SA_SetFIFONotify,			--: in std_logic;
			
			NH_I_USBAck					=> SA_USBAckNotify,			--: in std_logic;
			
			NH_O_NotifyFlag			=> SA_NotifyAvailable,
			NH_O_NotificationCode	=> SA_NotificationCode		-- : out std_logic_vector(7 downto 0)
			);
		
		-- BPS Instantiation
		BPS: entity work.BPS_001(arch)
		port map( 
			clk 							=> int_sysclk,
         reset 						=> int_logic_reset,
			
			i_BPS_Sp1MaxTime			=> USB_Sp1MaxTime,
			i_BPS_Sp1MinTime			=> USB_Sp1MinTime,	
			i_BPS_MinPosMaxTime		=> USB_MinPosMaxTime,
			i_BPS_MinPosMinTime		=> USB_MinPosMinTime,
			i_BPS_MinPosMaxValue		=> USB_MinPosMaxValue,
			i_BPS_MinPosMinValue		=> USB_MinPosMinValue,
			i_BPS_ActMaxTime			=> USB_ActMaxTime,
			i_BPS_ActMinTime			=> USB_ActMinTime,
			
			i_BPS_Sp1Time				=> SA_SP1Time,
			i_BPS_MinPosTime			=> SA_MinTime,
			i_BPS_MinPosValue			=> SA_ActMinPos,
			i_BPS_ActTime				=> SA_ActTime,
			i_BPSDisabled				=> sl_BPSDisabled,
			
			i_check_limits				=> SA_CheckLimits,
			
			i_BPS_Resets				=> SA_USBBPSResets,
			
			i_WDT_Error					=> SA_WDTErrorState,
			i_DD_Error					=> SA_DDErrorState,
			i_QCCont_Error				=> SA_QuadContinuityErrorState,
			i_Frame_Error				=> SA_FrameNOTRaisedErrorState,
			
			o_BPS_Error					=> SA_SetBPSError,
			o_BPS_ErrorTick			=> SA_BPSErrorTickNC,
			o_BPS_ErrorsReg			=> SA_BPSErrors
			);
		
		-- PC Instantiation
		PC_Counter: entity work.pc_counter(arch)
		port map(  
			clk 				=> int_sysclk,
         reset 			=> int_logic_reset,
			
			pc_i_usbpc		=> USB_PCCounter,
			
			pc_i_1sec_tick	=> SA_1Hz_tick,
			
			pc_o_set_error	=> sl_PC_Out	
			);	
			
		-- SPI DAC INSTANTIATION	
		SPI_DAC: entity work.spi_tx_004(arch)
		generic map (g_SPI_Data_Width => DAC_WIDTH)
		port map(
			clk 							=> int_sysclk,
			reset 						=> int_logic_reset,
			i_spi_tx_fifo_data_in	=> DAC_FIFODataIn,
			i_spi_tx_fifo_write		=> sl_DAQ_FIFO_Write,
			i_spi_tx_sclk_edge		=> '1',
			i_spi_tx_en					=> SA_SPI_Enable, 
			
			--i_spi_tx_gate_sclk		=> '1', -- Original
			i_spi_tx_gate_sclk		=> '0', 
			--i_spi_tx_gate_value		=> '1', -- Original 
			i_spi_tx_gate_value		=> '0',
			
			o_spi_tx_fifo_full		=> DAC_fifofull,
			o_spi_tx_dout				=> C_O_DAQ_tx_dout,
			o_spi_tx_sclk				=> C_O_DAQ_tx_sclk, -- rate determined by Enable
			o_spi_tx_synch				=> C_O_DAQ_tx_synch
		);
	
		-- SPI CHA Gain Instantiation
		SPI_CHA_EPOT: entity work.spi_tx_004(arch)
		generic map (g_SPI_Data_Width => EPOT_WIDTH)
		port map(
			clk 							=> int_sysclk,
			reset 						=> int_logic_reset,
			i_spi_tx_fifo_data_in	=> ChA_EPOT_value_now,
			i_spi_tx_fifo_write		=> SA_EPOT_ChA_FifoWrite,
			i_spi_tx_sclk_edge		=> '0',
			i_spi_tx_en					=> SA_SPI_Enable, 
			
			i_spi_tx_gate_sclk		=> '1',
			i_spi_tx_gate_value		=> '0',
			
			o_spi_tx_fifo_full		=> ChA_EPOT_fifofull,
			o_spi_tx_dout				=> C_O_ChA_Epot_tx_dout,
			o_spi_tx_sclk				=> C_O_ChA_Epot_tx_sclk, -- should be running at clk/2 i.e. 20MHz
			o_spi_tx_synch				=> C_O_ChA_Epot_tx_synch
		);
		
		-- SPI CHB Gain Instantiation
		SPI_CHB_EPOT: entity work.spi_tx_004(arch)
		generic map (g_SPI_Data_Width => EPOT_WIDTH)
		port map(
			clk 							=> int_sysclk,
			reset 						=> int_logic_reset,
			i_spi_tx_fifo_data_in	=> ChB_EPOT_value_now,
			i_spi_tx_fifo_write		=> SA_EPOT_ChB_FifoWrite,
			i_spi_tx_sclk_edge		=> '0',
			i_spi_tx_en					=> SA_SPI_Enable, 
			
			i_spi_tx_gate_sclk		=> '1',
			i_spi_tx_gate_value		=> '0',
			
			o_spi_tx_fifo_full		=> ChB_EPOT_fifofull,
			o_spi_tx_dout				=> C_O_ChB_Epot_tx_dout,
			o_spi_tx_sclk				=> C_O_ChB_Epot_tx_sclk, -- should be running at clk/2 i.e. 20MHz
			o_spi_tx_synch				=> C_O_ChB_Epot_tx_synch
		);
		
		-- SPI INDX Gain Instantiation
		SPI_INDX_EPOT: entity work.spi_tx_004(arch)
		generic map (g_SPI_Data_Width => EPOT_WIDTH)
		port map(
			clk 							=> int_sysclk,
			reset 						=> int_logic_reset,
			i_spi_tx_fifo_data_in	=> Indx_EPOT_value_now,
			i_spi_tx_fifo_write		=> SA_EPOT_Indx_FifoWrite,
			i_spi_tx_sclk_edge		=> '0',
			i_spi_tx_en					=> SA_SPI_Enable, 
			
			i_spi_tx_gate_sclk		=> '1',
			i_spi_tx_gate_value		=> '0',
			
			o_spi_tx_fifo_full		=> Indx_EPOT_fifofull,
			o_spi_tx_dout				=> C_O_Indx_Epot_tx_dout,
			o_spi_tx_sclk				=> C_O_Indx_Epot_tx_sclk, -- should be running at clk/2 i.e. 20MHz
			o_spi_tx_synch				=> C_O_Indx_Epot_tx_synch
		);
		
		-- SPI ChA Monitor
		SPI_CHA_MONITOR: entity work.spi_monitor(arch)
		generic	map(SPI_DataWidth => ADC_WIDTH)
		port map( 
			clk 						=> int_sysclk,			
         reset 					=> int_logic_reset,
			
			i_rxmon_read			=> sl_monitor_trigger, -- this starts the acquisition process
			i_rxmon_spi_din		=> SA_ChA_ADC_rx_din,
			
			i_SPI_enable			=> SA_SPI_Enable,
			
			i_current_state		=> SA_CurrentState,
			i_channel_DG			=> SA_DeGlitchedCHA,
			
			-- Only Channel A can be monitored
			-- Useful for setting up an optics channel
			i_continuous_monitor	=> sl_OpticsADCMode,
			
			o_rxmon_value			=> SA_ChA_mon_value,
			o_rxlive_value			=> slv_CHAlive_value,
			o_rxmon_busy			=> open,
			o_rxmon_complete 		=> SA_ChA_mon_complete,
			
			o_rxmon_spi_sclk		=> SA_ChA_ADC_rx_sclk,	
			o_rxmon_spi_synch		=> SA_ChA_ADC_rx_synch,
			o_rxmon_error			=> SA_SetQuadADCErrorCHA
			);
		
		-- SPI ChB Monitor
		SPI_CHB_MONITOR: entity work.spi_monitor(arch)
		generic	map(SPI_DataWidth => ADC_WIDTH)
		port map( 
			clk 						=> int_sysclk,			
         reset 					=> int_logic_reset,
			
			i_rxmon_read			=> sl_monitor_trigger, -- this starts the acquisition process
			i_rxmon_spi_din		=> C_I_ChB_ADC_rx_din,
			
			i_SPI_enable			=> SA_SPI_Enable,
			
			i_current_state		=> SA_CurrentState,
			i_channel_DG			=> SA_DeGlitchedCHB,
			
			i_continuous_monitor	=> '0',
			
			o_rxmon_value			=> SA_ChB_mon_value,
			o_rxlive_value			=> open,
			o_rxmon_busy			=> open,
			o_rxmon_complete 		=> SA_ChB_mon_complete,
			
			o_rxmon_spi_sclk		=> C_O_ChB_ADC_rx_sclk,	
			o_rxmon_spi_synch		=> C_O_ChB_ADC_rx_synch,
			o_rxmon_error			=> SA_SetQuadADCErrorCHB
			);
		
		
		-- SPI Index Monitor
		SPI_INDX_MONITOR: entity work.spi_monitor(arch)
		generic	map(SPI_DataWidth => ADC_WIDTH)
		port map( 
			clk 						=> int_sysclk,			
         reset 					=> int_logic_reset,
			
			i_rxmon_read			=> sl_monitor_trigger, -- this starts the acquisition process
			i_rxmon_spi_din		=> C_I_Indx_ADC_rx_din,
			
			i_SPI_enable			=> SA_SPI_Enable,
			
			i_current_state		=> SA_CurrentState,
			i_channel_DG			=> SA_DeGlitchedIndex,
			
			i_continuous_monitor	=> '0',
			
			o_rxmon_value			=> SA_Indx_mon_value,
			o_rxlive_value			=> open,
			o_rxmon_busy			=> open,
			o_rxmon_complete 		=> SA_Indx_mon_complete,
			
			o_rxmon_spi_sclk		=> C_O_Indx_ADC_rx_sclk,	
			o_rxmon_spi_synch		=> C_O_Indx_ADC_rx_synch,
			o_rxmon_error			=> SA_SetQuadADCErrorINDEX
			);
		
		
		-- SPI Index Monitor
		--SPI_INDX_MONITOR: entity work.spi_indx_monitor(arch)
--		SPI_INDX_MONITOR: entity work.spi_index_monitor(arch)
--		generic	map(SPI_DataWidth => ADC_WIDTH)
--		port map( 
--			clk 						=> int_sysclk,			
--         reset 					=> int_logic_reset,
--			-- this starts the acquisition process
--			i_rxmon_read			=> sl_monitor_trigger,
--			-- This might be the wrong trigger - may required undeglitched output?
--			--i_rxmon_read			=> SA_DeGlitchedIndex, 
--										
--			i_rxmon_spi_din		=> C_I_Indx_ADC_rx_din,
--			
--			i_SPI_enable			=> SA_SPI_Enable,
--			
--			i_current_state		=> SA_CurrentState,
--			i_index_DG				=> SA_DeGlitchedIndex,
--			
--			o_rxmon_value			=> SA_Indx_mon_value,
--			o_rxmon_busy			=> open,
--			o_rxmon_complete 		=> SA_Indx_mon_complete,
--			
--			o_rxmon_spi_sclk		=> C_O_Indx_ADC_rx_sclk,	
--			o_rxmon_spi_synch		=> C_O_Indx_ADC_rx_synch,
--			o_rxmon_error			=> SA_SetQuadADCErrorINDEX
--			);
			
		-- Watchdog Protector for Capture
		CAPTURE_PROTECT: entity work.event_protector(arch)
		generic map(N => 32)
		port map(
			clk 				=> int_sysclk,
			reset 			=> int_logic_reset,		
			
			start				=> SA_CaptureStartProt,
			stopreset		=> SA_CaptureStopProt,
			time_max_in		=> "10001111000011010001100000000000", -- 1 minute at 40Mhz - has to be binary as integer value not supported
			overflow			=> SA_CaptureOverflowProt
			);
	
	--SPI Enable (runs SPI clocks at FPGAclk/2n)	
	SPI_EN_CLK:entity work.mod_n_counter_en_001(arch)
	generic	map(
				g_counter_bits	=> 3,
				g_modulus		=> 5  
				)
	port map( 
			clk 		=> int_sysclk,
         reset 	=> int_logic_reset,
			
			i_en		=> '1',
			i_reset_counters	=> '0',
			
			o_tick	=> SA_SPI_Enable,
			o_square => open
			);
		
	--2s Clk	
	TWO_S_CLK:entity work.mod_n_counter_en_001(arch)
	generic	map(
				g_counter_bits	=> 16,
				g_modulus		=> 50000  
				)
	port map( 
			clk 		=> int_sysclk,
         reset 	=> int_logic_reset,
			
			i_en		=> SA_50kHz_tick,
			i_reset_counters	=> SA_reset_counters,
			
			o_tick	=> SA_2s_tick	
	
,
			o_square => SA_2s_square
			);		
			
	--1Hz Tick 
	ONE_HZ_CLK:entity work.mod_n_counter_en_001(arch)
	generic	map(
				g_counter_bits	=> 15,
				g_modulus		=> 25000  
				)
	port map( 
			clk 		=> int_sysclk,
         reset 	=> int_logic_reset,
			
			i_en		=> SA_50kHz_tick,
			i_reset_counters	=> SA_reset_counters,
			
			o_tick	=> SA_1Hz_tick	
,
			o_square => SA_1Hz_square
			);
			
	--2Hz Tick	
	TWO_HZ_CLK:entity work.mod_n_counter_en_001(arch)
	generic	map(
				g_counter_bits	=> 14,
				g_modulus		=> 12500  
				)
	port map( 
			clk 		=> int_sysclk,
         reset 	=> int_logic_reset,
			
			i_en		=> SA_50kHz_tick,
			i_reset_counters	=> SA_reset_counters,
			
			o_tick	=> SA_2Hz_tick,
			o_square => SA_2Hz_square
			);
		
	--50kHz Clk	
	FIFTY_KHZ_CLK:entity work.mod_n_counter_en_001(arch)
	generic	map(
				g_counter_bits	=> 4,
				g_modulus		=> 10  
				)
	port map( 
			clk 		=> int_sysclk,
         reset 	=> int_logic_reset,
			
			--i_en		=> '1',
			i_en			=> SA_1MHz_tick,
			i_reset_counters	=> '0',
			
			o_tick	=> SA_50kHz_tick,
			o_square => SA_50kHz_square
			);
		
	--1MHz Clk	
	ONE_MHZ_CLK:entity work.mod_n_counter_en_001(arch)
	generic	map(
				g_counter_bits	=> 5,
				g_modulus		=> 20  
				)
	port map( 
			clk 		=> int_sysclk,
         reset 	=> int_logic_reset,
			
			i_en		=> '1',
			i_reset_counters	=> '0',
			
			o_tick	=> SA_1MHz_tick,
			o_square => SA_1MHz_square
			);
			
	LED_TEST:entity work.edge_detector(v0)
	port map(
		I_rst			=> int_logic_reset,
		I_clk			=> int_sysclk,
		--
		I_enable		=> '1',
		I_edge_type	=> '1',
		--
		I_data_in	=> sl_EngLampTestLED,
		O_edge_out	=> sl_EngLampTestLEDReset
	);
	
	IGBT_LAMP_TEST:entity work.edge_detector(v0)
	port map(
		I_rst			=> int_logic_reset,
		I_clk			=> int_sysclk,
		--
		I_enable		=> '1',
		I_edge_type	=> '1',
		--
		I_data_in	=> sl_EngLampTestIGBT,
		O_edge_out	=> sl_EngLampTestIGBTReset
	);

	--Instantiate a "x" ms Timer for NUDGE		
	Nudge_timer: entity work.Generic_Timer(arch)
	generic map (N=>18)
	port map   (
		clk => 	int_sysclk,
		reset => int_logic_reset,
		start => sl_Start_Nudge_Timer,
		en => '1',
		synch_clear => '0',
		time_max_in => NUDGE_TIME,
		time_current_out => sl_NUDGE_TIME_NC,
		tick => sl_Nudge_Timer_Complete
	);
	
	--Instantiate a "x" ms Timer for initialise		
	Initialise_timer: entity work.Generic_Timer(arch)
	generic map (N=>18)
	port map   (
		clk => 	int_sysclk,
		reset => int_logic_reset,
		start => sl_Start_Initialise_Timer,
		en => '1',
		synch_clear => '0',
		time_max_in => INITIALISE_TIME,
		time_current_out => sl_INITIALISE_TIME_NC,
		tick => sl_Initialise_Timer_Complete
	);
	
	-- Deglitch for Frame Raised/Lowered Signals
	Frame_Raised: entity work.glitch_filter
		generic map(
			G_bit_width	=> 32
		)
		port map(
			I_rst			=> int_logic_reset,
			I_clk			=> int_sysclk,
			--
			I_data_in	=> C_I_tgt_frame_raised,
			O_data_out	=> sl_DG_frame_raised
		);
		
	Frame_Lowered: entity work.glitch_filter
		generic map(
			G_bit_width	=> 32
		)
		port map(
			I_rst			=> int_logic_reset,
			I_clk			=> int_sysclk,
			--
			I_data_in	=> C_I_tgt_frame_lowered,
			O_data_out	=> sl_DG_frame_lowered
		);
		
	Frame_Lowered_DGMore : entity work.min_width_mod
	port map (
		clk => int_sysclk,
		en_tick => SA_50kHz_tick,
		sig_input => sl_DG_frame_lowered,
		sig_output => sl_DG_More_frame_lowered	
	);
	
	Frame_Raised_DGMore : entity work.min_width_mod
	port map (
		clk => int_sysclk,
		en_tick => SA_50kHz_tick,
		sig_input => sl_DG_frame_raised,
		sig_output => sl_DG_More_frame_raised	
	);
		
	Quad_Continuity_CHA: entity work.QuadCont
		port map(
			reset			=> int_logic_reset,
			clk			=> int_sysclk,
			
			i_runQC		=> SA_QuadratureCheck,
			i_DGCH		=> SA_DeGlitchedCHA,
			i_selectTime => SA_SelectQCTime,
			
			o_QCError	=> SA_SetQuadContinuityErrorA
		);
	
	Quad_Continuity_CHB: entity work.QuadCont
		port map(
			reset			=> int_logic_reset,
			clk			=> int_sysclk,
			
			i_runQC		=> SA_QuadratureCheck,
			i_DGCH		=> SA_DeGlitchedCHB,
			i_selectTime => SA_SelectQCTime,
			
			o_QCError	=> SA_SetQuadContinuityErrorB
		);
		
	Velocity: entity	work.target_velocity
		port map(
			reset 			=> int_logic_reset,
			clk 				=> int_sysclk,
				
			I_Actuating		=>	SA_Actuating,
				
			I_Locked_SP1			=> SA_SP1LockedState,
			I_Position				=> SA_quad_position,
			I_Quadrant				=>	SA_ActQuadrant,	
			I_Actuation_timer		=> SA_Actuation_Timer,
				
			O_Velocity				=> slv_Velocity
		);
	
	-- *******************************************************
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                END of Instantiations					  	*
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *******************************************************
	
	SA_SetQuadADCError <=  (SA_SetQuadADCErrorCHA or SA_SetQuadADCErrorCHB	
		or SA_SetQuadADCErrorINDEX);

	SA_SetFrameNOTRaisedError <= (SA_SetFrameNOTRaisedError1 or SA_SetFrameNOTRaisedError2);
	
	SA_SetQuadContinuityError <= (SA_SetQuadContinuityErrorA or SA_SetQuadContinuityErrorB);
	--SA_SetQuadContinuityError <= '0';
	
	SA_reset_counters		<= (SA_reset_counters1 or SA_reset_counters2);
	
	-- Target Frame is only enabled if PPS is good.
	SA_TargetFrameEnableMASTER	<= (SA_TargetFrameEnable AND SA_PPS_OK);
	
	SA_move_to_hold_in 	<= USB_ControlReg(0);
	SA_move_to_park_in 	<= USB_ControlReg(1);
	SA_EnableActuation 	<= USB_ControlReg(2);
	SA_EnableEngineering <= USB_ControlReg(3);
	
	--C_O_FRAME_EN_RELAY	<= SA_TargetFrameEnable;
							
	C_O_LASER_RELAY_1	 	<= sl_EngLASERControl(0);
	C_O_LASER_RELAY_2		<= sl_EngLASERControl(1);
	C_O_LASER_RELAY_3		<= sl_EngLASERControl(2);
		
	C_O_SPARE_RELAY		<= '0';
	
	SA_ChA_ADC_rx_din			<= C_I_ChA_ADC_rx_din;
	C_O_ChA_ADC_rx_sclk		<= SA_ChA_ADC_rx_sclk;	
	C_O_ChA_ADC_rx_synch		<= SA_ChA_ADC_rx_synch;
		
	C_O_Spare_2_50R	<= SA_triggerAccept50r_NoGate;
	C_O_Spare_3_50R	<= '0';
	
	-- ***** TEMPORARY *****************************
	--C_O_Spare_2_50R	<= SA_QuadratureCheck;
	--C_O_Spare_2_50R	<= SA_DeGlitchedCHA;
	
	--C_O_Spare_3_50R	<= SA_SetQuadContinuityError;
	--C_O_Spare_3_50R	<= SA_DeGlitchedCHB;
	
	--C_O_Spare_2_50R		<= SA_1MHz_tick;
	--C_O_Spare_3_50R		<= SA_1MHz_square;
	
	--C_O_Spare_2_50R	<= SA_ChA_ADC_rx_sclk;
	--C_O_Spare_3_50R	<= SA_ChA_ADC_rx_synch;
	
	--C_O_Spare_2_50R	<= SA_PWMEnable;
	--C_O_Spare_3_50R	<= SA_PWMOut;
	
	--C_O_Spare_3_50R	 <= USB_Engineering(27); -- BPS Disable
	-- *********************************************
	
	SA_AquireActData		<= SA_AquireActData1 or SA_AquireActData2;	
	sl_DAQ_FIFO_Write		<= (SA_position_change or sl_TestCounterWrite or
		sl_TestCounterWrite2);

	SA_SetPoint1 			<= USB_SP1Wide(8 downto 0);
	SA_MTH_timer_value 	<= USB_MTH_Timer(27 downto 0);
			
	SA_hold_position_max <= USB_HoldPosMax(5 downto 0);
	--SA_PWMDutyCycle2		<= USB_HoldPosMax(31 downto 24);
	SA_boost_value			<= unsigned(USB_CaptureBoost(7 downto 0));
	
	SA_PWMCycleCountMax	<= USB_PWMSetup(7 downto 0);
	SA_PWMTimer				<= USB_PWMSetup(15 downto 8);
	SA_PWMDutyCycle		<= USB_PWMSetup(23 downto 16);
	SA_WDT_TimerValue		<= USB_WDTValue(23 downto 0);	
	SA_ISISDelay1 			<= USB_ISISDelay1(29 downto 0);
	SA_ISISDelay2 			<= USB_ISISDelay2	(19 downto 0);
	SA_ECCountMax			<=	USB_ErrorCorrection(7 downto 0);				
	SA_ECThreshold			<= USB_ErrorCorrection(15 downto 8);
	SA_ECDefaultOffset	<= USB_ErrorCorrection(24 downto 16); 	--9 bits
	SA_CapSettleTime		<= USB_CapSettleTime(31 downto 0);
	SA_CapWaitTime			<= USB_CapWaitTime(31 downto 0);	
	SA_QCPositionStart	<=	USB_QCSetup(8 downto 0);				-- 9 bits			
	SA_QCIndexOffset		<= USB_QCSetup	(17 downto 9);				-- 9 bits
	SA_DeadTime				<= USB_DeadTime(11 downto 0);
	SA_KickBackTime		<=	USB_KickBackTime(17 downto 0);
	SA_DropLimit			<= USB_DropLimit(8 downto 0);
	SA_LegalSpillDelay	<= USB_LegalSpillDelay(19 downto 0);
	
	-- Other notify bits are used to report CHA/B/INDX monitor valued
	USB_NotifyRegister(15 downto 8) <= (others => '0');
	USB_NotifyRegister(7 downto 0) <= SA_NotificationCode;

	-- USB Status Register
	USB_StatusRegister(0)				<= SA_KeyEnabled;
	USB_StatusRegister(1)				<= sl_DG_frame_raised;
	USB_StatusRegister(2)				<= sl_DG_frame_lowered;
	USB_StatusRegister(3)				<= SA_TargetFrameEnable;
	USB_StatusRegister(4)				<= SA_TargetFrameStateGood;
	USB_StatusRegister(5)				<= SA_Parked;
	USB_StatusRegister(6)				<= SA_Hold;
	USB_StatusRegister(7)				<= SA_ActuateEnabled;
	USB_StatusRegister(8)				<= SA_TargetReady;
	USB_StatusRegister(9)				<= C_I_mice_daq_ok;
	USB_StatusRegister(10)				<= SA_Actuating;
	USB_StatusRegister(11)				<= SA_CapActive;
	USB_StatusRegister(12)				<= SA_ActDataAvailable;
	USB_StatusRegister(13)				<= SA_NotifyAvailable;
	USB_StatusRegister(14)				<= SA_FatalError;
	USB_StatusRegister(18 downto 15)	<= SA_CurrentState;
	USB_StatusRegister(22 downto 19)	<= SA_ErrorCode;
	USB_StatusRegister(31 downto 23) <= SA_quad_position;

	SA_EngineerReg			<= USB_Engineering(4 downto 0);
	
	
	-- sl_StayAlive_ON_OFF 
	-- 0 = On and 1 = Off
	sl_StayAlive_ON_OFF	<= USB_Engineering(31);
	SA_SetPCError <= (not sl_StayAlive_ON_OFF) and sl_PC_Out;
	
	
	-- Frame Raised BPS:  Off = 0 (Default) On = 1
	-- When Off the BPS system relays are permanently closed when the frame is raised
	-- (This prevents BPS trips when the frame is raised)
	-- sl_FrameRaisedBPS_ON_OFF	<= USB_Engineering(30); 
	sl_FrameRaisedBPS_ON_OFF	<= '1'; 
	
	-- USB Engineering bits 29 & 28 control which CH monitor
	-- is fed to bits 31 down to 16 of the notify register.
	-- 00 No Channel selected
	-- 01 CHA selected 
	-- 10 CHB selected
	-- 11 INDX selected
	
	process(USB_Engineering(29 downto 28), SA_ChA_mon_value, SA_ChB_mon_value,
		SA_Indx_mon_value)	
	begin
	
		slv_ChMonSelect <= USB_Engineering(29 downto 28);
		
		case(slv_ChMonSelect) is	
			when "00" =>
				USB_NotifyRegister(31 downto 16) <= (others => '0');
			when "01" =>
				USB_NotifyRegister(31 downto 16) <= SA_ChA_mon_value;
			when "10" =>
				USB_NotifyRegister(31 downto 16) <= SA_ChB_mon_value;
			when others =>
				USB_NotifyRegister(31 downto 16) <= SA_Indx_mon_value;
		end case;
	end process;

	process(sl_DG_frame_raised, sl_DG_frame_lowered)
	begin
	-- Modified by ed to use DGMore signals.
	-- for even more deglitchification!
		--if (sl_DG_frame_raised = '1' and sl_DG_frame_lowered = '1') then
	if (sl_DG_More_frame_raised = '1' and sl_DG_More_frame_lowered = '1') then
			SA_SetFrameNOTRaisedError2 <= '1';
		else
			SA_SetFrameNOTRaisedError2 <= '0';
		end if;
	end process;
	
	-- USB Engineering bits 27 control whether the BPS is disabled
	-- Bit set to '1 to disable BPS
	-- Bit set to '0' to enable BPS
	--sl_BPSDisabled <= USB_Engineering(27);	
	-- MUST USE THIS IN R78 and on ISIS Target!!!!
	sl_BPSDisabled <= '0';
	
	-- USB Engineering bit 26 controls which ROM table is used (0 = old, 1 = new)
	-- Allows new ROMS to be easily tested and compared with a reference ROM
	sl_ROMSelect <= USB_Engineering(26);
	
	-- the channel monitor triggers are triggered by an actuation or 
	-- when the target is moved from park/hold to hold/park
	-- This is unlikely to give an answer for the index channel as
	-- the channels are only monitored for a few ms.
	-- These values are only available from the terminal app
	-- i.e. are not implemented in the GUI
	sl_monitor_trigger <= (SA_move_to_hold_trigger or SA_actuate_trigger 
									or SA_move_to_park_trigger);
	
	process(sl_OpticsMode, SA_DeGlitchedCHA, SA_DeGlitchedCHB, 
		SA_DeGlitchedIndex, sl_actuating_50r_2, sl_actuating_50r_3,
		sl_actuating_trig_50r_2)
	begin
	
		if (sl_OpticsMode = '1') then
			C_O_actuating_50r_2			<= SA_DeGlitchedCHA;
			C_O_actuating_50r_3			<= SA_DeGlitchedCHB;
			C_O_actuating_trig_50r_2	<= SA_DeGlitchedIndex;
		else
			C_O_actuating_50r_2			<= sl_actuating_50r_2;
			C_O_actuating_50r_3			<= sl_actuating_50r_3;
			C_O_actuating_trig_50r_2 	<= sl_actuating_trig_50r_2;		
		end if;
	end process;
	
	
	
	-- USB Reset Register
	
	-- ERRORS

	-- 1 - WDT Error				Bit 0
	-- 2 - DD Error				Bit 1
	-- 3 - Index Error			Bit 2
	-- 4 - Key Enable Error		Bit 3
	-- 5 - USB nc Error			Bit 4
	-- 6 - 5V Int Error			Bit 5
	-- 7 - 24V Int Error			Bit 6
	-- 8 - AE Error				RESET WITH ACTUATE DISABLE
	-- 9 - BPS Error				Bit 7
	-- 10 - PC Error				Bit 8
	-- 11 - CAP Error				Bit 9
	-- 12 - QuadADC Error		Bit 10
	-- 13 - QuadContinuity Err Bit 11
	-- 14 - FrameNotRaisedBPS	Bit 12
	
	-- BPS ERRORS - bit mapping to reset the errors!
	-- 1 - SP1 Time too long Error 		Bit 16
	-- 2 - SP1 Time too short Error 		Bit 17
	-- 3 - Min Time too long Error 		Bit 18
	-- 4 - Min Time too short Error 		Bit 19
	-- 5 - Min Pos too long Error 		Bit 20
	-- 6 - Min Pos too short Error 		Bit 21
	-- 7 - Act Time too long Error 		Bit 22
	-- 8 - Act Time too short Error 		Bit 23
	
	--
	--SA_USBFatalResets	<= USB_ResetRegister(11 downto 0);
	--
	--SA_USBBPSResets 	<= USB_ResetRegister(23 downto 16);
	--
	SA_USBFatalResets(0)		<= (USB_ResetRegister(0) and sl_Enable_Error_Reset_1); -- 1 - WDT Error
	SA_USBFatalResets(1)		<= (USB_ResetRegister(1) and sl_Enable_Error_Reset_1); -- 2 - DD Error	
	SA_USBFatalResets(2)		<= (USB_ResetRegister(2) and sl_Enable_Error_Reset_1); -- 3 - Index Error	
	SA_USBFatalResets(3)		<= (USB_ResetRegister(3) and sl_Enable_Error_Reset_1); -- 4 - Key Enable Error
	SA_USBFatalResets(4)		<= (USB_ResetRegister(4) and sl_Enable_Error_Reset_1); -- 5 - USB nc Error
	SA_USBFatalResets(5)		<= (USB_ResetRegister(5) and sl_Enable_Error_Reset_1); -- 6 - 5V Int Error	
	SA_USBFatalResets(6)		<= (USB_ResetRegister(6) and sl_Enable_Error_Reset_1); -- 7 - 24V Int Error	
	-- BPS Error is different
	SA_USBFatalResets(7)		<= (USB_ResetRegister(7) and sl_Enable_Error_Reset_2); -- 9 - BPS Error	
	--
	SA_USBFatalResets(8)		<= (USB_ResetRegister(8)  and sl_Enable_Error_Reset_1); -- 10 - PC Error	
	SA_USBFatalResets(9)		<= (USB_ResetRegister(9)  and sl_Enable_Error_Reset_1); -- 11 - CAP Error		
	SA_USBFatalResets(10)	<= (USB_ResetRegister(10) and sl_Enable_Error_Reset_1); -- 12 - QuadADC Error
	SA_USBFatalResets(11)	<= (USB_ResetRegister(11) and sl_Enable_Error_Reset_1); -- 13 - QuadContinuity Err
	SA_USBFatalResets(12)	<= (USB_ResetRegister(12) and sl_Enable_Error_Reset_1); -- 14 - FrameNOTRaisedBPSError
	
	SA_USBBPSResets(0) 	<= (USB_ResetRegister(16) and sl_Enable_Error_Reset_2); -- 1 - SP1 Time too long Error
	SA_USBBPSResets(1) 	<= (USB_ResetRegister(17) and sl_Enable_Error_Reset_2); -- 2 - SP1 Time too short Error 
	SA_USBBPSResets(2) 	<= (USB_ResetRegister(18) and sl_Enable_Error_Reset_2); -- 3 - Min Time too long Error
	SA_USBBPSResets(3) 	<= (USB_ResetRegister(19) and sl_Enable_Error_Reset_2); -- 4 - Min Time too short Error
	SA_USBBPSResets(4) 	<= (USB_ResetRegister(20) and sl_Enable_Error_Reset_2); -- 5 - Min Pos too long Error 
	SA_USBBPSResets(5) 	<= (USB_ResetRegister(21) and sl_Enable_Error_Reset_2); -- 6 - Min Pos too short Error
	SA_USBBPSResets(6) 	<= (USB_ResetRegister(22) and sl_Enable_Error_Reset_2); -- 7 - Act Time too long Error 
	SA_USBBPSResets(7) 	<= (USB_ResetRegister(23) and sl_Enable_Error_Reset_2); -- 8 - Act Time too short Error 
	
	SA_USBAckActData  <= USB_ResetRegister(30);
	SA_USBAckNotify	<= USB_ResetRegister(31);
	
	-- Other Internal Assignments
	SA_GMS <= C_I_isis_ms;	
	SA_TargetFrameStateGood <= (sl_DG_frame_raised XOR sl_DG_frame_lowered);
	SA_WDT_MasterReset <= (SA_WDT_Reset OR SA_WDT_Reset2);
	USB_ActuationMinMax <= SA_ActuationMinMax;
	
	USB_BPSErrors <= SA_BPSErrors;
	
	SA_Error_LED <= SA_FatalError or (SA_BPSErrorState and SA_2Hz_square);
	
	--C_O_Frame_Enable_50r	<= SA_TargetFrameEnable;												
	--C_O_unused3				<= '0';

	-- Minimum Dip Depth
	SA_SP1MINIMUM			<= std_logic_vector(to_unsigned(190,9));
	
	-- SPI DAQ setup
	process(sl_EngPosTest, sl_EngPosTest, u_counter_now, SA_quad_position,
		sl_OpticsADCMode, SA_ChA_mon_value, slv_CHAlive_value)
	begin
		if (sl_EngPosTest = '0' and sl_OpticsADCMode = '0') then
			DAC_FIFODataIn(DAC_WIDTH-1 downto 16) <= (others => '0');
			
			-- Test to see if Monitor Value > 0
			--DAC_FIFODataIn(15 downto 0)	<= SA_ChA_mon_value;
			
			DAC_FIFODataIn(15 downto 7) 	<= SA_quad_position;
			DAC_FIFODataIn(6 downto 0) 	<= (others => '0');
			
		elsif (sl_EngPosTest = '0' and sl_OpticsADCMode = '1') then
		
			DAC_FIFODataIn(15 downto 0) 	<= slv_CHAlive_value;
		
		else
			DAC_FIFODataIn(DAC_WIDTH-1 downto 16) <= (others => '0');
			DAC_FIFODataIn(15 downto 0)<= std_logic_vector(u_counter_now);
		end if;
	end process;
	
	-- EPOT Setup
	ChA_EPOT_value_now  <= USB_ChannelGains(7 downto 0);
	ChB_EPOT_value_now  <= USB_ChannelGains(15 downto 8);
	Indx_EPOT_value_now <= USB_ChannelGains(23 downto 16);
	
	-- *******************************************************
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *              	  MAIN Processes						  	*
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *******************************************************
					
	-- CLOCKING PROCESS 
	process(int_sysclk, int_logic_reset)
	begin
		if (int_logic_reset = '1') then
			T_state_now <= State_Initialise;
			SA_IndexErrorState 	<= '0';
			SA_WDTErrorState 		<= '0';
			SA_DDErrorState 		<= '0';
			SA_USBncErrorState 	<= '0';
			SA_KEErrorState 		<= '0';
			SA_5VIntErrorState 	<= '0';
			SA_24VIntErrorState 	<= '0';
			SA_SP1LockedState 	<= (others => '0');
			
			SA_PCErrorState	<= '0';
			SA_CapErrorState	<= '0';
			SA_BPSErrorState	<= '0';
			
			SA_BPSErrorStateEnable	<= '0';
			SA_QuadADCErrorState		<= '0';
			SA_QuadContinuityErrorState	<= '0';
			
			SA_FrameNOTRaisedErrorState 	<= '0';	

		elsif (int_sysclk'event and int_sysclk = '1') then
			T_state_now 				<= T_state_next;
			SA_IndexErrorState 		<= SA_NextIndexErrorState;	
			SA_WDTErrorState 			<= SA_NextWDTErrorState;
			SA_DDErrorState 			<= SA_NextDDErrorState;
			SA_USBncErrorState 		<= SA_NextUSBncErrorState;
			SA_KEErrorState 			<= SA_NextKEErrorState;
			SA_5VIntErrorState 		<= SA_Next5VIntErrorState;
			SA_24VIntErrorState 		<= SA_Next24VIntErrorState;
			SA_SP1LockedState 		<= SA_NextSP1LockedState;
			
			SA_PCErrorState			<= SA_NextPCErrorState;
			SA_CapErrorState			<= SA_NextCapErrorState;
			SA_BPSErrorState			<= SA_NextBPSErrorState;
			SA_BPSErrorStateEnable	<= SA_NextBPSErrorStateEnable;
			SA_QuadADCErrorState		<= SA_NextQuadADCErrorState;
			SA_QuadContinuityErrorState	<= SA_NextQuadContinuityErrorState;
			SA_FrameNOTRaisedErrorState 	<= SA_NextFrameNOTRaisedErrorState;
			-- Following added Feb 11 to generate a single tick signal if position changes.
			SA_quad_position_prev	<= SA_quad_position;
			
			ChA_EPOT_value_prev		<= ChA_EPOT_value_now;
			ChB_EPOT_value_prev		<= ChB_EPOT_value_now;
			Indx_EPOT_value_prev		<= Indx_EPOT_value_now;

		end if;
	end process;
	
	-- Next State Logic & Data Path Functional Units/Routing
	process(T_state_now, SA_move_to_hold_in, SA_move_to_hold_complete, SA_move_to_park_in,
			  SA_move_to_park_complete, SA_CapComplete, SA_Actuate_complete, SA_DelayedGMS,
			  SA_IndexCheck, SA_DropDetect, SA_KeyEnabled, 
			  SA_Interlocks5V_ok, SA_Interlocks24V_ok, 
			  sl_DG_frame_raised, sl_DG_frame_lowered, 
			  SA_TargetFrameStateGood, SA_TargetFrameEnable,
			  C_I_mice_daq_ok, SA_EnableActuation, SA_WDT_Out,
			  SA_FatalError, SA_quad_direction,  
			  SA_HoldState1, SA_SetPoint1, SA_SP1LockedState, SA_USB_nc_DG, 
			  SA_FatalErrorReset, SA_SetBPSError, SA_ErrorCode, SA_BPSErrorState, 
			  SA_CaptureOverflowProt, SA_EnableEngineering, 
			  SA_2Hz_square, SA_Cap10sComplete, SA_Cap500usComplete,
			  SA_SetQuadContinuityError, sl_OpticsMode, SA_QuadADCErrorState,
			  sl_Initialise_Timer_Complete)
	
	begin
		--General Signals
		T_state_next <= T_state_now	;
		
		--  ** State Signals **
		SA_CurrentState   	<= "0000";
		
		-- 						"0000" No State Defined
		--							"0001" Parked
		--							"0010" Moving to Hold
		--							"0011" Moving to Park
		--							"0100" Hold Mode
		--							"0101" Actuate Enable Mode
		--							"0110" Actuating
		--							"0111" Capturing
		--							"1000" Kill
		--							"1001" MakeSafe
		--							"1010" Engineering
		-- 						"1011" BPS_Error
		
		--Initialise
		SA_FatalErrorReset		<= '0';
		
		SA_ResetQC					<= '0';
		SA_TargetFrameEnable 	<= '0';
		
		SA_Hold						<= '0';
		SA_Parked					<= '0';
		
		SA_move_to_hold_trigger <= '0';
		SA_move_to_park_trigger <= '0';
		
		SA_hold_select 			<= '0';	
		SA_actuate_select 		<= '0';
		
		SA_actuate_trigger 		<= '0';
		SA_killActuation			<= '0';
		
		SA_TargetReady    		<= '0';
		
		SA_IndexEnable				<= SA_quad_direction;
		SA_IndexDirection 		<= '1';
		SA_IndexDirection 		<= '1';
		
		SA_PWMFullOn				<= '0';
		SA_PWMenable 				<= '0';
		SA_PWMSynchronise 		<= '0';
		
		SA_HoldSwitch1				<= '0';				
		SA_HoldSwitch2				<= '0';

		SA_StartCapture 	 		<= '0';
		SA_KillCapture				<= '0';
		
		SA_WDT_Reset				<= '0';
		SA_WDT_Trigger 			<= '0';
		
		SA_ResetIndexLatch	   <= '0';
		
		SA_DropDetectEnable  	<= '0';

		SA_TriggerEnable			<= '0';
		SA_ResetActCount			<= '0';
		
		SA_SetIndexError			<= '0';
		SA_SetParkNotify			<= '0';	
		SA_SetWDTError				<= '0';
		
		SA_SetInitOKNotify		<= '0';
		
		SA_NextSP1LockedState	<= SA_SetPoint1;
		
		SA_CheckLimits 			<= '0';
		SA_AquireActData2 		<= '0';
		
		SA_CaptureStartProt		<= '0';
		SA_CaptureStopProt		<= '0';
		
		SA_Cap10sStart				<= '0';
	   SA_Cap500usStart			<= '0';
		
		sl_parking					<= '0';
		
		sl_Enable_Error_Reset_1	<= '0';
		sl_Enable_Error_Reset_2	<= '0';
		
		SA_SetFrameNOTRaisedError1 <= '0';
		sl_Start_Initialise_Timer	<= '0';
					
		case T_state_now is
		
			-- ***************************************************
			-- System Initialisation on Power Up
			when State_Initialise =>
			-- ***************************************************
				-- System Initialisation Procedures Will go Here
				
				SA_ResetQC <= '1';				-- Set the Start Position on the Quadrature Counter
				SA_ResetIndexLatch <= '1';		-- Reset the Index Checker so that it can be checked when target is in hold
				SA_FatalErrorReset <= '1';		-- Ensure Fatal Errors are Cleared
				
				-- A short pause so that signals have chance to propogate
				-- through the deglitching circuits
				sl_Start_Initialise_Timer <= '1';
				T_state_next <= State_Initialise2;
				
			-- ***************************************************
			-- System Initialisation on Power Up
			when State_Initialise2 =>
			-- ***************************************************
				T_state_next <= State_Initialise2;
				
				-- A short pause so that signals have chance to propogate
				-- through the deglitching circuits
				if (sl_Initialise_Timer_Complete = '1') then
					T_state_next <= State_Parked;
					--Then once that this has been completed then:
					SA_SetInitOKNotify <= '1';
				end if;
				
			
			-- ***************************************************
			when State_Parked =>
			-- ***************************************************
			-- The system is now waiting for the hold signal
			
				T_state_next <= State_Parked;
				SA_Parked <= '1';
				SA_CurrentState   	<= "0001";
				
				-- Target Frame must be raised for Errors to be reset
				-- Note that this is catch-all because for any fatal error
				-- the system will move to state kill anyway...
				if (sl_DG_frame_raised = '1') then
					sl_Enable_Error_Reset_1	<= '1';
					sl_Enable_Error_Reset_2	<= '1';
				end if;
				
				-- Frame NOT raised error when target is parked or MTH.
				if (sl_DG_frame_raised = '0' or SA_TargetFrameStateGood = '0') then
					SA_SetFrameNOTRaisedError1 <= '1';
				end if;
				
				if (SA_FatalError = '1') then
					-- Force User to reset Fatal Errors
					T_state_next <= State_Kill;
				elsif (SA_EnableEngineering = '1') then
					T_state_next <= State_Engineering;
				-- 10/05/2012 - Added SA_TargetFrameStateGood before target can be moved!
				elsif (SA_KeyEnabled = '1' AND SA_USB_nc_DG = '1' and SA_EnableEngineering = '0'
					and SA_TargetFrameStateGood = '1') then
					if	(SA_move_to_park_in = '0' and SA_move_to_hold_in = '1') then
						T_state_next <= State_Moving_to_Hold_1;
						SA_ResetQC	  <= '1';
					end if;
				end if;
				
			-- ***************************************************	
			when State_Moving_to_Hold_1 =>
			-- ***************************************************
			-- Stator is moved from unpowered state to hold state
			
				-- Frame NOT raised error when target is parked or MTH.
				if (sl_DG_frame_raised = '0' or SA_TargetFrameStateGood = '0') then
					SA_SetFrameNOTRaisedError1 <= '1';
				end if;
			
				-- Trigger the Move to Hold State Machine
				SA_move_to_hold_trigger <= '1';
				SA_HoldSwitch1 			<= '1';
				SA_hold_select 			<= '1';
				SA_ResetIndexLatch  		<= '1';
				SA_PWMenable 				<= '1';
				SA_WDT_Reset 				<= '1';
				SA_resetActCount 			<= '1';
				SA_CurrentState   		<= "0010";
				-- Move to next State
				T_state_next <= State_Moving_to_Hold_2;
				
			-- ***************************************************	
			when State_Moving_to_Hold_2 =>
			-- ***************************************************
			
				-- Frame NOT raised error when target is parked or MTH.
				if (sl_DG_frame_raised = '0' or SA_TargetFrameStateGood = '0') then
					SA_SetFrameNOTRaisedError1 <= '1';
				end if;
			
			-- Wait until Move to Hold State Machine has finished			
				SA_move_to_hold_trigger <= '0';
				SA_hold_select 			<= '1';
				SA_HoldSwitch1 			<= '1';
				SA_PWMenable 				<= '1';
				SA_CurrentState   		<= "0010";
				if (SA_move_to_hold_complete) = '1' then
					T_state_next 	<= State_Moving_To_Hold_3;
				else
					T_state_next 	<= State_Moving_To_Hold_2;
				end if;
				
			-- ***************************************************
			when State_Moving_to_Hold_3 =>
			-- ***************************************************
			
				-- Frame NOT raised error when target is parked or MTH.
				if (sl_DG_frame_raised = '0' or SA_TargetFrameStateGood = '0') then
					SA_SetFrameNOTRaisedError1 <= '1';
				end if;
			
				SA_move_to_hold_trigger <= '0';
				SA_hold_select 			<= '1';
				SA_HoldSwitch1 			<= '1';
				SA_PWMenable 				<= '1';
				SA_CurrentState   		<= "0010";
				T_state_next <= State_HoldMode;
				
				--T_state_next <= State_MakeSafe1;
			
				if (SA_IndexCheck = '0') then
					SA_SetIndexError 	<= '1';
					T_state_next <= State_MakeSafe1;
				elsif (SA_QuadADCErrorState = '1') then
					T_state_next <= State_MakeSafe1;
				end if;
			
			-- ***************************************************
			when State_HoldMode =>
			-- ***************************************************
				
				-- Only turn on Drop Enable if NOT in optics mode
				if (sl_OpticsMode = '0') then
					SA_DropDetectEnable  <= '1';
				end if;
				SA_hold_select 		<= '1';
				SA_HoldSwitch1 		<= '1';
				SA_Hold					<= '1';				
				SA_TargetFrameEnable <= '1';
				SA_PWMenable 			<= '1';
				SA_CurrentState		<= "0100";
				T_state_next <= State_HoldMode;
				
				--This is done after every actuation??
				if (SA_IndexCheck = '0') Then
					SA_SetIndexError 	<= '1';
				end if;
		
				if  (SA_FatalError = '0') then			
					if	((SA_move_to_park_in = '1') and (SA_move_to_hold_in = '0')) then
						if(sl_DG_frame_raised = '1') then
							T_state_next <= State_Moving_to_Park_1;
						else
							SA_SetParkNotify	<= '1';	
						end if;
					end if;	
				
					if ((SA_EnableActuation = '1') and (SA_TargetFrameStateGood = '1')) then	
						SA_TargetReady   <= '1';
						T_state_next <= State_ActuateEnableMode;
					end if;	
									
				else
				
					if (SA_DropDetect = '1') Then					
						T_state_next <= State_Kill;		
					else
						-- For any other error in Hold Mode try and
						-- make the system safe
						T_state_next <= State_MakeSafe1;			
					end if;
				
				end if;
				
			-- ***************************************************	
			when State_ActuateEnableMode =>
			-- ***************************************************
				
				SA_DropDetectEnable  <= '1';
				SA_hold_select 		<= '1';
				SA_HoldSwitch1 		<= '1';
				SA_Hold					<= '1';
				SA_TargetReady	    	<= '1';
				SA_TriggerEnable		<= '1';
				SA_TargetFrameEnable <= '1';
				SA_PWMenable 			<= '1';
				T_state_next  			<= State_HoldMode;
				SA_CurrentState		<= "0101";
				
				--This is done after every actuation??
				if (SA_IndexCheck = '0') then
					SA_SetIndexError 	<= '1';
				end if;
					
				if  (SA_FatalError = '0') Then
					
					if ((SA_EnableActuation) = '1' and (SA_TargetFrameStateGood = '1')) then	
						if ((SA_DelayedGMS = '1') and (C_I_mice_daq_ok = MICE_DAQ_OK_VALUE)) then
							T_state_next <= State_Actuate;
						else	
							T_state_next <= State_ActuateEnableMode;
						end If;
					end if;

				else
				
					if (SA_DropDetect = '1') Then
						T_state_next <= State_Kill;
					else
						-- For any other error in Hold Mode try and
						-- make the system safe
						T_state_next <= State_MakeSafe1;
					end if;
					
				end if;
				
			-- ***************************************************				
			when State_Moving_to_Park_1 =>
			-- ***************************************************
			
				SA_hold_select 	<= '1';
				SA_HoldSwitch1 	<= '1';
				SA_PWMenable 		<= '1';
				SA_CurrentState   <= "0011";
				-- Trigger the Move to Hold State Machine	
				SA_move_to_park_trigger <= '1';
				-- Move to next State
				T_state_next <= State_Moving_to_Park_2;
			
			-- ***************************************************
			when State_Moving_to_Park_2 =>
			-- ***************************************************
			
				sl_parking			<= '1';
				
				SA_hold_select 	<= '1';
				SA_HoldSwitch1 	<= '1';
				SA_PWMenable 		<= '1';
				SA_CurrentState   <= "0011";
				if SA_move_to_park_complete = '1' then
					T_state_next <= State_Parked;
				else
					T_state_next <= State_Moving_to_Park_2;
				end if;
				
			-- ***************************************************
			when State_Actuate =>
			-- ***************************************************			
				-- We Commit to an actuation at this point whatever
				SA_TargetFrameEnable 	<= '1';
				SA_actuate_select 		<= '1';
				SA_HoldSwitch1 			<= '1';
				SA_actuate_trigger 		<= '1';
				SA_WDT_Reset  				<= '1';
				SA_PWMenable 				<= '1';
				SA_PWMFullOn				<= '1';
				SA_ResetIndexLatch		<= '1';
				SA_IndexEnable				<= SA_quad_direction;
				SA_IndexDirection 		<= '1';
				SA_CurrentState			<= "0110";
				T_state_next 				<= State_Actuate2;
				SA_NextSP1LockedState 	<= SA_SP1LockedState;
				
			-- ***************************************************	
			when State_Actuate2 =>
			-- ***************************************************
				SA_TargetFrameEnable 	<= '1';
				SA_actuate_select 		<= '1';
				SA_HoldSwitch1 			<= '1';
				SA_WDT_Trigger 			<= '1';
				SA_IndexEnable				<= SA_quad_direction;
				SA_IndexDirection 		<= '1';
				SA_CurrentState			<= "0110";
				SA_NextSP1LockedState 	<= SA_SP1LockedState;
				
				-- IF WDT has not timed out (WDT='1')
				if (SA_WDT_Out = '1' and SA_SetQuadContinuityError = '0') Then
					SA_PWMFullOn		<= '1';
					SA_PWMenable 		<= '1';		
					if SA_actuate_complete = '1' then
						T_state_next <= State_Capture1;
					else
						T_state_next <= State_Actuate2;
					end if;
				else
					if (SA_WDT_Out = '0') then
						SA_SetWDTError 	<= '1';
					end if;
					 -- This resets the actuation FSM
					SA_killActuation	<= '1';
					T_state_next 		<= State_Actuate3;
					-- Checks Actuation Limits, may give additional info
					SA_CheckLimits 	<= '1'; 
				end if;
			
			-- ***************************************************
			when State_Actuate3 =>
			-- ***************************************************
				-- This state was added because it will take a clock cycle
				-- to Check Limits
				SA_AquireActData2 <= '1'; -- Add any actuation data to fifo
				T_state_next 		<= State_Kill;
			
			-- ***************************************************
			when State_Capture1 =>
			-- ***************************************************
				SA_PWMenable 				<= '1';
				SA_TargetFrameEnable 	<= '1';
				SA_DropDetectEnable  	<= '1';
				SA_StartCapture 			<= '1';
				SA_CurrentState			<= "0111";
				SA_NextSP1LockedState 	<= SA_SP1LockedState;
				SA_CheckLimits 			<= '1'; -- Checks Actuation Limits	
				SA_CaptureStartProt		<= '1'; -- Added a Capture Watchdog
				T_state_next 				<= State_Capture2;
		
				-- Start 10s Timer
				SA_Cap10sStart				<= '1';
	   
			-- ***************************************************	
			when State_Capture2 =>
			-- ***************************************************
				SA_PWMenable 				<= '1';
				SA_TargetFrameEnable 	<= '1';
				SA_hold_select 			<= '1';
				SA_HoldSwitch2 			<= '1';
				SA_DropDetectEnable  	<= '1';
				SA_CurrentState			<= "0111"; -- State 7
				SA_NextSP1LockedState 	<= SA_SP1LockedState;	
				T_state_next 				<= State_Capture2;
				
				if (SA_DropDetect = '1') then
					SA_PWMenable 			<= '0';
					SA_TargetFrameEnable <= '0';
					SA_KillCapture 		<= '1';
					SA_CheckLimits 		<= '1'; -- Checks Actuation Limits, may give additional info
					T_state_next 			<= State_Kill;
					-- Added 16/06/2012 - Possible missing line...
					SA_CaptureStopProt 	<= '1';
					
				-- SA_CaptureOverflowProt times out after 1 minute...	
				elsif (SA_CaptureOverflowProt = '1') then
					SA_CaptureStopProt <= '1';
					-- Target appears to be stuck, however
					-- ensure that the frame is raised 
					-- before powering down.
					T_state_next <= State_MakeSafe3;
					
				elsif (SA_CapComplete = '1') then
					SA_CaptureStopProt <= '1';
						
					-- Check for BPS Error
					-- If **ONLY** a BPS Error then place in Hold Mode
					if (SA_ErrorCode = "1011") then -- 11
						
						T_state_next <= State_BPSError;
					
					-- If there is NO error then
					elsif(SA_ErrorCode = "0000") then
					
						if(SA_EnableActuation = '1') then
							T_state_next <= State_ActuateEnableMode;
						else
							T_state_next <= State_HoldMode ;
						end if;
					else
						-- There is another type of Error so
						T_state_next <= State_MakeSafe1;
					end if;
					
				--(if 10s timer is complete then target may be stuck so
				-- start 500us timer and go to state
				-- CaptureAdjustment1 to give the target an active 'nudge')
				elsif  (SA_Cap10sComplete = '1') then
					T_state_next 	<= State_CaptureAdjustment1;
				end if;			
			
			-- ***************************************************
			when State_CaptureAdjustment1 =>	
			-- ***************************************************
			
			-- This bumps the capture current up to full current for 0.5ms
				SA_PWMenable 	<= '1';
				
				-- Note this turns The Current to MAX!!
				SA_PWMFullOn	<= '1';
				
				SA_TargetFrameEnable 	<= '1';
				SA_hold_select 			<= '1';
				SA_HoldSwitch2 			<= '1';
				SA_DropDetectEnable  	<= '1';
				SA_CurrentState			<= "0111"; -- State 7
				SA_NextSP1LockedState 	<= SA_SP1LockedState;	
				T_state_next 				<= State_CaptureAdjustment2;
				
				-- Added this line here to prevent some strange combinatorial loop
				-- Should have been able to add at the end of Capture2 ???
				SA_Cap500usStart	<= '1';
				
				if (SA_DropDetect = '1') then
					SA_PWMenable 			<= '0';
					SA_TargetFrameEnable <= '0';
					SA_KillCapture 		<= '1';
					-- Checks Actuation Limits, may give additional info
					SA_CheckLimits 		<= '1';
					SA_CaptureStopProt 	<= '1';					
					T_state_next 			<= State_Kill;	
					
				elsif (SA_CaptureOverflowProt = '1') then
					SA_CaptureStopProt <= '1';
					T_state_next <= State_MakeSafe3;
					
				elsif (SA_CapComplete = '1') then
					SA_CaptureStopProt <= '1';
						
					-- Check for BPS Error
					-- If **ONLY** a BPS Error then hold target.
					if (SA_ErrorCode = "1011") then -- 11
						
						T_state_next <= State_BPSError;
					
					-- If there is NO error then
					elsif(SA_ErrorCode = "0000") then
					
						if(SA_EnableActuation = '1') then
							T_state_next <= State_ActuateEnableMode;
						else
							T_state_next <= State_HoldMode ;
						end if;
					else
						-- There is another type of Error so
						T_state_next <= State_MakeSafe1;
					end if;
				
				end if;
				
			-- ***************************************************
			when State_CaptureAdjustment2 =>	
			-- ***************************************************
				-- Capture current is up to full current for 0.5ms
				-- to try and free the target if it appears stuck
			
				SA_PWMenable 		<= '1';
				
				-- Note this turns The Current to MAX!!
				SA_PWMFullOn		<= '1';
				
				SA_TargetFrameEnable 	<= '1';
				SA_hold_select 			<= '1';
				SA_HoldSwitch2 			<= '1';
				SA_DropDetectEnable  	<= '1';
				SA_CurrentState			<= "0111"; -- State 7
				SA_NextSP1LockedState 	<= SA_SP1LockedState;	
				T_state_next 				<= State_CaptureAdjustment2;
				
				if (SA_DropDetect = '1') then
					SA_PWMenable 			<= '0';
					SA_TargetFrameEnable <= '0';
					SA_KillCapture 		<= '1';
					-- Checks Actuation Limits, may give additional info
					SA_CheckLimits 		<= '1'; 
					T_state_next 			<= State_Kill;
					SA_CaptureStopProt <= '1';					
					
				elsif (SA_CaptureOverflowProt = '1') then
					SA_CaptureStopProt <= '1';
					T_state_next <= State_MakeSafe3;
					
				elsif (SA_CapComplete = '1') then
					SA_CaptureStopProt <= '1';
						
					-- Check for BPS Error
					-- If **ONLY** a BPS Error then hold target
					if (SA_ErrorCode = "1011") then -- 11
			
						T_state_next <= State_BPSError;
					
					-- If there is NO error then
					elsif(SA_ErrorCode = "0000") then
					
						if(SA_EnableActuation = '1') then
							T_state_next <= State_ActuateEnableMode;
						else
							T_state_next <= State_HoldMode ;
						end if;
					else
						-- There is another type of Error so
						T_state_next <= State_MakeSafe1;
					end if;
					
				--(0.5 ms timer is complete then start 10s timer and return to state
				-- Capture 2)
				elsif  (SA_Cap500usComplete = '1') then
					SA_Cap10sStart	<= '1';
					T_state_next 	<= State_Capture2;				
				end if;
				
			-- ***************************************************
			when State_Kill =>
			-- ***************************************************
				SA_CurrentState		<= "1000";
				T_state_next 			<= State_Kill;
				
				SA_TargetFrameEnable <= '0';			
				SA_KillCapture 		<= '1';
				
				-- Target should now remain dead until:
				-- Target Frame Position is Raised
				
				-- Indicate that the target is parked (which technically it is 
				-- when it has been killed!) 04/11/2011
				SA_Parked <= '1';
				
				if (SA_EnableEngineering = '1') then
					T_state_next <= State_Engineering;
					
				-- Note SA_EnableActuation incorporates Fatal_Errors & BPS Errors
				elsif (sl_DG_frame_raised = '1') and (sl_DG_frame_lowered = '0') then				 
					 T_state_next <= State_Kill_2;
				end if;
				
			-- ***************************************************
			when State_Kill_2 =>
			-- ***************************************************
				SA_CurrentState		<= "1000";
				T_state_next 			<= State_Kill;
				
				SA_TargetFrameEnable <= '0';			
				SA_KillCapture 		<= '1';
				
				sl_Enable_Error_Reset_1	<= '1';
				sl_Enable_Error_Reset_2	<= '1';
				
				-- Target should now remain dead until:
				-- Actuate Enable signal has been removed
				-- Target Frame Position is Raised
				-- Fatal Errors Have Been Cleared
				
				-- The System can then be restarted.
				-- Note Engineering Menu options can be accessed from
				-- State Kill
				
				-- Indicate that the target is parked (which technically it is 
				-- when it has been killed!) 04/11/2011
				SA_Parked <= '1';
				
				if (SA_EnableEngineering = '1') then
					T_state_next <= State_Engineering;
					
				-- Note SA_EnableActuation incorporates Fatal_Errors & BPS Errors
				elsif ((SA_EnableActuation = '0') and (sl_DG_frame_raised = '1') and
					 (sl_DG_frame_lowered = '0') and (SA_FatalError = '0')) then				 
					 T_state_next <= State_Initialise;
				end if;
				
			-- ***************************************************
			when State_MakeSafe1 =>
			-- ***************************************************
				-- This State is designed to power down the controller
				-- In a Controlled Manner
				-- This will Happen If:
				
				-- 1) The Enable Key is turned Off
				-- 2) USB Connection is Lost
				-- 3) PC Counter Time Out
				
				-- Also:
				-- 1) Any error other than DD when in Hold Mode/Actuate Enable
				
				SA_hold_select 		<= '1';
				SA_HoldSwitch1 		<= '1';
				SA_Hold					<= '1';	--LED			
				
				SA_PWMenable 			<= '1';
				
				SA_CurrentState		<= "1001"; 
				SA_TargetFrameEnable <= '0';
				
				T_state_next <= State_MakeSafe1;
				
				-- First: Is the Target Frame in its Raised Position?
				-- If Not Wait Until It is (This will happen automatically)
				if (sl_DG_frame_raised = '1' and sl_DG_frame_lowered = '0') then
					T_state_next <= State_MakeSafe2;
				end if;
				
			-- ***************************************************	
			when State_MakeSafe2 =>
			-- ***************************************************
				SA_hold_select 	<= '1';
				SA_HoldSwitch1 	<= '1';
				SA_PWMenable 		<= '1';
				
				-- T_state_next <= State_Parked;
				-- Changed to State Kill so that the error needs 
				-- to be acknowledged.
				T_state_next 		<= State_Kill;
				SA_CurrentState	<= "1001"; 
				-- If the stator needs placing in park mode then do this
				if (SA_HoldState1 /= "00000") then
						T_state_next <= State_Moving_to_Park_1;
				end if;
				
			-- ***************************************************	
			when State_MakeSafe3 =>
			-- ***************************************************
				-- This State is designed to turn power off to the 
				-- controller once the frame has been raised
				-- No attempt to park the target will be made.
				-- This will Happen If:
				
				-- 1) Capture Error
				
				SA_PWMenable 				<= '1';
				SA_TargetFrameEnable 	<= '0';
				SA_hold_select 			<= '1';
				SA_HoldSwitch2 			<= '1';
				SA_DropDetectEnable  	<= '1';
				SA_NextSP1LockedState 	<= SA_SP1LockedState;
				
				
				SA_CurrentState		<= "1001"; 
				SA_TargetFrameEnable <= '0';
				T_state_next <= State_MakeSafe3;
				
				-- First: Is the Target Frame in its Raised Position
				-- If Not Wait Until It is (This will happen automatically)
				if (sl_DG_frame_raised = '1' and sl_DG_frame_lowered = '0') then
					T_state_next <= State_Kill;
				end if;
				
			-- ***************************************************	
			when State_Engineering =>
			-- ***************************************************
				SA_CurrentState		<= "1010";
				sl_Enable_Error_Reset_1	<= '1';
				sl_Enable_Error_Reset_2	<= '1';
				
				if SA_EnableEngineering = '0' then
					T_state_next <= State_Kill;
				end if;
	
			-- ***************************************************	
			when State_BPSError =>
			-- ***************************************************
				-- This state is for actuation based BPS error only
				-- If there are additional errors of higher priority (i.e. DD etc)
				-- Then state will goto MakeSafe1, MakeSafe3 or Kill
				
				sl_Enable_Error_Reset_2	<= '1';
			
				SA_DropDetectEnable  <= '1';
				SA_hold_select 		<= '1';
				SA_HoldSwitch1 		<= '1';
				SA_Hold					<= '1';				
				SA_TargetFrameEnable <= '1';
				SA_PWMenable 			<= '1';
				SA_CurrentState		<= "1011";
				T_state_next 			<= State_BPSError;
				
				
				--21/05/2011
				-- Simplify this process by just parking the target if there is a 
				-- BPS error
				-- This makes this state redundant but will leave state in because 
				
				-- 1) there would be a lot of changes to code to remove state
				-- 2) it is easier to reimplement if I wish to reinstate this state later
				
				
				if (SA_FatalError = '1') then
					if (SA_DropDetect = '1') then
						T_state_next <= State_Kill;									
					else
						T_state_next <= State_MakeSafe1;
					end if;
				else
					T_state_next <= State_MakeSafe1;
				end if;
				
				
--				T_state_next 			<= State_BPSError;
--				
--				-- "1000" = AE Error, 
--				-- "1011" = BPS Error. This is NOT a fatal error on its own.
--				
--				-- However the BPS error will cause an AE error which is
--				-- a fatal error
--				-- This is necessary to prevent automatic restarting of target when error has
--				-- been cleared.
--				
--				if (SA_FatalError = '1') then
--					if (SA_DropDetect = '1') then
--						T_state_next <= State_Kill;									
--					elsif ((SA_ErrorCode /= "1000") or (SA_ErrorCode /= "1011")) then			
--						T_state_next <= State_MakeSafe1;		
--					end if;
--					
--				-- Only move to hold mode if there are no fatal errors 
--				-- and BPS errors and Enable Actuation = 0
--				elsif ((SA_BPSErrorState = '0') and (SA_EnableActuation = '0')) then
--					T_state_next <= State_HoldMode;
--				end if;
				
		end case;
	end process;
	
	-- *******************************************************
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *              	  Error Processes						  	*
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *                                                     *
	-- *******************************************************
	
	-- Index Error
	process(SA_SetIndexError, SA_ResetIndexError, SA_IndexErrorState)
	begin	
		SA_NextIndexErrorState <= SA_IndexErrorState;
	
		if (SA_SetIndexError = '1' and SA_ResetIndexError = '0') then
				SA_NextIndexErrorState <= '1';
		elsif (SA_SetIndexError = '0' and SA_ResetIndexError = '1') then
				SA_NextIndexErrorState <= '0';
		end If;
	end process;
	
	process(SA_SetWDTError, SA_ResetWDTError, SA_WDTErrorState)
	begin
	
		SA_NextWDTErrorState <= SA_WDTErrorState;
		SA_WDT_Reset2 <= '0'; -- This resets the WDT after a WDT error has occurred!
		
		if (SA_SetWDTError = '1' and SA_ResetWDTError = '0') then
				SA_NextWDTErrorState <= '1';
				
				SA_WDT_Reset2 <= '0'; -- This must also be reset
		elsif (SA_SetWDTError = '0' and SA_ResetWDTError = '1') then
				SA_NextWDTErrorState <= '0';
				SA_WDT_Reset2 <= '1';
		end if;
	
	end process;
	
	SA_SetDDError <= SA_DropDetect;
	process(SA_SetDDError, SA_ResetDDError, SA_DDErrorState)
	begin	
		SA_NextDDErrorState <= SA_DDErrorState;
	
		if (SA_SetDDError = '1' and SA_ResetDDError = '0') then
				SA_NextDDErrorState <= '1';
		elsif (SA_SetDDError = '0' and SA_ResetDDError = '1') then
				SA_NextDDErrorState <= '0';
		end If;
		
	end process;
	
	
	SA_SetUSBncError <= SA_USBnc_Tick;
	process(SA_SetUSBncError, SA_ResetUSBncError, SA_USBncErrorState)
	begin	
		SA_NextUSBncErrorState <= SA_USBncErrorState;
	
		if (SA_SetUSBncError = '1' and SA_ResetUSBncError = '0') then
				SA_NextUSBncErrorState <= '1';
		elsif (SA_SetUSBncError = '0' and SA_ResetUSBncError = '1') then
				SA_NextUSBncErrorState <= '0';
		end If;
		
	end process;
	
	-- EnableActuation is only valid during
	-- Hold Mode, ActuateEnable, Actuate, Capture, Engineering 
	
	--							"0100" Hold Mode
	--							"0101" Actuate Enable Mode
	--							"0110" Actuating
	--							"0111" Capturing
	--							"1010" Engineering
	
	process(SA_CurrentState, SA_EnableActuation, SA_2Hz_square)
	begin
		SA_ActuateEnabled  		<= '0';
		SA_ActuateEnabledLED		<= '0';
		SA_ActuateEnableError 	<= '0';  -- This goes into the fatal_error_entity
		if (SA_EnableActuation = '1') then
			if ((SA_CurrentState = "0100") or (SA_CurrentState = "0101") 
				or (SA_CurrentState = "0110") or (SA_CurrentState = "0111")) 
				or (SA_CurrentState = "1010") then
				
				SA_ActuateEnableError 	<= '0';
				SA_ActuateEnabled  		<= '1';
				SA_ActuateEnabledLED  	<= '1';
				
			else	
				SA_ActuateEnableError 	<= '1';
				SA_ActuateEnabled  		<= '0';
				SA_ActuateEnabledLED  	<= SA_2Hz_square;
			end if;
		end if;
	end process;
	
	process(SA_CurrentState, SA_KeyEnabled)
	begin
	-- Determines whether it is ok for System Enable Key to be set to zero
	-- for key enable to be equal zero
	-- 
	-- "0000" 	No State Defined/Initialise
	-- "0001"	Parked
	-- "1000" 	Kill
	-- "1001"	MakeSafe_1/2/3
	-- "1010"	Engineering
	
		SA_SetKEError <= '0';
		if ((SA_CurrentState = "0000") or (SA_CurrentState = "0001") 
			or (SA_CurrentState = "1000") or (SA_CurrentState = "1001")
			or (SA_CurrentState = "1010")) then
			SA_SetKEError <= '0';
		elsif(SA_KeyEnabled = '0') then
			SA_SetKEError <= '1';
		end if;

	end process;
	
	process(SA_SetKEError, SA_ResetKEError, SA_KEErrorState)
	begin	
		SA_NextKEErrorState <= SA_KEErrorState;
	
		if (SA_SetKEError = '1' and SA_ResetKEError = '0') then
				SA_NextKEErrorState <= '1';
		elsif (SA_SetKEError = '0' and SA_ResetKEError = '1') then
				SA_NextKEErrorState <= '0';
		end if;
	
	end process;
	
	process(SA_Set5VIntError, SA_Reset5VIntError, SA_5VIntErrorState, SA_Interlocks5V_ok,
	SA_Interlocks24V_ok)
	begin	
		SA_Next5VIntErrorState <= SA_5VIntErrorState;
	
		If (SA_Set5VIntError = '1' and SA_Reset5VIntError = '0') then
				SA_Next5VIntErrorState <= '1';
		elsif (SA_Set5VIntError = '0' and SA_Reset5VIntError = '1' and SA_Interlocks5V_ok = '1') then
				SA_Next5VIntErrorState <= '0';
		end if;
	end process;
	
	process(SA_Set24VIntError, SA_Reset24VIntError, SA_24VIntErrorState, SA_Interlocks24V_ok)
	begin	
		SA_Next24VIntErrorState <= SA_24VIntErrorState;
	
		if (SA_Set24VIntError = '1' and SA_Reset24VIntError = '0') then
				SA_Next24VIntErrorState <= '1';
		elsif (SA_Set24VIntError = '0' and SA_Reset24VIntError = '1' and SA_Interlocks24V_ok = '1') then
				SA_Next24VIntErrorState <= '0';
		end if;
	end process;
	
	-- PC ERROR STATE
	process(SA_SetPCError, SA_ResetPCError, SA_PCErrorState)
	begin	
		SA_NextPCErrorState <= SA_PCErrorState;
	
		if (SA_SetPCError = '1' and SA_ResetPCError = '0') then
				SA_NextPCErrorState <= '1';
		elsif (SA_SetPCError = '0' and SA_ResetPCError = '1') then
				SA_NextPCErrorState <= '0';
		end if;
	end process;
	
	
	SA_SetCapError <= SA_CaptureOverflowProt; -- This signal should only go high momentarily as capture timer should be reset.
	-- CAP ERROR STATE
	process(SA_SetCapError, SA_ResetCapError, SA_CapErrorState)
	begin	
		SA_NextCapErrorState <= SA_CapErrorState;	
		if (SA_SetCapError = '1') then
				SA_NextCapErrorState <= '1';
		elsif (SA_SetCapError = '0' and SA_ResetCapError = '1') then
				SA_NextCapErrorState <= '0';
		end if;
	end process;
	
	-- FrameNOTRaised BPS error (If frame is not raised when target is parked)
	process(SA_SetFrameNOTRaisedError, SA_ResetFrameNOTRaisedError,
	SA_FrameNOTRaisedErrorState)
	begin	
		SA_NextFrameNOTRaisedErrorState <= SA_FrameNOTRaisedErrorState;	
		if (SA_SetFrameNOTRaisedError = '1') then
				SA_NextFrameNOTRaisedErrorState <= '1';
		elsif (SA_SetFrameNOTRaisedError = '0' and SA_ResetFrameNOTRaisedError = '1') then
				SA_NextFrameNOTRaisedErrorState <= '0';
		end if;
	end process;
		
	process(SA_SetBPSError, SA_ResetBPSError, SA_BPSErrorState, sl_BPSDisabled)
	begin	
		SA_NextBPSErrorState <= SA_BPSErrorState;
	
		if (sl_BPSDisabled = '0') then
			if (SA_SetBPSError = '1' and SA_ResetBPSError = '0') then
				SA_NextBPSErrorState <= '1';
			elsif (SA_SetBPSError = '0' and SA_ResetBPSError = '1') then
				SA_NextBPSErrorState <= '0';
			end if;
		else
			SA_NextBPSErrorState <= '0';
		end if;
	end process;
	
	-- This variable should only get set if there is a BPS error AND the frame is 
	-- NOT raised.
	-- This is used to latch the relay outputs which are seperate from the actual
	-- error state itself.
	-- This latch may not be needed. (Default is that it is not needed!)
	-- In this case set all SA_NextBPSErrorStateEnable <= '0';	
	process(SA_SetBPSError, SA_ResetBPSError, SA_BPSErrorStateEnable, sl_BPSDisabled,
		sl_DG_frame_raised)
	begin	
		SA_NextBPSErrorStateEnable <= SA_BPSErrorStateEnable;
	
		if (sl_BPSDisabled = '0' and sl_DG_frame_raised = '0') then
			if (SA_SetBPSError = '1' and SA_ResetBPSError = '0') then
				--SA_NextBPSErrorStateEnable <= '1';
				SA_NextBPSErrorStateEnable <= '0';
			elsif (SA_SetBPSError = '0' and SA_ResetBPSError = '1') then
				SA_NextBPSErrorStateEnable <= '0';
			end if;
		else
			SA_NextBPSErrorStateEnable <= '0';
		end if;
	end process;
	
	process(SA_SetQuadADCError, SA_ResetQuadADCError, SA_QuadADCErrorState,
		SA_CurrentState)
	begin	
		SA_NextQuadADCErrorState <= SA_QuadADCErrorState;
	
		if (SA_SetQuadADCError = '1' and SA_ResetQuadADCError = '0') then
				SA_NextQuadADCErrorState <= '1';
				-- Do not allow user to reset if still moving between park and hold
		elsif (SA_SetQuadADCError = '0' and 
			(not((SA_CurrentState = "0010") or (SA_CurrentState = "0011"))) and
			SA_ResetQuadADCError = '1') then
				SA_NextQuadADCErrorState <= '0';
		end if;
	end process;
		
	-- Quad Continuity Error Latch
	process(SA_SetQuadContinuityError, SA_ResetQuadContinuityError, SA_QuadContinuityErrorState,
		SA_CurrentState)
	begin	
		SA_NextQuadContinuityErrorState <= SA_QuadContinuityErrorState;
	
		if (SA_SetQuadContinuityError = '1' and SA_ResetQuadContinuityError = '0') then
				SA_NextQuadContinuityErrorState <= '1';
		elsif (SA_SetQuadContinuityError = '0' and SA_ResetQuadContinuityError = '1') then
				SA_NextQuadContinuityErrorState <= '0';
		end if;
	end process;

	
	-- Generates a single clock signal if the position value changes	
	-- and the DAQ fifo is not full
	process(SA_quad_position_prev, SA_quad_position, DAC_fifofull)
	begin
		SA_position_change <= '0';
		if ((SA_quad_position_prev /= SA_quad_position) and (DAC_fifofull = '0')) then
			SA_position_change <= '1';
		end if;
	end process;
	
	-- EPOTS
	-- Generates a single clock signal if the EPOT gain value changes	
	-- and the EPOT fifo is not full
	process(ChA_EPOT_value_prev, ChA_EPOT_value_now, ChA_EPOT_fifofull)
	begin
		SA_EPOT_ChA_FifoWrite <= '0';
		if ((ChA_EPOT_value_prev /= ChA_EPOT_value_now) and (ChA_EPOT_fifofull = '0')) then
			SA_EPOT_ChA_FifoWrite <= '1';
		end if;
	end process;
	
	process(ChB_EPOT_value_prev, ChB_EPOT_value_now, ChB_EPOT_fifofull)
	begin
		SA_EPOT_ChB_FifoWrite <= '0';
		if ((ChB_EPOT_value_prev /= ChB_EPOT_value_now) and (ChB_EPOT_fifofull = '0')) then
			SA_EPOT_ChB_FifoWrite <= '1';
		end if;
	end process;
	
	process(Indx_EPOT_value_prev, Indx_EPOT_value_now, Indx_EPOT_fifofull)
	begin
		SA_EPOT_Indx_FifoWrite <= '0';
		if ((Indx_EPOT_value_prev /= Indx_EPOT_value_now) and (Indx_EPOT_fifofull = '0')) then
			SA_EPOT_Indx_FifoWrite <= '1';
		end if;
	end process;
	
	--Engineering	
	u_counter_now <= (others => '0');
	u_counter_next <= (others => '0');
	
	-- CLOCKING PROCESS 
	process(int_sysclk, int_logic_reset)
	begin
	
		-- Because the 1Hz clks are not synchronised to the reset
		-- the first shift register update can happen at any time after the reset
		-- To mitigate this the first bit is ignored to ensure each output is on for 
		-- the desired time.
		if (int_logic_reset = '1') then
			slv_EngLampTestLED_now	<= "0000000000001";
			slv_EngLampTestIGBT_now	<= "0000001";
			T_BPS_state_now <= State_BPS_start;
			T_LASER_state_now <= State_LASER_start;
			T_NUDGE_state_now <= State_NUDGE_start;
			u_counter_now <= (others => '0');
		elsif (int_sysclk'event and int_sysclk = '1') then
			slv_EngLampTestLED_now <= slv_EngLampTestLED_next;
			slv_EngLampTestIGBT_now <= slv_EngLampTestIGBT_next;
			u_counter_now <= u_counter_next;
			T_NUDGE_state_now <= T_NUDGE_state_next;	
			if (SA_2s_tick	= '1') then
				T_BPS_state_now <= T_BPS_state_next;
				T_LASER_state_now <= T_LASER_state_next;			
			end if;
		end if;
	end process;
	
	-- Lamp Tests
	process(slv_EngLampTestLED_now, SA_2Hz_tick, sl_EngLampTestLEDReset)
	begin
		slv_EngLampTestLED_next <= slv_EngLampTestLED_now;
		if sl_EngLampTestLEDReset = '1' then
			slv_EngLampTestLED_next	<= "0000000000001";
		elsif (SA_2Hz_tick = '1' and slv_EngLampTestLED_now(12) /= '1' ) then
			slv_EngLampTestLED_next <=  slv_EngLampTestLED_now(11 downto 0) 
				& '1';
		end if;	
	end process;
	
	process(slv_EngLampTestIGBT_now, SA_1Hz_tick	
, sl_EngLampTestIGBTReset)
	begin
		slv_EngLampTestIGBT_next <= slv_EngLampTestIGBT_now;
		if sl_EngLampTestIGBTReset = '1' then
			slv_EngLampTestIGBT_next	<= "0000001";
		elsif (SA_1Hz_tick	
 = '1' and slv_EngLampTestIGBT_now /= "1000000" ) then
			slv_EngLampTestIGBT_next <=  slv_EngLampTestIGBT_now(5 downto 0) 
				& slv_EngLampTestIGBT_now(6);
		end if;	
	end process;
	
--	--BPS Test Relays Process
	process(sl_EngBPSRelays, T_BPS_state_now)
	begin
	
		sl_EngBPSControl  <= "000";
		T_BPS_state_next	<= T_BPS_state_now;
		SA_reset_counters1	<= '0';
		
		case T_BPS_state_now is
		
			when State_BPS_start =>
				if sl_EngBPSrelays = '1' then
					T_BPS_state_next <= State_BPS1;
					SA_reset_counters1 <= '1';
				end if;
			
			when State_BPS1 =>
				sl_EngBPSControl <= "001";
				T_BPS_state_next <= State_BPS2;
				
			when State_BPS2 =>
				sl_EngBPSControl <= "000";
				T_BPS_state_next <= State_BPS3;
			
			when State_BPS3 =>
				sl_EngBPSControl <= "010";
				T_BPS_state_next <= State_BPS4;
			
			when State_BPS4 =>
				sl_EngBPSControl <= "000";
				T_BPS_state_next <= State_BPS5;
				
			when State_BPS5 =>
				sl_EngBPSControl <= "100";
				T_BPS_state_next <= State_BPS6;
				
			when State_BPS6 =>
				sl_EngBPSControl <= "000";
				T_BPS_state_next <= State_BPS7;
				
			when State_BPS7 =>
				sl_EngBPSControl <= "111";
				if sl_EngBPSRelays = '0' then
					T_BPS_state_next <= State_BPS_start;
				end if;
			
		end case;
		
	end process;
	
	--LASER Test Relays Process
	process(sl_EngLASERRelays, T_LASER_state_now)
	begin
	
		sl_EngLASERControl  <= "000";
		T_LASER_state_next	<= T_LASER_state_now;
		SA_reset_counters2	<= '0';
		
		case T_LASER_state_now is
		
			when State_LASER_start =>
				if sl_EngLASERRelays = '1' then
					T_LASER_state_next <= State_LASER1;
					SA_reset_counters2 <= '1';
				end if;
			
			when State_LASER1 =>
				sl_EngLASERControl <= "001";
				T_LASER_state_next <= State_LASER2;
				
			when State_LASER2 =>
				sl_EngLASERControl <= "000";
				T_LASER_state_next <= State_LASER3;
			
			when State_LASER3 =>
				sl_EngLASERControl <= "010";
				T_LASER_state_next <= State_LASER4;
			
			when State_LASER4 =>
				sl_EngLASERControl <= "000";
				T_LASER_state_next <= State_LASER5;
				
			when State_LASER5 =>
				sl_EngLASERControl <= "100";
				T_LASER_state_next <= State_LASER6;
				
			when State_LASER6 =>
				sl_EngLASERControl <= "000";
				T_LASER_state_next <= State_LASER7;
				
			when State_LASER7 =>
				sl_EngLASERControl <= "111";
				if sl_EngLASERRelays = '0' then
					T_LASER_state_next <= State_LASER_start;
				end if;
			
		end case;
		
	end process;
	
	-- NUDGE Process
	process(sl_EngNudge, T_NUDGE_state_now, sl_Nudge_Timer_Complete,
	SA_1Hz_tick	
)
	begin
	
		slv_EngNUDGEControl	<= "000000";
		T_NUDGE_state_next	<= T_NUDGE_state_now;
		sl_Start_Nudge_Timer	<= '0';
		
		case T_NUDGE_state_now is
		
			when State_NUDGE_start =>
				if sl_EngNUDGE = '1' then
					T_NUDGE_state_next <= State_NUDGE1;
					--sl_Start_Nudge_Timer <= '1';
				end if;
			
			when State_NUDGE1 =>
				-- Synchronise to 1Hz Timer
				if (SA_1Hz_tick	
 = '1') then
					T_NUDGE_state_next <= State_NUDGE2;
				end if;
				
			when State_NUDGE2 =>
				slv_EngNUDGEControl	<= "001001";
				sl_Start_Nudge_Timer <= '1';
				T_NUDGE_state_next <= State_NUDGE3;
			
			when State_NUDGE3 =>
				slv_EngNUDGEControl	<= "001001";
				if (sl_Nudge_Timer_Complete = '1') then
					T_NUDGE_state_next <= State_NUDGE4;
				end if;
				
			when State_NUDGE4 =>
				if (SA_1Hz_tick	
 = '1') then
					T_NUDGE_state_next <= State_NUDGE5;
				end if;
			
			when State_NUDGE5 =>
				slv_EngNUDGEControl	<= "100001";
				sl_Start_Nudge_Timer <= '1';
				T_NUDGE_state_next <= State_NUDGE6;
			
			when State_NUDGE6 =>
				slv_EngNUDGEControl	<= "100001";
				if (sl_Nudge_Timer_Complete = '1') then
					T_NUDGE_state_next <= State_NUDGE7;
				end if;
				
			when State_NUDGE7 =>
				if (SA_1Hz_tick	
 = '1') then
					T_NUDGE_state_next <= State_NUDGE8;
				end if;
			
			when State_NUDGE8 =>
				slv_EngNUDGEControl	<= "100100";
				sl_Start_Nudge_Timer <= '1';
				T_NUDGE_state_next <= State_NUDGE9;
			
			when State_NUDGE9 =>
				slv_EngNUDGEControl	<= "100100";
				if (sl_Nudge_Timer_Complete = '1') then
					T_NUDGE_state_next <= State_NUDGE10;
				end if;
			
			when State_NUDGE10 =>
				if (SA_1Hz_tick	
 = '1') then
					T_NUDGE_state_next <= State_NUDGE11;
				end if;
			
			when State_NUDGE11 =>
			
				if sl_EngNUDGE = '0' then
					T_NUDGE_state_next <= State_NUDGE_start;
				end if;
			
		end case;
		
	end process;
	
	--Position Test
	process(sl_EngPosTest, SA_50kHz_tick, u_counter_now, DAC_fifofull)
	
	begin
		
		u_counter_next <= (others => '0');
		sl_TestCounterWrite <= '0';
		
		if  (sl_EngPosTest = '1') then
			u_counter_next <= u_counter_now;
			if (SA_50kHz_tick = '1') then
				u_counter_next <= u_counter_now + 1;
				if (DAC_fifofull = '0') then
					sl_TestCounterWrite <= '1';
				end if;
			end if;
		end if;
		
	end process;
	
	--CHA Analogue Monitor
	process(sl_OpticsADCMode, SA_50kHz_tick, DAC_fifofull)
	
	begin
	
		sl_TestCounterWrite2 <= '0';
		
		if  (sl_OpticsADCMode = '1' and SA_50kHz_tick = '1' and 
			DAC_fifofull = '0') then
			
			sl_TestCounterWrite2 <= '1';			
		end if;
		
	end process;
	
	
	process(SA_EngineerReg, SA_EnableEngineering)
	begin
	
		sl_EngLampTestLED  <= '0';
		sl_EngLampTestIGBT <= '0';
		sl_EngTrigAccept 	 <= '0';
		sl_EngTargetReady  <= '0';
		sl_EngFrameEnable  <= '0';
		sl_EngActTrig		 <= '0';
		sl_EngBPS50R		 <= '0';
		sl_EngActuating	 <= '0';
		sl_EngSpillMarker  <= '0';
		sl_EngBPSRelays	 <= '0';
		sl_EngLASERRelays	 <= '0';
		sl_EngFERelay		 <= '0';
		sl_EngPosTest		 <= '0';
		sl_EngNudge			 <= '0';
		sl_OpticsADCMode	 <= '0';
		
		if (SA_EnableEngineering = '1') then
			case SA_EngineerReg is
	
			when "00000" 	=>
		
			when "00001"	=>
		
				sl_EngLampTestLED <= '1';
				
			when "00010"	=>
		
				sl_EngLampTestIGBT <= '1';
				
			when "00011"	=>
			
				sl_EngTrigAccept <= '1';
				
			when "00100"	=>
			
				sl_EngTargetReady <= '1';
			
			when "00101"	=>
			
				sl_EngFrameEnable <= '1';
			
			when "00110"	=>
			
				sl_EngActTrig	<= '1';
							
			when "00111"	=>
			
				sl_EngBPS50R 	<= '1';
			
			when "01000"	=>
			
				sl_EngActuating	<= '1';				
				
			when "01001"	=>
			
				sl_EngSpillMarker	<= '1';
				
			when "01010"	=>
			
				sl_EngBPSRelays	<= '1';
				
			when "01011"	=>
			
				sl_EngLASERRelays	<= '1';
				
			when "01100"	=>
			
				sl_EngFERelay	<= '1';
				
			when "01101"	=>
			
				sl_EngPosTest	<= '1';
				
			when "01110"	=>
			
				sl_EngNudge	<= '1';
				
			when "10000"	=>
				
				sl_OpticsADCMode <= '1';
				
			when others =>
	
			end case;
		end if;
	end process;
	
	-- This turns on Optics Mode which still requires the controller to function
	-- Hence no sl_Engineering flag
	process(SA_EngineerReg, SA_EnableEngineering)
	begin
	
		sl_OpticsMode <= '0';
		
		if (SA_EnableEngineering = '0') then
		
			case (SA_EngineerReg) is
			when "01111"	=>
				sl_OpticsMode <= '1';
			when others =>
				sl_OpticsMode <= '0';
			end case;
		end if;
		
	end process;
	
end arch;

