----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:50:10 10/01/2009 
-- Design Name: 
-- Module Name:    Actuate - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments:
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Actuate is
    Port (
			ACT_I_clk 				: in std_logic;
         ACT_I_reset 			: in std_logic;
			ACT_I_resetCount		: in std_logic;
			ACT_I_SetPoint1		: in std_logic_vector(8 downto 0);
			ACT_I_SP1MINIMUM		: in std_logic_vector(8 downto 0);
			ACT_I_SetPoint2		: in std_logic_vector(8 downto 0);
			ACT_I_QuadPos			: in std_logic_vector(8 downto 0);
			ACT_I_QuadDir			: in std_logic; -- 0 = Down & 1 = Up
			ACT_I_StartAct			: in std_logic;
			ACT_I_KillActuation 	: in std_logic;
			
			ACT_I_DeadTime			: in std_logic_vector(11 downto 0);
			ACT_I_KickBackTime	: in std_logic_vector(17 downto 0);
			
			ACT_I_ROMSelect		: in std_logic;
			
			ACT_O_MosState			: out std_logic_vector(3 downto 0);
			ACT_O_MinPos			: out std_logic_vector(8 downto 0);
			ACT_O_ActComplete		: out std_logic;
			ACT_O_HoldState		: out std_logic_vector(5 downto 0);
			ACT_O_ActCount			: out std_logic_vector(23 downto 0);
			ACT_O_Actuating		: out std_logic;
			ACT_O_StartPos			: out std_logic_vector(8 downto 0);
			
			ACT_O_SP1Time			: out std_logic_vector(31 downto 0);
			ACT_O_MinTime			: out std_logic_vector(31 downto 0);
			ACT_O_ActTime			: out std_logic_vector(31 downto 0);
			ACT_O_MasterTimer		: out std_logic_vector(31 downto 0);		
			ACT_O_Quad_Check		: out std_logic;
			ACT_O_QCTime			: out std_logic;
			ACT_O_Quadrant_State	: out std_logic_vector(2 downto 0)	
			);
			
end Actuate;

architecture arch of Actuate is

		--constant ACT_dead_time		: std_logic_vector(11 downto 0) := "011111010000";--50 uS
		--constant ACT_dead_time		: std_logic_vector(11 downto 0) := "000000010100";--0.5 uS
		--constant ACT_KickBack_time		: std_logic_vector(17 downto 0) := "001001110001000000";--1 ms
		--constant ACT_KickBack_time		: std_logic_vector(17 downto 0) := "000000000000010100";--0.5 us
			
		-- General Signals
		type TA_state_type is 	
								(
								State_Initialise,
								State_Start,
								State_Quadrant1a,
								State_Quadrant1b,
								State_Quadrant1c,
								
								State_Quadrant2a,
								State_Quadrant2b,
								State_Quadrant2c,
								State_Quadrant3a,
								State_Quadrant4a,
								State_Quadrant4b,
								State_Quadrant4c,
								State_KickBack1,
								State_KickBack2,
								State_KickBack3,
								State_KickBack4,
								State_KickBack5,
								State_KickBack6
								);
								
		signal TA_state_reg, TA_state_next: TA_state_type;

	
		signal ACT_MOS_state_raw 	: std_logic_vector(15 downto 0);
		signal ACT_MOS_state_raw_1 : std_logic_vector(15 downto 0);
		signal ACT_MOS_state_raw_2 : std_logic_vector(15 downto 0);
		signal ACT_timer_start 		: std_logic;
		signal ACT_timer_complete 	: std_logic;
		signal ACT_rom_enable 		: std_logic;
		
		signal ACT_SP1Pass			: std_logic;
		signal ACT_SP2Pass			: std_logic;
		signal ACT_GetStartPos		: std_logic;
		
		signal ROM_NO_CONNECTION	:std_logic_vector(1 downto 0);
		signal ACTUATE_TIME_NO_CONNECTION_1	:std_logic_vector(11 downto 0);
		signal ACTUATE_TIME_NO_CONNECTION_2	:std_logic_vector(17 downto 0);
		signal ACT_MinPos				: std_logic_vector(8 downto 0);
		signal ACT_MinPosNext		: std_logic_vector(8 downto 0);
		signal ACT_StartPos			: std_logic_vector(8 downto 0);
		signal ACT_StartPosNext		: std_logic_vector(8 downto 0);
		
		signal ACT_ActuationCount		: std_logic_vector(23 downto 0);
		signal ACT_ActuationCountNext	: std_logic_vector(23 downto 0);
		signal ACT_ActuationInc			: std_logic_vector(23 downto 0);	
		
		-- KickBack Timer
		signal ACT_KickBackTimer_start 	 :std_logic;
		signal ACT_KickBackTimer_complete :std_logic;
		
		-- '0' selects Lower Nibble of Quad Rom = Downward Force
		-- '1' selects Upper Nibble of Quad Rom = Upward Force	
		signal ACT_force_select		: std_logic;

		-- Counter		
		signal SA_ResetCounter 	: std_logic;
		signal SA_CounterValue 	: std_logic_vector(31 downto 0);
		signal SA_SP1TimeNow		: std_logic_vector(31 downto 0);
		signal SA_SP1TimeNext	: std_logic_vector(31 downto 0);
		signal SA_MinTimeNow		: std_logic_vector(31 downto 0);
		signal SA_MinTimeNext	: std_logic_vector(31 downto 0);
		signal SA_ActTimeNow		: std_logic_vector(31 downto 0);
		signal SA_ActTimeNext	: std_logic_vector(31 downto 0);
		
		signal slv_ActQuadState	: std_logic_vector(2 downto 0);
		
begin

			ACT_O_StartPos <= ACT_StartPos;
			
			ACT_O_SP1Time	<= SA_SP1TimeNow;
			ACT_O_MinTime	<= SA_MinTimeNow;
			ACT_O_ActTime	<= SA_ActTimeNow;

			ACT_O_Quadrant_State	<= slv_ActQuadState;
			-- Instantiate the Change State Timer 
			-- (50 us dead time when system changes state)
			
			ACT_O_MasterTimer	<= SA_CounterValue;
			
			dead_time_timer: entity work.Generic_Timer(arch)
			generic map (N=>12)
			port map   (
						clk => ACT_I_clk,
						reset => ACT_I_reset,
						start => ACT_timer_start,
						en => '1',
						synch_clear => '0',
						time_max_in => ACT_I_deadtime,
						time_current_out => ACTUATE_TIME_NO_CONNECTION_1,
						tick => ACT_timer_complete
					);
					
			-- Instantiate the Kick Back Timer 
			-- (~1 ms Kickback before capture state)
			
			KickBack_timer: entity work.Generic_Timer(arch)
			generic map (N=>18)
			port map   (
						clk => ACT_I_clk,
						reset => ACT_I_reset,
						start => ACT_kickbackTimer_start,
						en => '1',
						synch_clear => '0',
						time_max_in => ACT_I_kickbackTime,
						time_current_out => ACTUATE_TIME_NO_CONNECTION_2,
						tick => ACT_KickBackTimer_complete
					);
	
			-- Instantiate the Quad to Act ROM			
			Rom1: entity work.Quad_2_Act_ROM_1(arch)
			
			port map(
				clk => ACT_I_clk,
				addr => ACT_I_QuadPos,
				mos_rom_enable => ACT_rom_enable,
				data => ACT_MOS_state_raw_1
					);
						
			-- Instantiate the Quad to Act ROM			
			Rom2: entity work.Quad_2_Act_ROM_2(arch)
			
			port map(
					clk => ACT_I_clk,
					addr => ACT_I_QuadPos,
					mos_rom_enable => ACT_rom_enable,
					data => ACT_MOS_state_raw_2
						);
									
			-- Instantiate Counter		
			Counter: entity work.Binary_Counter(arch)
			Port map(
				BC_I_clk 			=> ACT_I_clk,			--: in  STD_LOGIC;
				BC_I_reset 			=> ACT_I_reset,		--: in  STD_LOGIC
				BC_I_ResetCounter	=> SA_ResetCounter,	--: in std_logic;
			
				BC_O_CountValue	=> SA_CounterValue	--: out std_logic_vector(31 downto 0);
				);
						
			-- Internal Assignments
			ACT_O_MinPos <= ACT_MinPos;
			ACT_O_ActCount <= ACT_ActuationCount;
			
			-- Assignments to Output Pins (where necessary)	
			ACT_O_HoldState(5 downto 0) <= ACT_MOS_state_raw(5 downto 0);
			ROM_NO_CONNECTION <= ACT_MOS_state_raw(7 downto 6);
			
			-- Determine which ROM is selected
			process(ACT_I_ROMSelect, ACT_MOS_state_raw_1, ACT_MOS_state_raw_2)		
			begin
				if (ACT_I_ROMSelect = '0') then
					ACT_MOS_state_raw <= ACT_MOS_state_raw_1;
				else
					ACT_MOS_state_raw <= ACT_MOS_state_raw_2;
				end if;
			end process;
			
			-- State and Data Registers
			process(ACT_I_clk, ACT_I_reset, ACT_I_KillActuation)
			
			begin		
				if (ACT_I_reset = '1') then
					TA_state_reg <= State_Initialise;
					ACT_MinPos <= (Others => '0');
					ACT_ActuationCount <= (Others => '0');
					ACT_StartPos <= (Others => '0');
					SA_SP1TimeNow <= (others => '0');
					SA_MinTimeNow <= (others => '0');
					SA_ActTimeNow <= (others => '0');
				elsif (ACT_I_clk'event and ACT_I_clk = '1') then
					if (ACT_I_KillActuation = '1') Then
						TA_state_reg <= State_Initialise;
					else
						TA_state_reg <= TA_state_next;
						ACT_MinPos <= ACT_MinPosNext;
						ACT_ActuationCount <= ACT_ActuationCountNext;
						ACT_StartPos	<= ACT_StartPosNext;
						SA_SP1TimeNow	<= SA_SP1TimeNext;
						SA_MinTimeNow	<=	SA_MinTimeNext;
						SA_ActTimeNow  <= SA_ActTimeNext;
					end if;
				end if;				
			end process;

			
			-- Next State Logic & Data Path Functional Units/Routing
			-- Next State Logic & Data Path Functional Units/Routing
			process(TA_state_reg, Act_timer_complete,ACT_SP1Pass, ACT_SP2Pass, ACT_I_QuadDir, ACT_I_StartAct,
					  ACT_KickBackTimer_complete, SA_SP1TimeNow, SA_MinTimeNow, SA_ActTimeNow, SA_CounterValue)
				begin
				--General Signals
				TA_state_next <= TA_state_reg;
		
				SA_SP1TimeNext <= SA_SP1TimeNow;
				SA_MinTimeNext <= SA_MinTimeNow;
				SA_ActTimeNext <= SA_ActTimeNow;
				
				--  ** State Signals **
				ACT_rom_enable <= '0'; -- Default is Disabled
				ACT_timer_start <= '0';
				ACT_kickbackTimer_start <= '0';
				ACT_O_ActComplete <= '0';
				ACT_O_Actuating <= '1';
				ACT_GetStartPos <= '0';
				SA_ResetCounter <='0';
						
				-- '0' selects Lower Nibble of Quad Rom = Downward Force
				-- '1' selects Upper Nibble of Quad Rom =  Upward Force	
				ACT_force_select <= '0';
				
				ACT_O_Quad_Check	<= '0';
				ACT_O_QCTime		<= '0';
				
				slv_ActQuadState	<= "000";
	
				case TA_state_reg is
	
				-- System Initialisation on Power Up
				when State_Initialise =>
					ACT_O_Actuating <= '0';
					TA_state_next <= State_Start;
									
				when State_Start =>
				-- It is important that ACT_I_StartAct is not held 
				-- high else the actuator will actuate continuously
				-- This signal should be a single Tick
					ACT_O_Actuating <= '0';
					if (ACT_I_StartAct = '1') Then
						TA_state_next <= State_Quadrant1a;
						ACT_GetStartPos <= '1';
						SA_ResetCounter <= '1';
						SA_SP1TimeNext <= (others => '0');
						SA_MinTimeNext <= (others => '0');
						SA_ActTimeNext <= (others => '0');
					else
						TA_state_next <= State_start;
					end if;
									
				when State_Quadrant1a =>
				
					slv_ActQuadState	<= "001";
					--Trigger 50uS Timer
					ACT_timer_start <= '1';
					TA_state_next <= State_Quadrant1b;
									
				when State_Quadrant1b =>
					slv_ActQuadState	<= "001";
					if (ACT_timer_complete = '1') then
						TA_state_next <= State_Quadrant1c;
					else
						TA_state_next <= State_Quadrant1b;
					end If;
					
				when State_Quadrant1c =>
						slv_ActQuadState	<= "001";
						--Now We can turn the ROM on
						ACT_rom_enable <= '1';
						ACT_O_Quad_Check <= '1';
						--At this point we are looking for the first SetPoint
						if (ACT_SP1Pass = '1') then
							TA_state_next <= State_Quadrant2a;
						else
							TA_state_next <= State_Quadrant1c;
							SA_SP1TimeNext	<= SA_CounterValue; -- Updates until SP1 has been passes!
						end if;
													
				when State_Quadrant2a =>
					slv_ActQuadState	<= "010";
					-- Current Directions are Reversed so we must have a 50uS Wait State			
					ACT_timer_start <= '1';
					ACT_O_Quad_Check <= '1';
					TA_state_next <= State_Quadrant2b;
									
				when State_Quadrant2b =>
					slv_ActQuadState	<= "010";
					ACT_O_Quad_Check <= '1';
					if ACT_timer_complete = '1' then
						TA_state_next <= State_Quadrant2c;
					else
						TA_state_next <= State_Quadrant2b;
					end If;
					
				when State_Quadrant2c =>
					slv_ActQuadState	<= "010";
					--Now We can turn the ROM on
					ACT_rom_enable <= '1';
					Act_Force_Select <= '1';
					ACT_O_Quad_Check <= '1';
					ACT_O_QCTime <= '1'; -- Increase QuadContinuity Check Time 
					--At this point we are looking for a change in direction
						if (ACT_I_QuadDir = '0') then
							TA_state_next <= State_Quadrant2c;
							SA_MinTimeNext <= SA_CounterValue; -- Updates until this is no longer true!
						else
							TA_state_next <= State_Quadrant3a;
							
						end if;
									
				when State_Quadrant3a =>
					slv_ActQuadState	<= "011";
					ACT_rom_enable <= '1';
					Act_Force_Select <= '1';
					ACT_O_Quad_Check <= '1';
					ACT_O_QCTime <= '1'; -- Increase QuadContinuity Check Time 
					-- Now we are looking for the second setpoint
					if (ACT_SP2Pass = '1') then
							TA_state_next <= State_Quadrant4a;
						else
							TA_state_next <= State_Quadrant3a;
						end if;
					
				when State_Quadrant4a =>
					slv_ActQuadState	<= "100";	
					-- Current Directions are Reversed so we must have a 50uS Wait State			
					ACT_timer_start <= '1';
					TA_state_next <= State_Quadrant4b;
					ACT_O_Quad_Check <= '1';
					ACT_O_QCTime <= '1'; -- Increase QuadContinuity Check Time 
					
				when State_Quadrant4b =>
					slv_ActQuadState	<= "100";
					ACT_O_Quad_Check <= '1';
					ACT_O_QCTime <= '1'; -- Increase QuadContinuity Check Time 
					if ACT_timer_complete = '1' then
						TA_state_next <= State_Quadrant4c;
					else
						TA_state_next <= State_Quadrant4b;
					end if;
					
				when State_Quadrant4c =>
					slv_ActQuadState	<= "100";
					--Now We can turn the ROM on
					ACT_rom_enable <= '1';
					ACT_O_Quad_Check <= '1';
					ACT_O_QCTime <= '1'; -- Increase QuadContinuity Check Time 
					--At this point we are looking for a change in direction
						if (ACT_I_QuadDir = '1') then
							TA_state_next <= State_Quadrant4c;
						else
							TA_state_next <= State_KickBack1;
							SA_ActTimeNext <= SA_CounterValue;
						end if;
						
				when State_KickBack1 =>
					-- Current Directions are Reversed so we must have a 50uS Wait State			
					ACT_timer_start <= '1';
					TA_state_next <= State_KickBack2;
					
					
				when State_KickBack2 =>
					if ACT_Timer_complete = '1' then
						TA_state_next <= State_KickBack3;
					else
						TA_state_next <= State_KickBack2;
					end If;
					
				when State_KickBack3 =>
					--Now Turn the KickBack On
					ACT_rom_enable <= '1';
					Act_Force_Select <= '1';
					Act_KickBackTimer_Start <= '1';
					TA_state_next <= State_KickBack4;
					
				
				when State_KickBack4 =>
					--Outputs must be on during this KickBack period
					ACT_rom_enable <= '1';
					Act_Force_Select <= '1';
					if ACT_KickBackTimer_complete = '1' then
						TA_state_next <= State_KickBack5;
					else
						TA_state_next <= State_KickBack4;
					end If;
				
				when State_KickBack5 =>
					-- Current Directions are Reversed so we must have a 50uS Wait State			
					ACT_timer_start <= '1';
					TA_state_next <= State_KickBack6;
					
					
				when State_KickBack6 =>
					if ACT_Timer_complete = '1' then
						TA_state_next <= State_Start;
						ACT_O_ActComplete <= '1';
						--SA_ActTimeNext <= SA_CounterValue;
					else
						TA_state_next <= State_KickBack6;
					end if;
					
				end case;
			
			end process;
						
			--This part selects whether it is the force up or force down mosfet state 
			--That is output from the Actuate module
			process(ACT_force_select, Act_MOS_state_Raw)
			
				begin			
				-- '0' selects Lower Nibble of Quad Rom = Downward Force
				-- '1' selects Upper Nibble of Quad Rom = Upward Force			
				if (ACT_force_select = '1') then
					ACT_O_MosState <= Act_MOS_state_Raw(15 downto 12);
				else
					Act_O_MosState <= Act_MOS_state_Raw(11 downto 8);
				end if;
				
			end process;
			
			
			-- This determines the trigger conditions between the quadrature 
			-- position and the SetPoints
			process(ACT_I_SetPoint1,ACT_I_SetPoint2,ACT_I_QuadDir,ACT_I_QuadPos, ACT_I_SP1MINIMUM)
			begin
			
				ACT_SP1Pass <= '0';
				ACT_SP2Pass <= '0';
			
				if ((Act_I_QuadDir = '0') and (unsigned(ACT_I_SetPoint1) > unsigned(ACT_I_QuadPos) 
						or unsigned(ACT_I_SP1MINIMUM) > unsigned(ACT_I_QuadPos))) Then	
					ACT_SP1Pass <= '1';					
				else				
					Act_SP1Pass <= '0';				
				end if;
				
				if ((Act_I_QuadDir = '1') and (unsigned(ACT_I_SetPoint2) < unsigned(ACT_I_QuadPos))) Then			
					ACT_SP2Pass <= '1';		
				else				
					Act_SP2Pass <= '0';			
				end if;
							
			end process;
			
			
			-- This Process is for determining the minimum position
			process(ACT_MinPos, TA_state_reg, ACT_I_QuadPos)
			begin
			
				ACT_MinPosNext <= ACT_MinPos;
				
				-- Lock the First Value of the Min Pos to the Start Value
				if (TA_state_reg = State_Quadrant1a) then
					ACT_MinPosNext <= ACT_I_QuadPos;
				end if;
				
				if (TA_State_Reg = State_Quadrant2c) then	
					if (ACT_I_QuadPos < ACT_MinPos) then	
						ACT_MinPosNext <= ACT_I_QuadPos;
					end if;
					
				end if;
			end process;
			
			
			-- Actuation Counter
			process(ACT_I_resetCount, ACT_ActuationCount, ACT_I_StartAct, ACT_ActuationInc)			
			begin
				ACT_ActuationCountNext <= ACT_ActuationCount;
				
				if (ACT_I_resetCount  = '1') then		
					ACT_ActuationCountNext <= (others => '0');		
				end if;
				
				if (ACT_I_resetCount = '0' and ACT_I_StartAct = '1') then	
					ACT_ActuationCountNext <= ACT_ActuationInc;
				end If;
			
			end process;
			
			ACT_ActuationInc <= std_logic_vector(unsigned(ACT_ActuationCount) + 1);
			
			-- Start Position
			process(ACT_StartPos, ACT_I_QuadPos, ACT_GetStartPos)			
			begin
				ACT_StartPosNext <= ACT_StartPos;
				
				if (ACT_GetStartPos = '1') then		
					ACT_StartPosNext <= ACT_I_QuadPos;		
				end if;
				
			end process;
			
end arch;

