----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:24:19 10/26/2010 
-- Design Name: 
-- Module Name:    BPS_001 - arch 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity BPS_001 is
    Port ( 
			clk 							: in std_logic;
         reset 						: in std_logic;
			
			i_BPS_Sp1MaxTime			: in std_logic_vector(31 downto 0);
			i_BPS_Sp1MinTime			: in std_logic_vector(31 downto 0);
			i_BPS_MinPosMaxTime		: in std_logic_vector(31 downto 0);
			i_BPS_MinPosMinTime		: in std_logic_vector(31 downto 0);
			i_BPS_MinPosMaxValue		: in std_logic_vector(31 downto 0);
			i_BPS_MinPosMinValue		: in std_logic_vector(31 downto 0);
			i_BPS_ActMaxTime			: in std_logic_vector(31 downto 0);
			i_BPS_ActMinTime			: in std_logic_vector(31 downto 0);
			
			i_BPS_Sp1Time				: in std_logic_vector(31 downto 0);
			i_BPS_MinPosTime			: in std_logic_vector(31 downto 0);
			i_BPS_MinPosValue			: in std_logic_vector(8 downto 0);
			i_BPS_ActTime				: in std_logic_vector(31 downto 0);
			i_BPSDisabled				: in std_logic;
			
			i_check_limits				: in std_logic;
			
			i_BPS_Resets				: in std_logic_vector(7 downto 0);
			
			i_WDT_Error					: in std_logic;
			i_DD_Error					: in std_logic;
			i_QCCont_Error				: in std_logic;
			i_Frame_Error				: in std_logic;
			
			o_BPS_Error					: out std_logic;
			o_BPS_ErrorTick			: out std_logic;
			o_BPS_ErrorsReg			: out std_logic_vector(31 downto 0)
			);
end BPS_001 ;

architecture arch of BPS_001 is

		-- These signals = '1' if value is out of range of limit
		-- otherwise '0';
		signal	sl_Sp1MaxTime			: std_logic;
		signal	sl_Sp1MinTime			: std_logic;
		signal 	sl_MinPosMaxTime		: std_logic;
		signal 	sl_MinPosMinTime		: std_logic;
		signal 	sl_MinPosMaxValue		: std_logic;
		signal 	sl_MinPosMinValue		: std_logic;
		signal 	sl_ActMaxTime			: std_logic;
		signal 	sl_ActMinTime			: std_logic;
		
		signal 	slv_BPS_InternalReg	: std_logic_vector(7 downto 0);
		signal 	slv_BPS_Register		: std_logic_vector(7 downto 0);
		signal 	slv_BPS_RegAndResets	: std_logic_vector(7 downto 0);
		
		signal 	sl_BPS_Error_Int		: std_logic;
		signal 	sl_BPS_Error			: std_logic;
		
		signal 	slv_BPS_ResetsInt		: std_logic_vector(7 downto 0);
		
begin

	-- Instantiate Edge Detector - Gives single tick output when bps error occurs
	BPS_tick: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> sl_BPS_Error,
			O_edge_out	=> o_BPS_ErrorTick
		);

	-- Ensure all BPS resets are seen as a single tick by instantiating 
	-- a set of edge detectors
	reset_0: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> i_BPS_Resets(0),
			O_edge_out	=> slv_BPS_ResetsInt(0)
		);

		reset_1: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> i_BPS_Resets(1),
			O_edge_out	=> slv_BPS_ResetsInt(1)
		);
		
		reset_2: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> i_BPS_Resets(2),
			O_edge_out	=> slv_BPS_ResetsInt(2)
		);
		
		reset_3: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> i_BPS_Resets(3),
			O_edge_out	=> slv_BPS_ResetsInt(3)
		);
		
		reset_4: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> i_BPS_Resets(4),
			O_edge_out	=> slv_BPS_ResetsInt(4)
		);
		
		reset_5: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> i_BPS_Resets(5),
			O_edge_out	=> slv_BPS_ResetsInt(5)
		);
		
		reset_6: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> i_BPS_Resets(6),
			O_edge_out	=> slv_BPS_ResetsInt(6)
		);
		
		reset_7: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> i_BPS_Resets(7),
			O_edge_out	=> slv_BPS_ResetsInt(7)
		);
		
	o_BPS_ErrorsReg(7 downto 0) <= slv_BPS_Register;
	o_BPS_ErrorsReg(8) 	<= i_WDT_Error;
	o_BPS_ErrorsReg(9) 	<= i_DD_Error;
	o_BPS_ErrorsReg(10)	<= i_QCCont_Error;
	o_BPS_ErrorsReg(11)	<= i_Frame_Error;
	o_BPS_ErrorsReg(31 downto 12) <= (others => '0');
	
	o_BPS_Error <= sl_BPS_Error; 
	
	-- Reset Functionality
	slv_BPS_RegAndResets <= slv_BPS_Register and (not(slv_BPS_ResetsInt));

	process(clk, reset)
	begin
		if (reset = '1') then
			slv_BPS_Register <= (others => '0');
		elsif(clk'event and clk = '1') then
			-- BPS is disabled if i_BPSDisabled = '1'
			if (i_check_limits = '1' and i_BPSDisabled = '0') then
				slv_BPS_Register <= slv_BPS_InternalReg;
			else
				slv_BPS_Register <= slv_BPS_RegAndResets;
			end if;
		end if;
	end process;
			
	-- Map Comparison to Internal Reg
	-- This is a list of the BPS errors and shows the bit mapping.
	slv_BPS_InternalReg(0) <= sl_Sp1MaxTime;
	slv_BPS_InternalReg(1) <= sl_Sp1MinTime;
	slv_BPS_InternalReg(2) <= sl_MinPosMaxTime;
	slv_BPS_InternalReg(3) <= sl_MinPosMinTime;
	slv_BPS_InternalReg(4) <= sl_MinPosMaxValue;
	slv_BPS_InternalReg(5) <= sl_MinPosMinValue;
	slv_BPS_InternalReg(6) <= sl_ActMaxTime;
	slv_BPS_InternalReg(7) <= sl_ActMinTime;
	
	-- Or'ing the Error signals to create a generic BPS Error signal
	--o_BPS_Error <= slv_BPS_Register(0);
	sl_BPS_Error_Int <= (((slv_BPS_Register(0) or slv_BPS_Register(1)) or (slv_BPS_Register(2) or slv_BPS_Register(3))) or
						((slv_BPS_Register(4) or slv_BPS_Register(5)) or (slv_BPS_Register(6) or slv_BPS_Register(7)))) or
						(((i_WDT_Error or i_DD_Error) or (i_QCCont_Error or i_Frame_Error)));
						
	sl_BPS_Error <= sl_BPS_Error_Int; 
		
	-- Checking Value vs Limits
	sl_Sp1MaxTime <= '1' when unsigned(i_BPS_Sp1Time) > unsigned(i_BPS_Sp1MaxTime)
					else '0';
	sl_Sp1MinTime <= '1' when unsigned(i_BPS_Sp1Time) < unsigned(i_BPS_Sp1MinTime)
					else '0';
					
	sl_MinPosMaxTime <= '1' when unsigned(i_BPS_MinPosTime) > unsigned(i_BPS_MinPosMaxTime)
					else '0';
	sl_MinPosMinTime <= '1' when unsigned(i_BPS_MinPosTime) < unsigned(i_BPS_MinPosMinTime)
					else '0';				
	
	sl_MinPosMaxValue <= '1' when unsigned(i_BPS_MinPosValue) > unsigned(i_BPS_MinPosMaxValue)
					else '0';
	sl_MinPosMinValue <= '1' when unsigned(i_BPS_MinPosValue) < unsigned(i_BPS_MinPosMinValue)
					else '0';
					
	sl_ActMaxTime <= '1' when unsigned(i_BPS_ActTime) > unsigned (i_BPS_ActMaxTime)
					else '0';
	sl_ActMinTime <= '1' when unsigned(i_BPS_ActTime) < unsigned (i_BPS_ActMinTime)
					else '0';
						
end arch;

