----------------------------------------------------------------------------------
--
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:57:14 10/08/2009 
-- Design Name: 
-- Module Name:    ErrorCorrection - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_Std.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

-- EC_I_undershoot & EC_I_overshoot & EC_I_count
-- Single tick inputs that keeps a cumulative count of the number of 
-- under/over shoots 
-- When Count Reaches Count Max



entity ErrorCorrection is

		Port ( 	
				EC_I_clk 				: in std_logic;
				EC_I_reset 				: in std_logic;
				EC_I_trigger			: in std_logic;
				EC_I_undershoot 		: in std_logic;
				EC_I_overshoot			: in std_logic;
				EC_I_count				: in std_logic;
				EC_I_resetCount		: in std_logic;
				EC_I_countMax			: in std_logic_vector(7 downto 0);
				EC_I_Threshold 		: in std_logic_vector(7 downto 0);
				EC_I_DefaultOffset	: in std_logic_vector(8 downto 0);
				EC_I_SP1					: in std_logic_vector(8 downto 0);
				EC_I_SP1MINIMUM		: in std_logic_vector(8 downto 0);
				
				EC_O_SP2					: out std_logic_vector(8 downto 0)
				);
				
end ErrorCorrection;

architecture arch of ErrorCorrection is

		-- General Signals
		type TA_state_type is 	
								(
								State_Initialise,
								State_State1,
								State_State2							
								);

		signal TA_state_reg, TA_state_next: TA_state_type;
		
		signal EC_PulseCount				: unsigned(7 downto 0);
		signal EC_pulseCountInc			: unsigned(7 downto 0);
		signal EC_pulseCountNext		: unsigned(7 downto 0);

		signal EC_Offset					: unsigned(8 downto 0);
		signal EC_OffsetNext				: unsigned(8 downto 0);
		signal EC_I_DefaultOffsetLast : unsigned(8 downto 0);
		signal EC_OverShootsInc			: unsigned(7 downto 0);
		signal EC_OverShoots				: unsigned(7 downto 0);
		signal EC_UnderShootsInc		: unsigned(7 downto 0);
		signal EC_UnderShoots			: unsigned(7 downto 0);
		signal EC_UnderShootsNext		: unsigned(7 downto 0);
		signal EC_OverShootsNext		: unsigned(7 downto 0);
		signal EC_CountMaxAdj			: unsigned(7 downto 0);

begin

		process(EC_I_clk, EC_I_reset, EC_I_DefaultOffset)
		begin		
			if (EC_I_reset = '1') then
				TA_state_reg <= State_Initialise;
				EC_PulseCount <= "00000000";
				EC_Offset <= unsigned(EC_I_DefaultOffset);
				EC_OverShoots <= "00000000";
				EC_UnderShoots <= "00000000";
				EC_I_DefaultOffsetLast <= to_unsigned(0,9);
				--EC_OverShoots <= "00000000";
			elsif (EC_I_clk'event and EC_I_clk = '1') then
				TA_state_reg <= TA_state_next;
				EC_PulseCount <= EC_PulseCountNext;
				EC_Offset <= EC_OffsetNext;
				EC_OverShoots <= EC_OverShootsNext;
				EC_UnderShoots <= EC_UnderShootsNext;
				EC_I_DefaultOffsetLast <= unsigned(EC_I_DefaultOffset);
			end if;
		end process;

		process(TA_state_reg, EC_PulseCount, EC_UnderShoots, EC_OverShoots, EC_Offset, EC_I_undershoot,
				  EC_I_overshoot, EC_I_count, EC_UnderShootsInc, EC_pulseCountInc, EC_OverShootsInc,
				  EC_CountMaxAdj, EC_I_Threshold, EC_I_resetCount, EC_I_trigger, EC_I_DefaultOffset, 
				  EC_I_DefaultOffsetLast)
	
		begin
				--General Signals
				TA_state_next <= TA_state_reg;
				EC_PulseCountNext <= EC_pulseCount;
				EC_UnderShootsNext <= EC_UnderShoots;
				EC_OverShootsNext <= EC_OverShoots;
				EC_OffsetNext <= (EC_Offset - EC_I_DefaultOffsetLast)+ unsigned(EC_I_DefaultOffset);
				
				case TA_state_reg is
	
				-- System Initialisation on Power Up
				when State_Initialise =>
					EC_OffsetNext <= unsigned(EC_I_DefaultOffset);
					Ta_State_next <= State_State1;
					
				when State_State1 =>
				
					TA_state_next <= State_State1;
				
					If (EC_I_Trigger = '1' and EC_I_ResetCount = '0') then
						If (EC_I_undershoot = '1' and EC_I_overshoot = '0') then
							EC_UnderShootsNext <= EC_UnderShootsInc;
						End If;
						
						If (EC_I_undershoot = '0' and EC_I_overshoot = '1') then
							EC_OverShootsNext <= EC_OverShootsInc;
						End if;
						
						If (EC_I_Count = '1') then
							EC_PulseCountNext <= EC_PulseCountInc;
						End If;
						
					End if;
								
					If (EC_PulseCount >= EC_CountMaxAdj and EC_I_ResetCount = '0') then
							TA_State_next <= State_State2;
					End If;
					
					If (EC_I_ResetCount = '1') then
						EC_OverShootsNext <= (others => '0');
						EC_UnderShootsNext <= (others => '0');
						EC_PulseCountNext <= (others => '0');	
						--EC_OffsetNext <= (others => '0');
					End if;
						
				
				when State_State2 =>
				
					Ta_State_next <= State_State1;
					
					If ((EC_Overshoots = "00000000") and (EC_Undershoots > unsigned(EC_I_Threshold))) Then
						EC_OffsetNext <= EC_Offset + 1;
					End If;
					
					If((EC_Undershoots = "00000000") and (EC_Overshoots > unsigned(EC_I_Threshold))) Then 
						EC_OffsetNext <= EC_Offset - 1;
					End if;
					
					EC_OverShootsNext <= (others => '0');
					EC_UnderShootsNext <= (others => '0');
					EC_PulseCountNext <= (others => '0');
					
					--If (EC_I_ResetCount = '1') then
					--	EC_OffsetNext <= (others => '0');
					--End if;
							
				End case;

		End Process;
		
		Process(EC_OverShoots,EC_UnderShoots,EC_PulseCount)
		
		Begin
			EC_PulseCountInc <= EC_PulseCount + 1;
			
			
			--EC_OverShootsInc <= EC_OverShoots + 1;
			--EC_UnderShootsInc <= EC_UnderShoots + 1;
			
			If (EC_OverShoots + 1 = "00000000") Then
				EC_OverShootsInc <= (Others => '1');
			Else
				EC_OverShootsInc <= EC_OverShoots + 1;
			End If;
			
			If (EC_UnderShoots + 1 = "00000000") Then
				EC_UnderShootsInc <= (Others => '1');
			Else
				EC_UnderShootsInc <= EC_UnderShoots + 1;
			End If;
			
		End Process;
		
		--IF EC_I_CountMax = 0 then make EC_CountMaxAdj =1
		Process(EC_I_CountMax)
		
		Begin
		
			If (unsigned(EC_I_CountMax) = 0) Then
				EC_CountMaxAdj <= "00000001";
			Else
				EC_CountMaxAdj <= unsigned(EC_I_CountMax);
			End if;
			
		End Process;
		
		-- Output is offset from input by error correcting amount
		-- If SP1 is lower than minimum then add offset to minimum
		Process(EC_I_SP1MINIMUM, EC_Offset, EC_I_SP1)
		
		Begin
		
			If (EC_I_SP1MINIMUM > EC_I_SP1) Then
				EC_O_SP2 <= std_logic_vector(unsigned(EC_I_SP1MINIMUM) + EC_Offset);
			Else
				EC_O_SP2 <= std_logic_vector(unsigned(EC_I_SP1) + EC_Offset);
			End if;
			
		End Process;
end arch;

