----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:55:47 02/28/2011 
-- Design Name: 
-- Module Name:    event_protector - arch 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity event_protector is
	 generic(N: integer:= 8);
    port ( 
			reset 			: in std_logic;
         clk 				: in std_logic;
			
			start				: in std_logic;
			stopreset		: in std_logic;
			time_max_in		: in std_logic_vector(N-1 downto 0);
			overflow			: out std_logic
			);
			
end event_protector;

architecture arch of event_protector is

	type state_type is 	(
								State_Wait, State_Count, State_Final
								);
							
	signal state_now, state_next: state_type;
	
	signal counter_now:	unsigned (N-1 downto 0);
	signal counter_next: unsigned (N-1 downto 0);

begin

	-- Clocking Process
	process(clk, reset)
	begin
		if reset = '1' then
			state_now <= State_Wait;
			counter_now <= (others => '0');
		elsif (clk'event and clk = '1') then
			state_now <= state_next;
			counter_now <= counter_next;
		end if;
	end process;
	
	
	process(state_now, counter_now, start, stopreset, time_max_in)
	begin
	
		counter_next <= counter_now;
		state_next <= state_now;
		overflow <= '0';
		
		case state_now is
	
			when State_Wait =>
			
				if (start ='1' and stopreset = '0') then
					state_next <= State_Count;
					counter_next <= (others => '0');
				end if;
				
			when State_Count =>
			
				if (stopreset = '0') then
					counter_next <= counter_now + 1;
					if (counter_now > unsigned(time_max_in)) then
						state_next <= State_Final;
					end if;
				else
					state_next <= State_Wait;
				end if;
						
			when State_Final =>

				overflow <= '1';
				if (stopreset = '1') then
					state_next <= State_Wait;
				end if;
				
		end case;
		
	end process;

end arch;

