----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:28:33 03/05/2010 
-- Design Name: 
-- Module Name:    Fatal_Error_Priority - arch
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Fatal_Error_Priority is
    Port ( 
			clk 					: in  STD_LOGIC;
			reset 				: in  STD_LOGIC;
			
			FEP_I_WDT_Error	: in std_logic;
			FEP_I_DD_Error		: in std_logic;
			FEP_I_Index_Error	: in std_logic;
			FEP_I_KE_Error		: in std_logic;
			FEP_I_USBnc_Error	: in std_logic;
			FEP_I_IL5V_Error	: in std_logic;
			FEP_I_IL24V_Error	: in std_logic;
			-- Note that the following Error is reset 
			-- by 'setting' Actuate Disable 
			FEP_I_ActEn_Error	: in std_logic;
			FEP_I_PC_Error		: in std_logic;
			FEP_I_Cap_Error	: in std_logic;
			FEP_I_BPS_Error	: in std_logic;
			
			FEP_I_QuadADC_Error			: in std_logic;		
			FEP_I_QuadContinuity_Error	: in std_logic;
			
			FEP_I_FrameNOTRaisedError	: in std_logic;
					
			FEP_I_USBReset		: in std_logic_vector(12 downto 0);
			
			FEP_I_MasterReset : in std_logic;
			
			FEP_O_ErrorCode	: out std_logic_vector(3 downto 0);
			FEP_O_FatalError	: out std_logic;
			
			FEP_O_WDTReset		: out std_logic;
			FEP_O_DDReset		: out std_logic;
			FEP_O_IEReset		: out std_logic;
			FEP_O_KEReset		: out std_logic;
			FEP_O_USBncReset	: out std_logic;
			FEP_O_IL5VReset	: out std_logic;
			FEP_O_IL24VReset	: out std_logic;
			FEP_O_PCReset		: out std_logic;
			FEP_O_CEReset		: out std_logic;
			FEP_O_BPSReset		: out std_logic;
			FEP_O_QuadADCReset			: out std_logic;
			FEP_O_QuadContinuityReset	: out std_logic;
			
			FEP_O_FrameNOTRaisedReset	: out std_logic
			);
			
end Fatal_Error_Priority;

architecture arch of Fatal_Error_Priority is

	signal SA_WDTReset 	: std_logic;
	signal SA_DDReset		: std_logic;
	signal SA_IEReset		: std_logic;
	signal SA_KEReset		: std_logic;
	signal SA_USBncReset : std_logic;
	signal SA_IL5VReset  : std_logic;
	signal SA_IL24VReset : std_logic;
	signal SA_PCReset		: std_logic;
	signal SA_CapReset	: std_logic;
	signal SA_BPSReset	: std_logic;
	
	signal SA_QuadADCReset			: std_logic;
	signal SA_QuadContinuityReset	: std_logic;
	
	signal SA_FrameNOTRaisedReset	: std_logic;

begin

	WDT_USB_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> FEP_I_USBReset(0),
			O_edge_out	=> SA_WDTReset
		);
		
	DD_USB_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> FEP_I_USBReset(1),
			O_edge_out	=> SA_DDReset
		);
		
	IE_USB_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> FEP_I_USBReset(2),
			O_edge_out	=> SA_IEReset
		);
		
	KE_USB_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> FEP_I_USBReset(3),
			O_edge_out	=> SA_KEReset
		);
		
	USBnc_USB_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> FEP_I_USBReset(4),
			O_edge_out	=> SA_USBncReset
		);
		
		IL5V_USB_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> FEP_I_USBReset(5),
			O_edge_out	=> SA_IL5VReset
		);
		
		IL24V_USB_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> FEP_I_USBReset(6),
			O_edge_out	=> SA_IL24VReset
		);
		
		BPS_PC_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			-- NOTE THAT THIS IS OUT OF SEQUENCE -- PC is bit 8
			I_data_in	=> FEP_I_USBReset(8),
			O_edge_out	=> SA_PCReset
		);
		
		Cap_Error_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			
			I_data_in	=> FEP_I_USBReset(9),
			O_edge_out	=> SA_CapReset
		);
		
		
		BPS_USB_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			-- NOTE THAT THIS IS OUT OF SEQUENCE -- BPS is bit 7
			I_data_in	=> FEP_I_USBReset(7),
			O_edge_out	=> SA_BPSReset
		);
		
		QuadADC_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			
			I_data_in	=> FEP_I_USBReset(10),
			O_edge_out	=> SA_QuadADCReset
		);
			
		QuadContinuity_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			
			I_data_in	=> FEP_I_USBReset(11),
			O_edge_out	=> SA_QuadContinuityReset
		);
		
		FrameNOTRaised_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			
			I_data_in	=> FEP_I_USBReset(12),
			O_edge_out	=> SA_FrameNOTRaisedReset
		);
	
	
	--Assignments
	FEP_O_WDTReset 	<= (SA_WDTReset or FEP_I_MasterReset);
	FEP_O_DDReset 		<= (SA_DDReset or FEP_I_MasterReset);
	FEP_O_IEReset 		<= (SA_IEReset or FEP_I_MasterReset);
	FEP_O_KEReset 		<= (SA_KEReset or FEP_I_MasterReset);
	FEP_O_USBncReset 	<= (SA_USBncReset or FEP_I_MasterReset);
	FEP_O_IL5VReset 	<= (SA_IL5VReset or FEP_I_MasterReset);
	FEP_O_IL24VReset 	<= (SA_IL24VReset or FEP_I_MasterReset);
	FEP_O_PCReset 		<= (SA_PCReset or FEP_I_MasterReset);
	FEP_O_CEReset	 	<= (SA_CapReset or FEP_I_MasterReset);
	FEP_O_BPSReset 	<= (SA_BPSReset or FEP_I_MasterReset);
	
	FEP_O_QuadADCReset 	<= (SA_QuadADCReset or FEP_I_MasterReset);
	FEP_O_QuadContinuityReset 	<= (SA_QuadContinuityReset or FEP_I_MasterReset);	
	FEP_O_FrameNOTRaisedReset	<= (SA_FrameNOTRaisedReset or FEP_I_MasterReset);

	process(FEP_I_WDT_Error, FEP_I_DD_Error, FEP_I_Index_Error,
			   FEP_I_KE_Error, FEP_I_USBnc_Error, FEP_I_IL5V_Error,
				FEP_I_IL24V_Error, FEP_I_ActEn_Error, FEP_I_PC_Error,
				FEP_I_Cap_Error, FEP_I_BPS_Error, FEP_I_QuadADC_Error,
				FEP_I_QuadContinuity_Error, FEP_I_FrameNOTRaisedError)
	begin
	
		--  ERRORS
	
		-- 1  - WDT Error
		-- 2  - DD Error
		-- 3  - Index Error
		-- 4  - Key Enable Error
		-- 5  - USB nc Error
		-- 6  - 5V Int Error
		-- 7  - 24V Int Error
		-- 8  - AE Error
		-- 9  - PC Timer Error
		-- 10 - Capture Error
		-- 11 - BPS Error
		
		-- Added 05/01/12
		-- 12	- Quad ADC Error
		-- 13 - Quad Continuity Error
		
		-- Added 22/05/12
		-- 14 - Frame NOT raised error
	
		-- Also BPS error
		if (FEP_I_WDT_Error = '1') then
			FEP_O_ErrorCode <= "0001"; -- 1
			FEP_O_FatalError <= '1';
		
		-- Also BPS error		
		elsif (FEP_I_DD_Error = '1') then
			FEP_O_ErrorCode <= "0010"; -- 2
			FEP_O_FatalError <= '1';
		
		-- Also BPS error	
		elsif (FEP_I_QuadContinuity_Error = '1') then
			FEP_O_ErrorCode <= "1101"; -- 13
			FEP_O_FatalError <= '1';
		
		-- Also BPS error
		elsif (FEP_I_FrameNOTRaisedError = '1') then
			FEP_O_ErrorCode <= "1110"; -- 14
			FEP_O_FatalError <= '1';
		
		elsif (FEP_I_Index_Error = '1') then
			FEP_O_ErrorCode <= "0011"; -- 3
			FEP_O_FatalError <= '1';
			
		elsif (FEP_I_QuadADC_Error = '1') then
			FEP_O_ErrorCode <= "1100"; -- 12
			FEP_O_FatalError <= '1';

		elsif (FEP_I_KE_Error = '1') then
			FEP_O_ErrorCode <= "0100"; -- 4
			FEP_O_FatalError <= '1';
			
		elsif (FEP_I_USBnc_Error = '1') then
			FEP_O_ErrorCode <= "0101"; -- 5
			FEP_O_FatalError <= '1';
			
		elsif (FEP_I_IL5V_Error = '1') then
			FEP_O_ErrorCode <= "0110"; -- 6
			FEP_O_FatalError <= '1';
			
		elsif (FEP_I_IL24V_Error = '1') then
			FEP_O_ErrorCode <= "0111"; -- 7
			FEP_O_FatalError <= '1';
		
		-- 09/06/2011
		-- Moved AE Error to least priority as it was 
		-- causing a conflict
		-- with BPS/PC/Capture Errors 
		-- These should have higher 
		-- priority than AE error.
		
		-- Note that this Error is reset by setting
		-- Actuate Disable
		--elsif (FEP_I_ActEn_Error = '1') then
		--	FEP_O_ErrorCode <= "1000";
		--	FEP_O_FatalError <= '1';
			
		elsif (FEP_I_PC_Error = '1') then
			FEP_O_ErrorCode <= "1001"; -- 9
			FEP_O_FatalError <= '1';

		elsif (FEP_I_CAP_Error = '1') then
			FEP_O_ErrorCode <= "1010"; -- 10
			FEP_O_FatalError <= '1';
		
		-- This flag relates to actuation BPS errors
		elsif (FEP_I_BPS_Error = '1') then
			FEP_O_ErrorCode <= "1011"; -- 11
			-- A BPS error on its own is not a fatal error.
			--FEP_O_FatalError <= '0';
			
			-- Yes it is now! 23/05/2012
			FEP_O_FatalError <= '1';
			
		-- Note that this Error is reset by setting
		-- Actuate Disable
		elsif (FEP_I_ActEn_Error = '1') then
			FEP_O_ErrorCode <= "1000"; -- 8
			FEP_O_FatalError <= '1';
				
		else	
			FEP_O_ErrorCode <= "0000";
			FEP_O_FatalError <= '0';
		end if;
		
	end process;
	
end arch;

