----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:25:29 10/23/2009 
-- Design Name: 
-- Module Name:    FixedWidthPulse - arch 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity FixedWidthPulse2 is
    Port 	( 
				clk 					: in  STD_LOGIC;
				reset			 		: in  STD_LOGIC;
				FWP_I_SignalIn		: in  std_logic;
				FWP_O_PulseOut		: out std_logic
				);
				
end FixedWidthPulse2;

architecture arch of FixedWidthPulse2 is
	
		--constant FWP_TimeValue 			: std_logic_vector(15 downto 0) := "0100111000100000"; -- 500us
		--constant FWP_TimeValue 			: std_logic_vector(15 downto 0) := "0000000000000100"; -- ?us
		--constant FWP_TimeValue 			: std_logic_vector(19 downto 0) := std_logic_vector(to_unsigned(400,20));
		constant FWP_TimeValue				: std_logic_vector(19 downto 0) := std_logic_vector(to_unsigned(100000,20));
		
		
		signal FWP_EdgeOut 				: std_logic;
		signal FWP_TIME_NO_CONNECTION : std_logic_vector(19 downto 0);
		signal FWP_TimerComplete		: std_logic;
		signal FWP_TimerStart			: std_logic;
		
		type TA_state_type is 	
								(
								State_Initialise,
								State_State1,
								State_State2
								);
								
		signal TA_state_reg, TA_state_next: TA_state_type;

begin


		--Instantiate an Edge Detector
		Edge_detector: entity work.edge_detector
		port map(
					I_rst			=> reset,
					I_clk			=> clk,
					--
					I_enable		=> '1',
					I_edge_type	=> '1',
					--
					I_data_in	=> FWP_I_SignalIn,
					O_edge_out	=> FWP_EdgeOut
					);

	
		--Instantiate a Generic Timer
		Pulse_Time: entity work.Generic_Timer(arch)
		generic map (N=>20)
		port map (
					clk => clk,
					reset => reset,
					start => FWP_TimerStart,
					en => '1',
					synch_clear => '0',
					time_max_in => FWP_TimeValue,
					time_current_out => FWP_TIME_NO_CONNECTION,
					tick => FWP_TimerComplete
					);

		process(clk, reset)			
			begin
			
			if (reset = '1') then
				TA_state_reg <= State_Initialise;
			elsif (clk'event and clk = '1') then
				TA_state_reg <= TA_state_next;
			end if;
				
		end process;
		
		process(TA_state_reg, FWP_EdgeOut,FWP_TimerComplete)
		
			begin
			--General Signals
			TA_state_next <= TA_state_reg;
			
			FWP_TimerStart <= '0';
			FWP_O_PulseOut <= '0';


			case TA_state_reg is

			when State_Initialise =>
			
				TA_State_next <= State_State1;
			
			when State_State1 =>
				
				TA_State_next <= State_State1;
				
				If (FWP_EdgeOut = '1') Then
					FWP_TimerStart <= '1';
					FWP_O_PulseOut <= '1';
					TA_State_next <= State_State2;
				End if;
			
			When State_State2 =>
			
				FWP_O_PulseOut <= '1';
				TA_State_next <= State_State2;
				
				If (FWP_TimerComplete = '1') Then
					TA_State_next <= State_State1;
				End If;
			
			end case;
			
		end process;

end arch;

