----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:29:33 05/22/2009 
-- Design Name: 
-- Module Name:    Generic_Timer - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
-- NOTE that there has to be at least 1 clock cycle
-- between a tick out and a new trigger
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Generic_Timer is
	generic(N: integer:= 8);

   Port 		(
				clk : 				in  STD_LOGIC;
				reset : 				in  STD_LOGIC;
				start:				in std_logic;
				en:					in std_logic;
				synch_clear:		in std_logic;
				time_max_in:		in std_logic_vector(N-1 downto 0);
				time_current_out:	out std_logic_vector(N-1 downto 0);
				tick:					out std_logic
				);
				
end Generic_Timer;

architecture arch of Generic_Timer is

	type state_type is 	(
								State_Wait, State_Count,
								State_Input_Check
								);
							
	signal state_current, state_next: state_type;
	signal time_max: unsigned (N-1 downto 0);
	signal time_current:	unsigned (N-1 downto 0);
	signal time_next: unsigned (N-1 downto 0);
	signal time_inc: unsigned (N-1 downto 0);
	
begin


	-- Clocking Process
	process(clk, reset)
	begin
		if reset = '1' then
			state_current <= State_Wait;
			time_current <= (others => '0');
		elsif (clk'event and clk = '1') then
			state_current <= state_next;
			time_current <= time_next;
		end if;
	end process;
	
	
	-- Matching I/O to internal signals
		time_max <= unsigned(time_max_in);
		time_current_out <= std_logic_vector(time_current);
	
	-- Next State Logic 
		time_inc <= time_current + 1;
	
	
	-- FSM Process
	process(en, state_current, start, time_current, time_next, time_max, time_inc, 
	synch_clear)
	begin
	
		state_next <= State_Current;
		time_next  <= time_current;
				
		tick <= '0';
	
		case state_current is
	
			when State_Wait =>
		
				if (start = '1' and synch_clear = '0') then
				--if (start = '1') then
					time_next <= (others => '0');
					state_next <= State_Count;
				else
					state_next <= State_Wait;
				end if;
			
			when State_Count =>
			
				if (synch_clear = '1') then
					state_next <= State_Input_Check;
					time_next <= (others => '0');				
				elsif (time_current >= time_max) then
					state_next <= State_Input_Check;
					tick <= '1';
				else
					if (en = '1') then
						time_next <= time_inc;
					end if;
					state_next <= State_Count;
				end if;
	
			when State_Input_Check =>
				if (start = '1' or synch_clear = '1') then
					tick <='1';
					state_next <= State_Input_Check;
				else
					state_next <= State_Wait;
				end if;
				
			end case;
		
	end process;
		
end arch;

