-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
-----------------------------------------------------------


-- Entity
-----------------------------------------------------------
entity glitch_filter is
	generic	(
				G_bit_width : integer := 8
				);
				
	port		(
				I_rst			: in	std_logic;
				I_clk			: in	std_logic;
				--
				I_data_in	: in	std_logic;
				O_data_out	: out	std_logic
				);
				
end entity glitch_filter;

-- Architecture
-----------------------------------------------------------
architecture v0 of glitch_filter is
	
	-- Values / Signals
	-----------------------------
	
	constant CA_max_shift_reg_value : std_logic_vector(G_bit_width - 1 downto 0) := (others => '1');
	constant CA_min_shift_reg_value : std_logic_vector(G_bit_width - 1 downto 0) := (others => '0');
	
	signal SA_filter_result : std_logic;
	--signal SA_shift_reg_copy : std_logic_vector(G_bit_width - 1 downto 0);
	signal VP_shift_reg : std_logic_vector(G_bit_width - 1 downto 0);
	signal VP_shift_reg_next : std_logic_vector(G_bit_width - 1 downto 0);
	
begin
	
	-- Processes
	-----------------------------

	filter: process(I_rst, I_clk, VP_shift_reg, I_data_in)
		
	begin

	VP_shift_reg_next <= VP_shift_reg(G_bit_width - 2 downto 0) & I_data_in;
		
		if(I_rst = '1') then
			
			VP_shift_reg <= (others => '0');
			VP_shift_reg_next <= (others => '0');
			SA_filter_result <= '0';
			
		elsif(I_clk'event and I_clk = '1') then
			
			VP_shift_reg <= VP_shift_reg_next;
		
			
			if((VP_shift_reg = CA_max_shift_reg_value) and (SA_filter_result = '0')) then
				SA_filter_result <= '1';
			elsif((VP_shift_reg = CA_min_shift_reg_value) and (SA_filter_result = '1')) then
				SA_filter_result <= '0';
			end if;
			
		end if;
		
		
		
		--SA_shift_reg_cop <= VP_shift_reg;
		
	end process filter;
	
	-- Concurrent signal
	-- assignment
	-----------------------------
	
	O_data_out <= SA_filter_result;
	
end architecture v0;
