----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:19:26 10/22/2009 
-- Design Name: 
-- Module Name:    KeyEnable - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity KeyEnable is
    Port 	( 
				clk 						: in  STD_LOGIC;
				reset 					: in  STD_LOGIC;
				
				KE_I_KeyEnable 		: in std_logic;
				KE_O_KeyEnable 		: out std_logic
				
				);
				
end KeyEnable;

architecture arch of KeyEnable is

			--constant KE_Xms			:std_logic_vector(20 downto 0) := std_logic_vector(to_unsigned(200000,21));
			constant KE_Xms			:std_logic_vector(20 downto 0) := std_logic_vector(to_unsigned(20,21));			
			--constant KE_Xms			:std_logic_vector(20 downto 0) := "100001001110001000000";
			--constant KE_Xms			:std_logic_vector(20 downto 0) := "000000000000000000100";

			type TA_state_type is 	
								(
								State_Initialise,
								State_1,
								State_2,
								State_3,
								State_4,
								State_5,
								State_6
								);
								
			signal TA_state_reg, TA_state_next: TA_state_type;
			
			signal KE_TimerStart						 : std_logic;
			signal KE_TimerComplete					 : std_logic;			
			signal KEY_ENABLE_TIME_NO_CONNECTION : std_logic_vector(20 downto 0);
			signal KE_dg 									: std_logic;

begin

			-- Instantiate the 1ms Timer
			BounceTime: entity work.Generic_Timer(arch)
			generic map (N=>21)
			port map (
						clk => clk,
						reset => reset,
						start => KE_TimerStart,
						en => '1',
						synch_clear => '0',
						time_max_in => KE_Xms,
						time_current_out => KEY_ENABLE_TIME_NO_CONNECTION,
						tick => KE_TimerComplete
						);
						
						
		U_enable_filter:entity work.glitch_filter
		generic map	(	
						G_bit_width	=> 8
						)
		port map(
					I_rst			=> reset,
					I_clk			=> clk,
					--
					I_data_in	=> KE_I_KeyEnable,
					O_data_out	=> KE_dg
					);

		process(clk, reset)
			
			begin
			
				if (reset = '1') then
					TA_state_reg <= State_Initialise;
				elsif (clk'event and clk = '1') then
					TA_state_reg <= TA_state_next;
				end if;
				
			end process;

			process(TA_state_reg, TA_state_next, KE_dg, KE_TimerComplete)
			
				begin
				--General Signals
				TA_state_next <= TA_state_reg;
				KE_TimerStart <= '0';
				KE_O_KeyEnable <= '0';
				
				case TA_state_reg is
				
				when State_Initialise =>

					TA_state_next <= State_1;		

				when State_1 =>

					If (KE_dg = '1') Then
						KE_TimerStart <= '1';
						TA_State_next <= State_2;
					Else
						TA_state_next <= State_1;
					End if;
			
				when State_2 => 
				
					If (KE_TimerComplete = '1') Then
						TA_state_next <= State_3;
					else
						--TA_state_next <= State_1;
						TA_state_next <= State_2;
					End if;
				
				when State_3 => 
				
					If (KE_dg = '1') Then
						TA_State_next <= State_4;
					Else
						TA_state_next <= State_1;
					End if;
					
				when State_4 => 
				
					KE_O_KeyEnable <= '1';
					If (KE_dg = '0') Then
						KE_TimerStart <= '1';
						TA_State_next <= State_5;
					Else
						TA_state_next <= State_4;
					End if;
					
				when State_5 => 
				
					KE_O_KeyEnable <= '1';
					If KE_TimerComplete = '1' Then
						TA_state_next <= State_6;
					else
						TA_state_next <= State_5;
					End if;
					
				when State_6 => 
				
					KE_O_KeyEnable <= '1';
					If (KE_dg = '0') Then
						TA_State_next <= State_1;
					Else
						TA_state_next <= State_4;
					End if;


			end case;

			end process;

end arch;

