----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:22:12 05/21/2009 
-- Design Name: 
-- Module Name:    Move_To_Hold_State_Machine - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Move_To_Hold_State_Machine is

    Port ( 
			MTH_I_clk 									: in STD_LOGIC;
			MTH_I_reset 								: in STD_LOGIC;
			MTH_I_hold_position_max					: in std_logic_vector(5 downto 0);
			MTH_I_stator_move_to_hold				: in std_logic;
			MTH_I_stator_move_to_park				: in std_logic;
			MTH_timer_value							: in std_logic_vector(27 downto 0);
				
			MTH_O_stator_move_to_hold_complete	: out std_logic;
			MTH_O_stator_move_to_park_complete	: out std_logic;
			MTH_O_coarse_pos							: out std_logic_vector (5 downto 0);
			MTH_O_time_out								: out std_logic_vector(27 downto 0)
			);
			  
end Move_To_Hold_State_Machine;

architecture arch of Move_To_Hold_State_Machine is

	--constant N: integer:=28;
	
	type TA_state_type is 	(
										State_Wait,
										State_Hold_1,
										State_Hold_2,
										State_Hold_3,
										State_Hold_4,
										State_Park_1,
										State_Park_2,
										State_Park_3,
										State_Park_4
										);
							
	signal TA_state_reg, TA_state_next: TA_state_type;
	signal SA_hp_max: unsigned (5 downto 0);
	signal SA_current_coarse_pos: unsigned (5 downto 0);
	signal SA_next_coarse_pos: unsigned (5 downto 0);
	signal SA_incremented_pos: unsigned (5 downto 0);
	signal SA_decremented_pos: unsigned (5 downto 0);
	signal SA_timer_start: std_logic;
	--signal timer_value: std_logic_vector(27 downto 0);
	--signal SA_time_current_out: std_logic_vector(27 downto 0);
	signal SA_timer_complete: std_logic;
	

begin

	-- Instantiate a 28 bit timer
	
	park_hold_timer: entity work.Generic_Timer(arch)
	generic map (N=>28)
	port map   (
					clk => MTH_I_clk,
					reset => MTH_I_reset,
					start => SA_timer_start,
					en => '1',
					synch_clear => '0',
					time_max_in => MTH_timer_value,
					time_current_out => MTH_O_time_out,
					tick => SA_timer_complete
					);
					
	-- Next State Logic & Data Path Functional Units/Routing
	SA_incremented_pos <= SA_current_coarse_pos + 1;
	SA_decremented_pos <= SA_current_coarse_pos - 1;	
	
	SA_hp_max <= unsigned(MTH_I_hold_position_max);
	
	MTH_O_coarse_pos <= std_logic_vector(SA_current_coarse_pos);
		
	-- Processes
	process(MTH_I_clk, MTH_I_reset)
	begin
		if MTH_I_reset = '1' then
			TA_state_reg <= State_Wait;
			SA_current_coarse_pos <= (others => '0');
		elsif (MTH_I_clk'event and MTH_I_clk = '1') then
			TA_state_reg <= TA_state_next;
			SA_current_coarse_pos <= SA_next_coarse_pos;
		end if;
	end process;
	
	
	process(TA_state_reg, MTH_I_stator_move_to_hold, MTH_I_stator_move_to_park, SA_current_coarse_pos,
				SA_hp_max,SA_incremented_pos, SA_decremented_pos, SA_timer_complete)
	begin
		--General Signals
		TA_state_next <= TA_state_reg	;
		MTH_O_stator_move_to_hold_complete <= '0';
		MTH_O_stator_move_to_park_complete <= '0';
		SA_next_coarse_pos <= SA_current_coarse_pos;
		SA_timer_start <= '0';
		
		case TA_state_reg is
			
			when State_Wait =>
			
				if (MTH_I_stator_move_to_hold = '1' and MTH_I_stator_move_to_park  = '0') then
					---This next line might be dangerous if multiple calls to this routine are made accidently???
					--- (But no reason why they should be!)
					SA_next_coarse_pos <= (others => '0');
					---				
					TA_state_next <= State_Hold_1;
				elsif (MTH_I_stator_move_to_hold = '0' and MTH_I_stator_move_to_park  = '1') then
					TA_state_next <= State_Park_1;
					
				else	
					TA_state_next <= State_Wait;
				end if;
				
			when State_Hold_1 =>
				if (SA_current_coarse_pos >= SA_hp_max) then
					TA_state_next <= State_Hold_4;
				else 
					SA_next_coarse_pos <= SA_incremented_pos;
					SA_timer_start <= '1';
					TA_state_next <= State_Hold_2;
				end if;
				
			when State_Hold_2 =>
				if (SA_timer_complete = '1') then
					TA_state_next <= State_Hold_3;
				else
					TA_state_next <= State_Hold_2;
				end if;
				
			-- ensure that timer_complete has fallen to zero before retriggering timer!!	
			when State_Hold_3 =>
				if (SA_timer_complete = '0') then
					TA_state_next <= State_Hold_1;
				else
					TA_state_next <= State_Hold_3;
				end if;
					
						
			when State_Hold_4 =>
				-- Check to ensure that inputs are at zero
				if ((MTH_I_stator_move_to_hold = '0') and (MTH_I_stator_move_to_park  = '0')) then
					MTH_O_stator_move_to_hold_complete <= '1';
					TA_state_next <= state_Wait;
				else
					TA_state_next <= State_Hold_4;
				end if;
			
			when State_Park_1 =>
				--if (SA_current_coarse_pos <= "00000") then
				if (SA_current_coarse_pos = "00000") then
					TA_state_next <= State_Park_4;
				else 
					SA_next_coarse_pos <= SA_decremented_pos;
					SA_timer_start <= '1';
					TA_state_next <= State_Park_2;
				end if;
				
			when State_Park_2 =>
				if (SA_timer_complete = '1') then
					TA_state_next <= State_Park_3;
				else
					TA_state_next <= State_Park_2;
				end if;
				
			-- ensure that timer_complete has fallen to zero before retriggering timer!!	
			when State_Park_3 =>
				if (SA_timer_complete = '0') then
					TA_state_next <= State_Park_1;
				else
					TA_state_next <= State_Park_3;
				end if;
				
			when State_Park_4 =>
				-- Check to ensure that inputs are at zero
				if ((MTH_I_stator_move_to_hold = '0') and (MTH_I_stator_move_to_park  = '0')) then
					MTH_O_stator_move_to_park_complete <= '1';
					TA_state_next <= state_Wait;
				else
					TA_state_next <= State_Park_4;
				end if;
				
			end case;
	end process;

end arch;

