----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:19:05 02/17/2011 
-- Design Name: 
-- Module Name:    pc_counter - arch 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity pc_counter is
    port (  
			clk 	: 	in  std_logic;
         reset : 	in  std_logic;
			
			pc_i_usbpc		: in std_logic_vector(31 downto 0);
			
			pc_i_1sec_tick	: in std_logic;
			
			pc_o_set_error	: out std_logic		
			);
			
end pc_counter;

architecture arch of pc_counter is

	signal u_pc_now			: unsigned(31 downto 0);
	signal u_pc_next			: unsigned(31 downto 0);
	
	signal slv_usb_pc			: std_logic_vector(31 downto 0);
	
	signal u_counter_now		: unsigned(6 downto 0);
	signal u_counter_next	: unsigned(6 downto 0);
	
	signal sl_timer_reset	: std_logic;
	
	
	--constant COUNTER_MAX		: unsigned(63 downto 0):= (to_unsigned(4000,64));
	
	-- 1 minute - done in binary as integers will not go up to 64 bit!
	--constant COUNTER_MAX		: unsigned(63 downto 0):= "0000000000000000000000000000000010001111000011010001100000000000";
	
	-- 1 minute now using 1 sec clock as timebase 31/10/2011
	constant COUNTER_MAX		: unsigned(6 downto 0):= (to_unsigned(60,7));
	
	begin
	-- Define the next value for the PC_counter
	slv_usb_pc <= pc_i_usbpc;
	u_pc_next <= unsigned(slv_usb_pc);

	process(clk, reset)
	begin
		if (reset = '1') then
			u_pc_now <= (others => '0');
			u_counter_now <= (others => '0');
		elsif(clk'event and clk = '1') then
			u_pc_now <= u_pc_next;
			if sl_timer_reset = '1' then
				u_counter_now <= (others => '0');
			else
				u_counter_now <= u_counter_next;
			end if;
		end if;
	end process;
	
	process(u_pc_next, u_pc_now)
	begin
		sl_timer_reset <= '0';
		if (u_pc_next /= u_pc_now) then
			-- reset the timer as new value must have been loaded.
			sl_timer_reset <= '1';
		end if;
	end process;
	
	process(u_counter_now, pc_i_1sec_tick)
	begin
	
		pc_o_set_error <= '0';
		u_counter_next <= u_counter_now;
		
		if (pc_i_1sec_tick = '1') then
			if (u_counter_now < COUNTER_MAX) then 
				u_counter_next <= u_counter_now + 1;			
			else
				u_counter_next <= u_counter_now;
				-- will give an error tick every 1s until counter is reset
				pc_o_set_error <= '1';
			end if;	
			
		end if;
	end process;
		
end arch;

