----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:45:59 07/14/2009 
-- Design Name: 
-- Module Name:    Quad_2_Act_ROM_1 - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity Quad_2_Act_ROM_1 is

	Generic	(
				ADDR_WIDTH: integer := 9;
				DATA_WIDTH: integer := 16
				);

		Port 	( 
				clk 				: in std_logic;
				addr				: in std_logic_vector(ADDR_WIDTH-1 downto 0);
				mos_rom_enable	: in std_logic;
				data				: out std_logic_vector(DATA_WIDTH-1 downto 0)
				);
				
end Quad_2_Act_ROM_1;

architecture arch of Quad_2_Act_ROM_1 is

	--CONSTANT ADDR_WIDTH: integer:=4;
	--CONSTANT DATA_WIDTH: integer:= 8;
	
   type rom_type is array (0 to 2**ADDR_WIDTH-1)
        of std_logic_vector(DATA_WIDTH-1 downto 0);
--   -- ROM definition

--		This table returns the MOSFET state that gives the maximum
--		force on the target for a given quadrature position and is used
--		during the Actuate Sequence
--		The returned byte represents both the maximum upward and downward force
--		and is broken down as follows:
--
--		Upper Nibble = MOSFET state to give the maximum UPWARD force
--		Lower Nibble = MOSFET state to five the maximum DOWNWARD force
--
--		The values below are entered in Hex so the mosfet states can be 
--		read straight from the digits.
--		Post Processing will be required to select the correct Nibble
--		(depending on whether the required Force needs to be up or down)
--		and then use an additional conversion to get the MOSFET outputs
--		from the MOSFET states
--
--		The Coil Switch Points are 90 degrees out of phase with those 
--		in hold mode wrt to the absolute quadrature position.


  constant ROM_DATA: rom_type:=(
  
		-- Note addresses are in DECIMAL
		-- Stator geometry falls nicely into a decimal system
		-- 20 Quadrature counts = 1 coil distance = 3mm
		
		-- First Byte is UP/Down Mosfet State
		-- Second Byte is the equivalent Hold Position State
	
		-- addr 0 - 7
      X"0000", X"0000", X"0000", X"0000",  -- addr 0 - 3
		X"0000", X"0000", X"0000", X"0000",  -- addr 4 - 7
			
		-- Coil Switch Point	
		-- addr 16 represents Hold Position 0 & Power Off Position 
		--	addr 16 represents Maximum Force Point
		-- Addr 26 represents - Hold Position 2
		
      X"4000", X"4000", X"4000", X"4000", X"4000", -- addr 08 - 12 
		X"4001", X"4001", X"4001", X"4001", X"4001", -- addr 13 - 17
		X"4001", X"4001", X"4001", X"4001", X"4001", -- addr 18 - 22
		X"4002", X"4002", X"4002", X"4002", X"4002", -- addr 23 - 27
		
		-- Coil Switch Point
		-- addr 36 represents Maximum Force Point
		-- addr 46 represent Hold Position 4

		X"6002", X"6002", X"6002", X"6002", X"6002", -- addr 28 - 32
		X"6003", X"6003", X"6003", X"6003", X"6003", -- addr 33 - 37
		X"6003", X"6003", X"6003", X"6003", X"6003", -- addr 38 - 42
		X"6004", X"6004", X"6004", X"6004", X"6004", -- addr 43 - 47
		
		-- Coil Switch Point
		-- addr 56 represents Maximum Force Point
		-- addr 66 represent Hold Position 6
		
		X"8204", X"8204", X"8204", X"8204", X"8204", -- addr 48 - 52
		X"8205", X"8205", X"8205", X"8205", X"8205", -- addr 53 - 57
		X"8205", X"8205", X"8205", X"8205", X"8205", -- addr 58 - 62
		X"8206", X"8206", X"8206", X"8206", X"8206", -- addr 63 - 67
		
		-- Coil Switch Point
		-- addr 76 represents Maximum Force Point
		-- addr 86 represent Hold Position 8
		
		X"A406", X"A406", X"A406", X"A406", X"A406", -- addr 68 - 72
		X"A407", X"A407", X"A407", X"A407", X"A407", -- addr 73 - 77
		X"A407", X"A407", X"A407", X"A407", X"A407", -- addr 78 - 82
		X"A408", X"A408", X"A408", X"A408", X"A408", -- addr 83 - 87
		
		-- Coil Switch Point
		-- addr 96 represents Maximum Force Point
		-- addr 106 represent Hold Position 10
		
		X"C608", X"C608", X"C608", X"C608", X"C608", -- addr 88 - 92
		X"C609", X"C609", X"C609", X"C609", X"C609", -- addr 93 - 97
		X"C609", X"C609", X"C609", X"C609", X"C609", -- addr 98 - 102
		X"C60A", X"C60A", X"C60A", X"C60A", X"C60A", -- addr 103 - 107
		
		-- Coil Switch Point
		-- addr 116 represents Maximum Force Point
		-- addr 126 represent Hold Position 12
		
		X"280A", X"280A", X"280A", X"280A", X"280A", -- addr 108 - 112
		X"280B", X"280B", X"280B", X"280B", X"280B", -- addr 113 - 117
		X"280B", X"280B", X"280B", X"280B", X"280B", -- addr 118 - 122
		X"280C", X"280C", X"280C", X"280C", X"280C", -- addr 123 - 127
		
		-- Coil Switch Point
		-- addr 136 represents Maximum Force Point
		-- addr 146 represent Hold Position 14
		
		X"4A0C", X"4A0C", X"4A0C", X"4A0C", X"4A0C", -- addr 128 - 132
		X"4A0D", X"4A0D", X"4A0D", X"4A0D", X"4A0D", -- addr 133 - 137
		X"4A0D", X"4A0D", X"4A0D", X"4A0D", X"4A0D", -- addr 138 - 142
		X"4A0E", X"4A0E", X"4A0E", X"4A0E", X"4A0E", -- addr 143 - 147
		
		-- Coil Switch Point
		-- addr 156 represents Maximum Force Point
		-- addr 166 represent Hold Position 16
		
		X"6C0E", X"6C0E", X"6C0E", X"6C0E", X"6C0E", -- addr 148 - 152
		X"6C0F", X"6C0F", X"6C0F", X"6C0F", X"6C0F", -- addr 153 - 157
		X"6C0F", X"6C0F", X"6C0F", X"6C0F", X"6C0F", -- addr 158 - 162
		X"6C10", X"6C10", X"6C10", X"6C10", X"6C10", -- addr 163 - 167
		
		-- Coil Switch Point
		-- addr 176 represents Maximum Force Point
		-- addr 186 represent Hold Position 18
		
		X"8210", X"8210", X"8210", X"8210", X"8210", -- addr 168 - 172
		X"8211", X"8211", X"8211", X"8211", X"8211", -- addr 173 - 177
		X"8211", X"8211", X"8211", X"8211", X"8211", -- addr 178 - 182
		X"8212", X"8212", X"8212", X"8212", X"8212", -- addr 183 - 187
		
		-- Coil Switch Point
		-- addr 196 represents Maximum Force Point
		-- addr 206 represent Hold Position 20
		
		X"A412", X"A412", X"A412", X"A412", X"A412", -- addr 188 - 192
		X"A413", X"A413", X"A413", X"A413", X"A413", -- addr 193 - 197
		X"A413", X"A413", X"A413", X"A413", X"A413", -- addr 198 - 202
		X"A414", X"A414", X"A414", X"A414", X"A414", -- addr 203 - 207
		
		-- Coil Switch Point
		-- addr 216 represents Maximum Force Point
		-- addr 226 represent Hold Position 22
		
		X"C614", X"C614", X"C614", X"C614", X"C614", -- addr 208 - 212
		X"C615", X"C615", X"C615", X"C615", X"C615", -- addr 213 - 217
		X"C615", X"C615", X"C615", X"C615", X"C615", -- addr 218 - 222
		X"C616", X"C616", X"C616", X"C616", X"C616", -- addr 223 - 227
		
		-- Coil Switch Point
		-- addr 236 represents Maximum Force Point
		-- addr 246 represent Hold Position 24
		
		X"2816", X"2816", X"2816", X"2816", X"2816", -- addr 228 - 232
		X"2817", X"2817", X"2817", X"2817", X"2817", -- addr 233 - 237
		X"2817", X"2817", X"2817", X"2817", X"2817", -- addr 238 - 242
		X"2818", X"2818", X"2818", X"2818", X"2818", -- addr 243 - 247
		
		-- Coil Switch Point
		-- addr 256 represents Maximum Force Point
		-- addr 266 represent Hold Position 26
		
		X"4A18", X"4A18", X"4A18", X"4A18", X"4A18", -- addr 248 - 252
		X"4A19", X"4A19", X"4A19", X"4A19", X"4A19", -- addr 253 - 257
		X"4A19", X"4A19", X"4A19", X"4A19", X"4A19", -- addr 258 - 262
		X"4A1A", X"4A1A", X"4A1A", X"4A1A", X"4A1A", -- addr 263 - 267
		
		-- Coil Switch Point
		-- addr 276 represents Maximum Force Point
		-- addr 286 represent Hold Position 28
		
		X"6C1A", X"6C1A", X"6C1A", X"6C1A", X"6C1A", -- addr 268 - 272
		X"6C1B", X"6C1B", X"6C1B", X"6C1B", X"6C1B", -- addr 273 - 277
		X"6C1B", X"6C1B", X"6C1B", X"6C1B", X"6C1B", -- addr 278 - 282
		X"6C1C", X"6C1C", X"6C1C", X"6C1C", X"6C1C", -- addr 283 - 287	

		-- Coil Switch Point
		-- addr 296 represents Maximum Force Point
		-- addr 306 represent Hold Position 30
		
		X"821C", X"821C", X"821C", X"821C", X"821C", -- addr 288 - 292
		X"821D", X"821D", X"821D", X"821D", X"821D", -- addr 293 - 297
		X"821D", X"821D", X"821D", X"821D", X"821D", -- addr 298 - 302
		X"821E", X"821E", X"821E", X"821E", X"821E", -- addr 303 - 307
		
		-- Coil Switch Point
		-- addr 316 represents Maximum Force Point
		-- addr 326 represent Hold Position 32
		
		X"A41E", X"A41E", X"A41E", X"A41E", X"A41E", -- addr 308 - 312
		X"A41F", X"A41F", X"A41F", X"A41F", X"A41F", -- addr 313 - 317
		X"A41F", X"A41F", X"A41F", X"A41F", X"A41F", -- addr 318 - 322
		X"A420", X"A420", X"A420", X"A420", X"A420", -- addr 323 - 327
		
		-- Coil Switch Point
		-- addr 336 represents Maximum Force Point
		-- addr 346 represent Hold Position 34
		
		X"C620", X"C620", X"C620", X"C620", X"C620", -- addr 328 - 332
		X"C621", X"C621", X"C621", X"C621", X"C621", -- addr 333 - 337
		X"C621", X"C621", X"C621", X"C621", X"C621", -- addr 338 - 342
		X"C622", X"C622", X"C622", X"C622", X"C622", -- addr 343 - 347
		
		-- Coil Switch Point
		-- addr 356 represents Maximum Force Point
		-- addr 366 represent Hold Position 36
		
		X"2822", X"2822", X"2822", X"2822", X"2822", -- addr 348 - 352
		X"2823", X"2823", X"2823", X"2823", X"2823", -- addr 353 - 357
		X"2823", X"2823", X"2823", X"2823", X"2823", -- addr 358 - 362
		X"2824", X"2824", X"2824", X"2824", X"2824", -- addr 363 - 367
		
		-- Coil Switch Point
		-- addr 376 represents Maximum Force Point
		-- addr 386 represent Hold Position 38
		
		X"4A24", X"4A24", X"4A24", X"4A24", X"4A24", -- addr 368 - 372
		X"4A25", X"4A25", X"4A25", X"4A25", X"4A25", -- addr 373 - 377
		X"4A25", X"4A25", X"4A25", X"4A25", X"4A25", -- addr 378 - 382
		X"4A26", X"4A26", X"4A26", X"4A26", X"4A26", -- addr 383 - 387
		
		-- Coil Switch Point
		-- addr 396 represents Maximum Force Point
		-- addr 406 is probably at limit of the quadrature counter
		-- We're in dangerous territory here!!
		
		X"0C26", X"0C26", X"0C26", X"0C26", X"0C26", -- addr 388 - 392
		X"0C27", X"0C27", X"0C27", X"0C27", X"0C27", -- addr 393 - 397
		X"0C27", X"0C27", X"0C27", X"0C27", X"0C27", -- addr 398 - 402
		X"0C28", X"0C28", X"0C28", X"0C28", X"0C28", -- addr 403 - 407
			
		-- The rest of the data is now null data
		
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 408 - 415
		
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 416 - 423
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 424 - 431
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 432 - 439
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 440 - 447
		
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 448 - 455
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 456 - 463
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 464 - 471
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 472 - 479
		
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 480 - 487
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 488 - 495
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000", -- addr 496 - 503
		X"0000", X"0000", X"0000", X"0000", X"0000" ,X"0000", X"0000", X"0000"  -- addr 504 - 511
		
   );
	
	
	
   signal addr_reg: std_logic_vector(ADDR_WIDTH-1 downto 0);
	signal data1	:std_logic_vector(DATA_WIDTH-1 downto 0);
		
begin
   -- addr register to infer block RAM
   process (clk)
   begin
      if (clk'event and clk = '1') then
        addr_reg <= addr;
      end if;
   end process;

		data1 <= ROM_DATA(to_integer(unsigned(addr_reg)));
		
		
		-- This process only passes the MOSFET Data if the ROM is enabled
		-- This provides a mechanism for turning the Hex Bridge Off
		-- Hold State Details are always passed to the output
		-- As there is no need to blank this info
		
		
		process(mos_rom_enable, data1)
		begin
			if (mos_rom_enable = '1') then
				data (15 downto 8) <= data1 (15 downto 8);
			else
				data (15 downto 8) <= (others => '0');
			end if;
			
			data (7 downto 0) <= data1 (7 downto 0);
		end process;
	
end arch;
