----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:49:21 02/24/2011 
-- Design Name: 
-- Module Name:    spi_index_monitor - arch
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity spi_index_monitor is
	generic	(
				SPI_DataWidth		: natural:= 16  -- number of bits 
				);
    port ( 
			clk 						: in  std_logic;
         reset 					: in  std_logic;
			
			i_rxmon_read			: in std_logic;
			i_rxmon_spi_din		: in std_logic;
			
			i_SPI_enable			: in std_logic;
			
			i_current_state		: in std_logic_vector(3 downto 0);
			i_index_DG				: in std_logic;
			
			o_rxmon_value			: out std_logic_vector(SPI_DataWidth - 1 downto 0);
			o_rxmon_busy			: out std_logic;
			o_rxmon_complete 		: out std_logic;
			
			o_rxmon_spi_sclk		: out std_logic;
			o_rxmon_spi_synch		: out std_logic;
			o_rxmon_error			: out std_logic
			);
			
end spi_index_monitor;

architecture arch of spi_index_monitor is

type TA_state_type is 	(
										State_Wait,
										State_Start,
										State_Wait_Until_Signal,
										State_Delay,
										State_Read1,
										State_Read2
									);
							
	signal TA_state_now, TA_state_next: TA_state_type;
	
	signal sl_busy					: std_logic;
	signal sl_start_SPI_read	: std_logic;
	
	signal slv_SPI_data			: std_logic_vector(SPI_DataWidth - 1 downto 0);
	signal slv_USB_data			: std_logic_vector(SPI_DataWidth - 1 downto 0);
	--signal u_counter_now			: unsigned (10 downto 0);
	--signal u_counter_next 		: unsigned (10 downto 0);
	
	signal sl_SPI_enable			:std_logic;
	
	
	-- 1024 samples will be taken from SPI. Each sample will be added to the
	-- following register. The top SPI_DataWidth bits will give an average value after
	-- 1024 samples.
	signal u_averager_now		: unsigned (SPI_DataWidth + 9 downto 0);
	signal u_averager_next		: unsigned (SPI_DataWidth + 9 downto 0);
	
	-- Added 06_06_2011
	signal u_maximum_now			: unsigned(SPI_DataWidth - 1 downto 0);
	signal u_maximum_next		: unsigned(SPI_DataWidth - 1 downto 0);

	
	signal sl_start_timer		: std_logic;
	signal sl_timer_complete	: std_logic;
	--signal sl_random				: std_logic;  -- Used with Pseudo Random Number Generator for self-test
	
	constant DELAYWIDTH			: integer := 9; 
	constant DELAYTIME			: std_logic_vector(DELAYWIDTH-1 downto 0) := std_logic_vector(to_unsigned(400,DELAYWIDTH));
	constant SAMPLES				: integer := 64;
	
	constant MINIMUM_ADC			: integer := 43000;
	
begin
	--Instantiate a SPI_rx5
	SPI_Interface: entity work.spi_rx_005(arch)
	generic map (g_SPI_DataWidth => SPI_DataWidth)
	port map(
				clk 						=> clk,
				reset 					=> reset,
				i_spi_rx_sclk_edge	=> '0',
				
				-- 0 = din Sampled on the Rising Edge of sclk
				--		(din Changes on the Falling Edge of sclk?)
				-- ***
				-- 1 = Data Sampled on the Falling Edge of sclk
				--    (din changes on the Rising Edge of sclk?)
				
				-- For the AD7685 din Changes on the Falling Edge of sclk
				
				i_spi_rx_start_read	=> sl_start_SPI_read,
				i_spi_rx_din			=> i_rxmon_spi_din,
				
				i_spi_rx_en				=> sl_SPI_enable,
				
				i_spi_rx_gate_sclk	=> '1',
				-- 1 = sclk stopped when synch is high
				-- 0 = sclk runs continously
				
				i_spi_rx_gate_value	=> '0',
				-- 1 = sclk defaults to '1' when gated
				-- 0 = sclk defaults to '0' when gated
				
				o_spi_rx_busy			=> sl_busy,
				o_spi_rx_data			=> slv_SPI_data,	
			
				o_spi_rx_sclk			=> o_rxmon_spi_sclk,
				o_spi_rx_synch			=> o_rxmon_spi_synch	
				);

	--Instantiate a Generic Timer
	Pulse_Time: entity work.Generic_Timer(arch)
	generic map (N=>DELAYWIDTH)
	port map (
				clk => clk,
				reset => reset,
				start => sl_start_timer,
				en => '1',
				synch_clear => '0',
				time_max_in => DELAYTIME,
				time_current_out => open,
				tick => sl_timer_complete
				);
				
				
	--Instantiate a Pseudo Random Number Generator
	--Random: entity work.PseudoRandom_001(arch)
	--port map (
	--			clk => sl_spiclk,
	--			reset => reset,
	--			PR_O_rand_out => sl_random
	--			);

	--Assignments
		
	--USB_O_value <= slv_USB_data;
	o_rxmon_value <= slv_USB_data;
	sl_SPI_enable <= i_SPI_enable;
		
	-- Processes
	process(clk, reset)
			
	begin
			
		if (reset = '1') then
			TA_state_now <= State_Wait;
			--u_counter_now <= (others => '0');
			u_averager_now <= (others => '0');
			u_maximum_now <= (others => '0');
		elsif (clk'event and clk = '1') then
			TA_state_now <= TA_state_next;
			--u_counter_now <= u_counter_next;	
			u_averager_now <= u_averager_next;
			u_maximum_now <= u_maximum_next;
		end if;
				
	end process;

	-- Now we need a state Machine to orchestrate the whole process
--	process(TA_state_now, TA_state_next, u_counter_now, u_averager_now, 
--		u_maximum_now, i_rxmon_read, sl_busy, sl_timer_complete, slv_SPI_data, 
--		i_current_state, i_index_DG)
--	
--	begin
--	
--		TA_state_next 		<= TA_state_now;
--		u_counter_next 	<= u_counter_now;
--		u_averager_next 	<= u_averager_now;
--		u_maximum_next		<= u_maximum_now;
--		
--		sl_start_SPI_read <= '0';
--		sl_start_timer <= '0';
--		o_rxmon_busy <= '1';
--		o_rxmon_complete <= '0';
--		slv_USB_data <= (others => '0');
--		o_rxmon_error <= '0';
--		
--		case TA_state_now is
--		
--			when State_Wait =>
--				
--				o_rxmon_busy <= '0';
--				
--				-- Averager Connection
--				--slv_USB_data <= std_logic_vector(u_averager_now(SPI_DataWidth + 9 downto 10));
--				-- Maximiser Connection
--				slv_USB_data <= std_logic_vector(u_maximum_now);
--				
--				if i_rxmon_read = '1' then
--					--Read the SPI buffer and ignore the data (it is old)
--					--This will start the aquisition/conversion process
--					sl_start_SPI_read <= '1';
--					TA_state_next <= State_Start;
--					slv_USB_data <= (others => '0');
--				end if;
--				
--			when State_Start =>
--								
--				TA_state_next <= State_Start;
--				--Has SPI data been obtained?
--				if sl_busy = '0' then
--					TA_state_next <= State_Wait_Until_Signal;
--					sl_start_timer <= '1';
--					u_counter_next <= to_unsigned(SAMPLES, 11);
--					u_averager_next <= (others => '0');
--					u_maximum_next <= (others => '0');
--				end if;
--				
--			when State_Wait_Until_Signal =>
--			
--				TA_state_next <= State_Wait_Until_Signal;
--				
--				-- Moving to Hold, Moving to Park and Actuate
--				if ((i_current_state = "0010") or (i_current_state = "0011") 
--					or (i_current_state = "0110")) then
--			
--					-- When channel signal is seen start aquisition
--					if (i_index_DG = '1') then
--						sl_start_SPI_read <= '1';
--						TA_state_next <= State_Read1;
--					end if;
--			
--				else
--					-- Terminate the Reading Process
--					-- as not in a valid state!
--					TA_state_next <= State_Read2;
--				end if;
--			
--			when State_Delay =>
--				
--				TA_state_next <= State_Delay;
--				if sl_timer_complete = '1' then
--					sl_start_SPI_read <= '1';
--					TA_state_next <= State_Read1;
--				end if;
--				
--			when State_Read1 =>
--
--				TA_state_next <= State_Read1;
--				if sl_busy = '0' then
--					u_counter_next <= u_counter_now - 1;
--					u_averager_next <= u_averager_now + unsigned(slv_SPI_data);
--					
--					if (unsigned (slv_SPI_data) > u_maximum_now) then
--						u_maximum_next <= unsigned (slv_SPI_data);
--					end if;
--					
--					if u_counter_now > 1 then 
--						TA_state_next <= State_Delay;
--						sl_start_timer <= '1';
--					else
--						TA_state_next <= State_Read2;
--					end if;
--				end if;
--				 
--			when State_Read2 =>
--			
--				TA_state_next <= State_Wait;
--				o_rxmon_busy <= '0';
--				o_rxmon_complete <= '1';
--				
--				-- Averager Connection
--				--slv_USB_data <= std_logic_vector(u_averager_now(SPI_DataWidth + 9 downto 10));
--				-- Maximiser Connection
--				slv_USB_data <= std_logic_vector(u_maximum_now);
--				
--				if (u_maximum_now < MINIMUM_ADC) then
--					o_rxmon_error <= '1';
--				end if;
--				
--		end case;
--		
--	end process;

-- Now we need a state Machine to orchestrate the whole process
	process(TA_state_now, TA_state_next, 
	--u_counter_now, 
		u_averager_now, 
		u_maximum_now, i_rxmon_read, sl_busy, sl_timer_complete, slv_SPI_data, 
		i_current_state, i_channel_DG, i_continuous_monitor, slv_live_now)
	
	begin
	
		TA_state_next 		<= TA_state_now;
		--u_counter_next 	<= u_counter_now;
		u_averager_next 	<= u_averager_now;
		u_maximum_next		<= u_maximum_now;
		slv_live_next		<= slv_live_now;
		
		sl_start_SPI_read <= '0';
		sl_start_timer <= '0';
		o_rxmon_busy <= '1';
		o_rxmon_complete <= '0';
		slv_USB_data <= (others => '0');
		o_rxmon_error <= '0';
		
		case TA_state_now is
		
			when State_Wait =>
				
				o_rxmon_busy <= '0';
				
				-- Averager Connection
				--slv_USB_data <= std_logic_vector(u_averager_now(SPI_DataWidth + 9 downto 10));
				-- Maximiser Connection
				slv_USB_data <= std_logic_vector(u_maximum_now);
				
				if (i_rxmon_read = '1' or i_continuous_monitor = '1') then
					--Read the SPI buffer and ignore the data (it is old)
					--This will start the aquisition/conversion process
					sl_start_SPI_read <= '1';
					TA_state_next <= State_Start;
					slv_USB_data <= (others => '0');
				end if;
				
			when State_Start =>
								
				TA_state_next <= State_Start;
				--Has SPI data been obtained?
				if (sl_busy = '0') then
				
					TA_state_next <= State_Delay;
					--u_counter_next  <= to_unsigned(SAMPLES, 11);
					u_averager_next <= (others => '0');
					u_maximum_next  <= (others => '0');
				
		--			if (i_continuous_monitor = '0') then
		--				TA_state_next <= State_Wait_Until_Signal;				
		--			else
		--				sl_start_timer <= '1';
		--				TA_state_next <= State_Delay;
		--			end if;
					
				end if;
				
			when State_Wait_Until_Signal =>
			
				TA_state_next <= State_Wait_Until_Signal;
				
				-- Moving to Hold, Moving to Park and Actuate
				if ((i_current_state = "0010") or (i_current_state = "0011") 
					or (i_current_state = "0110")) then
			
					-- When channel signal is seen start aquisition
					if (i_Channel_DG = '1') then
						sl_start_timer <= '1';
						TA_state_next <= State_Delay;
					end if;
			
				else
					-- Terminate the Reading Process
					-- as not in a valid state!
					TA_state_next <= State_Read2;
				end if;
			
			when State_Delay =>
				
				TA_state_next <= State_Delay;
				if (sl_timer_complete = '1') then
					sl_start_SPI_read <= '1';
					TA_state_next <= State_Read1;
				end if;
				
			when State_Read1 =>
			
				TA_state_next <= State_Read1;
				
				if (sl_busy = '0') then
					--if (i_continuous_monitor = '0') then
					--	u_counter_next <= u_counter_now - 1;					
					--end if;
						
					u_averager_next <= u_averager_now + unsigned(slv_SPI_data);
					slv_live_next <= slv_SPI_data;
					
					if (unsigned (slv_SPI_data) > u_maximum_now) then
						u_maximum_next <= unsigned (slv_SPI_data);
					end if;
					
					if ((i_current_state = "0010") or (i_current_state = "0011") 
					or (i_current_state = "0110")) then
					
						TA_state_next <= State_Delay;
						sl_start_timer <= '1';
					else
						TA_state_next <= State_Read2;
					end if;
					
					--if (u_counter_now > 1) then 
					--	TA_state_next <= State_Delay;
					--	sl_start_timer <= '1';
					--else
					--	TA_state_next <= State_Read2;
					--end if;
					
					
				end if;
				 
			when State_Read2 =>
			
				TA_state_next <= State_Wait;
				o_rxmon_busy <= '0';
				o_rxmon_complete <= '1';
							
				-- Averager Connection
				--slv_USB_data <= std_logic_vector(u_averager_now(SPI_DataWidth + 9 downto 10));
				-- Maximiser Connection
				slv_USB_data <= std_logic_vector(u_maximum_now);
				
				if (u_maximum_now < MINIMUM_ADC) then
					o_rxmon_error <= '1';
				end if;
										
		end case;
		
	end process;

end arch;

